/***************************************************************************
 *  Drile.hpp
 *  Part of Drile
 *  2010  Tardigrade Inc.
 *  http://tardigrade-inc.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef DRILE_H
#define DRILE_H

#include <fstream>
#include <sstream>
#include <iostream>
#include <vector>
#include <map>
#include <pthread.h>
#include <string>
#include <libxml/parser.h>
#include <libxml/tree.h>

#include "ui/MainWindow.hpp"
#include "ui/StartWindow.hpp"

#include "devices/piivert/PiivertDevice.hpp"
#include "devices/hand/HandDevice.hpp"
#include "devices/MappingID.hpp"

#include "audio/AudioManager.hpp"
#include "env/EnvManager.hpp"
#include "input/InputManager.hpp"


class Drile {    
    public:
        static Drile* getInstance();
        ~Drile();

        void init(int,char*[]);
        void update();
        void updateInput();

        void parse(std::string);
        void parseApplications(xmlNodePtr);
        void parseCameras(xmlNodePtr);
        void parseRooms(xmlNodePtr);
        void parseRoom(xmlNodePtr, const unsigned int&);
        void parseZone(Room*, const unsigned int&, xmlNodePtr);


        void addWorm(NODE_TYPE, Worm*&, HllElement*&, int room, 
                        const unsigned int& zone, int parentID=-1, 
                        xmlNodePtr xmlNode=NULL);
        void addPiivert(Musician*, Piivert*&, PiivertDevice*&);
        void addHands(Musician*, Piivert*&, HandDevice*&);
        Room* addRoom();

        Camera* addCamera();

        void addMapping(MappingID&); 
        void addMappings(std::vector<MappingID>&);


    private:
        Drile();

        MainWindow* m_mainWindow;

        int m_deviceIDCounter;
        std::map<int, Device*> m_devices;

        friend void* inputThreadFunc(void*);
        pthread_t m_inputThread;
        RingBuffer *m_rbToEnv;
        RingBuffer *m_rbToInput;

        std::vector<InputEvent> m_inputEvents;
        std::map<unsigned int, Mapping*> m_mappings[INPUT_TYPES_NB];
        std::map<std::string, Mapping*> m_oscMappings;
        std::vector<Event> m_eventsToAudioThread;
        std::vector<Event> m_eventsToAudio;
        std::vector<Event> m_eventsToEnvThread;
        std::vector<Event> m_eventsToEnv;
        //void processEnvEvents();
        //void processAudioEvents();
        void processInputEvents();

        enum DRILE_APPLICATION{DRILE_MAIN, DRILE_DISPLAY};
};

#endif

