/***************************************************************************
 *  AudioManager.h
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _AUDIOMANAGER_H
#define _AUDIOMANAGER_H

#include <iostream>
#include <jack/jack.h>
#include <jack/types.h>
#include <cstdlib>
#include <vector>
#include <deque>
#include <map>
#include <exception>
#include <xtract/libxtract.h>

#include "Effect.hpp"
#include "Scene.hpp"
#include "../Event.hpp"
#include "../HllEnums.hpp"
#include "../utils/RingBuffer.hpp"

#include "HllNodeLoop.hpp"
#include "HllNodeSync.hpp"
#include "HllLeafLayeredGrain.hpp"
#include "Compressor.hpp"
#include "Reverb.hpp"


class AudioManager
{
    public:
        static AudioManager* getInstance();
        ~AudioManager();
        void start();
        void stop();
        void reset();

        inline const int& getSampleRate(){return m_sampleRate;}
        inline const int& getBufferSize(){return m_bufferSize;}
        jack_client_t* getClient();
        friend int audioCallback(jack_nframes_t , void *);
        friend int sRate(jack_nframes_t , void *);
        friend void shutdown(void*);
        
        HllElement* createNode(NODE_TYPE type, int parent=-1, 
                                xmlNodePtr xmlNode=NULL);
        void moveNode(HllElement* movedNode, HllNode* fromParent, 
                        HllNode* toParent);

        void process(jack_default_audio_sample_t *out[]=NULL);
        void update(std::vector<Event> & inputEvents, 
                    std::vector<Event> & outputEvents);

        static const int FFT_SIZE=1024;
        static const unsigned int NB_BARK_BANDS=25;
        inline int* getBarkBands(){return m_barkBands;}

        inline bool isStarted(){return m_started;}

        void printTrees();

    private:
        AudioManager();
        AudioManager( const AudioManager & ); 
        AudioManager & operator =( const AudioManager & ); 

        bool m_started;
        bool m_reset;

        jack_client_t *m_jackClient;
        jack_port_t* m_inputPorts[2];

        int m_sampleRate;    
        int m_bufferSize;    
    
        std::vector<std::vector<double> > m_tempNodeBuffer;
        std::vector<std::vector<double> > m_tempSceneBuffer;
        Compressor* m_compressor;
        Reverb* m_reverb;

        RingBuffer* m_ringBufferFromProcess;
        std::vector<Event> m_outputEventsFromProcess;
        RingBuffer* m_ringBufferFromUpdate;

        int* m_barkBands;

        //scenes and worms
        std::vector<HllElement*> m_nodesTrees;
        std::map<int, HllElement*> m_nodesMap;
        int m_nodeIdCounter;

        //leaves only, for the jack process
        std::vector<HllLeaf*> m_leaves;

        //samples
        std::vector<Sample*> m_samples;

        timeval m_currentTime, m_previousTime, m_timeDiff;
        double m_timeDiffD;

};


#endif 
