/***************************************************************************
 *  Compressor.hpp
 *  Part of Drile
 *
 *  Uses mostly code from Steve Harris SC3 LADSPA plugin
 *
 *  2010- Florent Berthaut
 *  hitmuri.net 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Compressor.hpp"

using namespace std;

#define A_TBL 256

Compressor::Compressor(const int& sampleRate): m_sampleRate(sampleRate) {
    m_rms = rms_env_new();
    m_sum = 0.0f;
    m_amp = 0.0f;
    m_gain = 0.0f;
    m_gainT = 0.0f;
    m_env = 0.0f;
    m_count = 0;


    m_as = (double*)malloc(A_TBL * sizeof(double));
    m_as[0] = 1.0f;
    for (int i=1; i<A_TBL; i++) {
      m_as[i] = expf(-1.0f / (m_sampleRate * (double)i / (double)A_TBL));
    }
    db_init();


    m_attack=2;
    m_release=200;
    m_threshold=-13;
    m_ratio=5;
    m_knee=1;
    m_makeupGain=0;
    m_chainBal=1;
    
}

Compressor::~Compressor() {
    rms_env_free(m_rms);
    free(m_as);
}

void Compressor::process(vector<double> & leftBuffer, 
                        vector<double> & rightBuffer,
                        vector<double> & sidechain ) {


    unsigned long pos;

    const double ga = m_as[f_round(m_attack * 0.001f * (double)(A_TBL-1))];
    const double gr = m_as[f_round(m_release * 0.001f * (double)(A_TBL-1))];
    const double rs = (m_ratio - 1.0f) / m_ratio;
    const double mug = db2lin(m_makeupGain);
    const double knee_min = db2lin(m_threshold - m_knee);
    const double knee_max = db2lin(m_threshold + m_knee);
    const double chain_bali = 1.0f - m_chainBal;
    const double ef_a = ga * 0.25f;
    const double ef_ai = 1.0f - ef_a;

    for (pos = 0; pos < leftBuffer.size(); pos++) {
      const double lev_in = chain_bali * 
                                (leftBuffer[pos] + rightBuffer[pos]) * 0.5f
                           + m_chainBal * sidechain[pos];
      m_sum += lev_in * lev_in;

      if (m_amp > m_env) {
        m_env = m_env * ga + m_amp * (1.0f - ga);
      } else {
        m_env = m_env * gr + m_amp * (1.0f - gr);
      }
      if (m_count++ % 4 == 3) {
        m_amp = rms_env_process(m_rms, m_sum * 0.25f);
        m_sum = 0.0f;
        if (isnan(m_env)) {
          // This can happen sometimes, but I dont know why
          m_env = 0.0f;
        } else if (m_env <= knee_min) {
          m_gainT = 1.0f;
        } else if (m_env < knee_max) {
          const double x = -(m_threshold - m_knee - lin2db(m_env)) / m_knee;
          m_gainT = db2lin(-m_knee * rs * x * x * 0.25f);
        } else {
          m_gainT = db2lin((m_threshold - lin2db(m_env)) * rs);
        }
      }
      m_gain = m_gain * ef_a + m_gainT * ef_ai;
      leftBuffer[pos]*= m_gain * mug;
      rightBuffer[pos]*= m_gain * mug;
    }

}


