/***************************************************************************
 *  Effect.hpp
 *  Part of Drile
 *  2010- Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef EFFECT_h
#define EFFECT_h

enum EFFECT_TYPE {EFFECT_AUDIO, EFFECT_CONTROL};
enum AUDIO_EFFECT {VOLUME_EFFECT, PAN_EFFECT, MUTE_EFFECT, FILTER_EFFECT, DISTORTION_EFFECT, REVERB_EFFECT, DIODE_EFFECT, AUDIO_EFFECTS_NB};
enum CONTROL_EFFECT {REPEAT_EFFECT, DELAY_EFFECT, PITCH_EFFECT, VELOCITY_EFFECT, CONTROL_EFFECTS_NB};
enum CONTROL_EFFECT_PARAM {CONTROL_EFFECT_VALUE, CONTROL_EFFECT_SPREAD, CONTROL_EFFECT_LENGTH, CONTROL_EFFECT_PARAMS_NB};

#include <sys/time.h>
#include <vector>

//#include "util/gverb.h"

class Effect
{    
    public:
        Effect(EFFECT_TYPE type=EFFECT_CONTROL, int fx=REPEAT_EFFECT);
        ~Effect();
        
        void processAudio(std::vector<std::vector<float> >&,int);
        void setParameters(std::vector<float>);

        //void processControl(std::vector<PatternEvent>&, timeval);
        void setParameters(std::vector<float>, long, int);
        inline long getDuration(){return m_duration;}

        inline std::vector<float> getParameters(){return m_parameters;}

        void setActive(bool);
        inline bool getActive(){return m_active;}

        inline timeval getStartTime(){return m_startTime;}
    private:

        EFFECT_TYPE m_type;
        int m_effect;
        std::vector<float> m_parameters;

        bool m_active;
        bool m_newActive;

        bool m_parametersChanged;
        std::vector<float> m_newParameters;

        //ty_gverb *m_gverb;

        void processFadedAudio(std::vector<std::vector<float> >&, int, float);


        long m_duration;
        timeval m_startTime;
        //std::vector<PatternEvent> m_eventsBuffer;
        bool m_initialized;
        
        timeval m_repeatStartTime;
        int m_repeatIndex;
        bool m_repeatInitialized;
        int m_repeatSizeMs;

        float m_variationPitch;
        float m_variationComponent;

        static const int BEAT_DIVISIONS_NB=7;
        float BEAT_DIVISIONS_ARRAY[BEAT_DIVISIONS_NB];

        //static const int DEFAULT_REPEAT_BEATS = 4;
};

#endif

