/***************************************************************************
 *  Filter.hpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Filter.hpp"

using namespace std;

Filter::Filter(double freq, double sampleRate):m_frequency(freq), 
                                                m_sampleRate(sampleRate), 
                                                m_stages(1) {
    m_gt = init_iir_stage(IIR_STAGE_HIGHPASS,10,3,2);
    m_iirf = init_iirf_t(m_gt);
    chebyshev(m_iirf, m_gt, 2*CLAMP((int)(m_stages),1,10), 
                IIR_STAGE_HIGHPASS, (m_frequency)/m_sampleRate, 0.5f);
}

Filter::~Filter() {
    free_iirf_t(m_iirf, m_gt);
    free_iir_stage(m_gt);
}

void Filter::process(vector<double> & buffer) {
    chebyshev(m_iirf, m_gt, 2*CLAMP((int)m_stages,1,10), 
                IIR_STAGE_HIGHPASS, m_frequency/m_sampleRate, 0.5f);
    iir_process_buffer_ns_5(m_iirf, m_gt, &buffer[0], 
                            &buffer[0], buffer.size());
}


