/***************************************************************************
 *  HllElement.hpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _HLL_ELEMENT_H
#define _HLL_ELEMENT_H

#include <iostream>
#include <cstring>
#include <cmath>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <vector>
#include <map>
#include <exception>

#include <libxml/parser.h>
#include <libxml/tree.h>

#include "../Event.hpp"
#include "../HllEnums.hpp"

class HllNode;

class HllElement {
    public:
        HllElement(const unsigned int&);
        virtual ~HllElement();
        virtual void parse(xmlNodePtr)=0;

        virtual void duplicate(std::vector<HllElement*>& createElements, 
                        int& elementsIdCounter)=0;
        
        inline const unsigned int& getID(){return m_id;}

        inline HllNode* getParent(){return m_parent;}
        inline void setParent(HllNode* parent){
            m_parent=parent;
            resetParentParameters();
        }

        virtual unsigned int getNbChildren()=0;
        virtual HllElement* getChild(unsigned int)=0;

        inline void setParentParameters(double parameters[]){
            for(unsigned int p=0; p<NODE_PARAMETERS_NB; ++p) {
                m_parentParametersNewValues[p]=parameters[p];
            }
            m_parametersChanged=true;
        }
        void resetParentParameters();
        void processParentNewValues();

        virtual void processEvent(const Event& event, 
                                    bool eventFromLoop=false);
        virtual void process(const double&, std::vector<std::vector<double> >&, 
                std::vector<std::vector<double> >&, std::vector<Event>&);
        
        void loopStartRecord(const timeval&);
        void loopStopRecord(const timeval&);
        void loopErase();
        inline virtual void loopPlay(double){};
        virtual void loopStop(const int& param=-1)=0;

        inline bool hasRecordedLoop(){return (m_loopState==LOOP_RECORDED);}
        inline std::vector<Event>& getRecordedExcitationLoopEvents(){
            return m_recordedExcitationLoopEvents;
        }
        inline std::vector<Event>& getRecordedModulationLoopEvents(
                                            unsigned int param){
            return m_recordedModulationLoopEvents[param];
        }
        inline void clearRecordedLoopEvents(){
            m_recordedExcitationLoopEvents.clear();
            m_hasRecordedExcitationEvents=false;
            for(unsigned int p=0; p < NODE_PARAMETERS_NB; ++p) { 
                m_recordedModulationLoopEvents[p].clear();
                m_hasRecordedModulationEvents[p]=false;
            }
        }
        inline timeval& getLoopRecordStopTime(){return m_loopRecordStopTime;}

        inline const double& getDynamicAnalysis(){return m_dynamicAnalysis;}

        bool isElementInSubtree(HllElement*);

        virtual void print(const std::string& prefix="");

    protected:

        unsigned int m_id;
        HllNode* m_parent;
        std::vector<HllElement*> m_children;

        double m_dynamicAnalysis;
        unsigned int m_dynamicAnalysisCounter;
        unsigned int m_dynamicAnalysisPeriod;
        std::vector<double> m_staticAnalysisValues;
        bool m_staticAnalysisSent;
        virtual void computeStaticAnalysis();

        bool m_newEventsWaiting;
        std::vector<Event> m_newEvents;
        std::vector<Event> m_events;

        bool m_parametersChanged;
        double m_parametersValues[NODE_PARAMETERS_NB];
        double m_parametersNewValues[NODE_PARAMETERS_NB];
        double m_dryParametersValues[NODE_PARAMETERS_NB];
        bool m_parametersChangedFromLoop;
        double m_parentParametersValues[NODE_PARAMETERS_NB];
        double m_parentParametersNewValues[NODE_PARAMETERS_NB];
            
        LOOP_STATE m_loopState;
        timeval m_loopRecordTime;
        timeval m_loopRecordStopTime;
        std::vector<Event> m_recordedExcitationLoopEvents;
        std::vector<Event> m_recordedModulationLoopEvents
                                                    [NODE_PARAMETERS_NB];
        bool m_hasRecordedExcitationEvents;
        bool m_hasRecordedModulationEvents[NODE_PARAMETERS_NB];

};



#endif
