/***************************************************************************
 *  HllLeafLayeredGrain.hpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _HLL_LEAF_LAYERED_GRAIN_NODE_H
#define _HLL_LEAF_LAYERED_GRAIN_NODE_H

#include <iostream>
#include <cstring>
#include <cmath>
#include <sys/time.h>
#include <dirent.h>
#include <unistd.h>
#include <stdlib.h>
#include <vector>
#include <list>
#include <map>
#include <stdexcept>

#include <sndfile.h>
#include <samplerate.h>

#include "util/gverb.h"
#include "util/iir.h"

#include "HllLeaf.hpp"
#include "Sample.hpp"
#include "Filter.hpp"

struct HllLeafLayeredGrainGrain {
    void copy(HllLeafLayeredGrainGrain* copiedGrain) {
        offset=copiedGrain->offset;
        approxOffset=copiedGrain->approxOffset;
        step=copiedGrain->step;
        counter=copiedGrain->counter;
        attackSamples=copiedGrain->attackSamples;
        releaseSamples=copiedGrain->releaseSamples;
        size=copiedGrain->size;
        volumes[0]=copiedGrain->volumes[0];
        volumes[1]=copiedGrain->volumes[1];
        env1=copiedGrain->env1;
        env2=copiedGrain->env2;
        envC=copiedGrain->envC;
        env=copiedGrain->env;
        type=copiedGrain->type;
        sample=copiedGrain->sample;
    }
    long offset;
    double approxOffset;
    double step;
    //count in grain
    int counter;
    int attackSamples, releaseSamples;
    //size of grain
    int size;
    //volume and envelopes
    double volumes[2];
    double env1, env2, env, envC;
    bool synthStopped;
    //type
    enum GRAIN_TYPE{GRAIN_NORMAL, GRAIN_BASE_SYNTH, GRAIN_BASE_SAMPLE};
    GRAIN_TYPE type;
    Sample* sample;
};

struct HllLeafLayeredGrainEvent {
    HllLeafLayeredGrainEvent():grainDensityCounter(0) {
        for(unsigned int i=0;i<10;++i) {
            grainReserve.push_back(new HllLeafLayeredGrainGrain());
        }
        firstGrain = new HllLeafLayeredGrainGrain();
    }
    int grainDensityCounter;
    std::list<HllLeafLayeredGrainGrain*> grains;
    std::vector<HllLeafLayeredGrainGrain*> grainReserve;
    HllLeafLayeredGrainGrain* firstGrain;
    double fade;
    bool fading;
    double fadeStep;
    int lastOffset;
    double velocity;
    double firstPitch;
    bool spatializeSide;
    int type;
    int id;
};

class EnvelopeManager {
	public:
		static EnvelopeManager* getInstance() { 
            static EnvelopeManager instance;
            return &instance;
        }
		~EnvelopeManager(){}

        void init(const unsigned int& minSize, const unsigned int& maxSize) {
            m_envelopes.clear();
            for(unsigned int s=minSize; s<=maxSize; ++s) {
                m_envelopes[s]=new jack_default_audio_sample_t[s];
                for(unsigned int i=0; i<s; ++i) { 
                    m_envelopes[s][i]=
                        0.5*(1.0 - cos ((2.0*M_PI*float(i))/float(s-1)));
                }
            }
            m_minSize=minSize;
            m_maxSize=maxSize;

        }

        jack_default_audio_sample_t* getEnvelope(const unsigned int& size) {
            return m_envelopes[size];
        }

        inline const unsigned int& getMinSize(){return m_minSize;}
        inline const unsigned int& getMaxSize(){return m_maxSize;}

	protected:
		EnvelopeManager(){}
        unsigned int m_minSize;
        unsigned int m_maxSize;
        std::map<unsigned int, jack_default_audio_sample_t*> m_envelopes;
};

class HllLeafLayeredGrain: public HllLeaf
{
    public:
        HllLeafLayeredGrain(const int&, const int&, const int&);
        ~HllLeafLayeredGrain();

        virtual void duplicate(std::vector<HllElement*>& createElements, 
                        int& elementsIdCounter);

        void parse(xmlNodePtr);

        void setType(const std::string&);
        void setBaseVolume(const float&);
        void setSamplesDir(const std::string&);

        void process(const double&, std::vector<std::vector<double> >&, 
                std::vector<std::vector<double> >&, std::vector<Event>&);

        void loopStop(const int& param=-1);

    private:

        std::string m_typeStr;
    
        void processFadedEffects(std::vector<std::vector<double> >&, double);

        Sample* m_baseSample;
        Sample* m_currentSample;
        std::string m_samplesDir;
        std::vector<Sample*> m_samples;

        Filter* m_filterL, *m_filterR;

        double m_filterFrequency;
        double m_baseVolume;

        double m_volume;
        int m_polyphony;
        bool m_baseSynth;

        double m_grainVolume;
        int m_grainDensity; //frames between grains
        int m_grainNumber; //current number
        int m_grainNumberMax; //maximum
        double m_grainDistance; //percentage of the sample used
        int m_grainSize; //frames
        double m_grainAttack; 
        double m_grainRelease;
        double m_pitch;
        double m_resonance; //in fade out vol per grain
        double m_phaseStep;
        double m_baseBassFrequency; //in Hz
        bool m_spatialized;

        double m_tempVal;

        void setNoisiness(const double&);
        void setResonance(const double&);
        void setPitch(const double&);
        void setVolume(const double&);
        void setIrregularity(const double&);

        timeval m_lastParameterFadeTime;

        //events
        std::list<HllLeafLayeredGrainEvent*> m_layeredGrainNodeEvents;
        std::vector<HllLeafLayeredGrainEvent*> m_layeredGrainNodeEventReserve;

        //analysis
        void computeStaticAnalysis();
};



#endif
