/***************************************************************************
 *  SampleNode.cpp
 *  Part of Drile
 *  2010- Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
#include "SampleNode.hpp"
#include "AudioManager.hpp"

using namespace std;

SampleNode::SampleNode(const int& id):Node(id), m_volume(1.0), m_polyphony(1)
{
    gettimeofday(&m_lastParameterFadeTime, NULL);

}

SampleNode::SampleNode(const int& id, const SampleNode& copiedSampleNode): Node(id,copiedSampleNode)
{

}

SampleNode::~SampleNode()
{

}

void SampleNode::parse(xmlNodePtr node)
{
    AudioManager* audioManager = AudioManager::getInstance();

    char* value=NULL;

    value=NULL;
    value = (char*)xmlGetProp(node,(xmlChar*)"volume");
    if(value!=NULL) {
        m_volume = atof(value);
    }

    //parse samples
    xmlNodePtr sampleNode;
    for (sampleNode = node->children; sampleNode; sampleNode = sampleNode->next) {
                if (sampleNode->type == XML_ELEMENT_NODE && !xmlStrcmp(sampleNode->name, (const xmlChar *) "Sample")) {
            value=NULL;
            value = (char*)xmlGetProp(sampleNode,(xmlChar*)"file");
            if(value!=NULL) {
                m_samples.push_back(audioManager->addSample(std::string(value)));
            }
        }
    }
}

void SampleNode::load()
{
    Node::load();    

    //load the effects
    m_filterUfc = (4.0 + 40000.0*0.5f)/m_sampleRate;
    m_filterLfc = (4.0 - 40000.0*0.5f)/m_sampleRate;
    m_filterFirstL = init_iir_stage(IIR_STAGE_LOWPASS,10,3,2);
    m_filterFirstR = init_iir_stage(IIR_STAGE_LOWPASS,10,3,2);
    m_filterSecondL = init_iir_stage(IIR_STAGE_HIGHPASS,10,3,2);                  
    m_filterSecondR = init_iir_stage(IIR_STAGE_HIGHPASS,10,3,2);                  
    m_filterGtL = init_iir_stage(IIR_STAGE_BANDPASS,20,3,2); 
    m_filterGtR = init_iir_stage(IIR_STAGE_BANDPASS,20,3,2); 
    m_filterIirfL = init_iirf_t(m_filterGtL);
    m_filterIirfR = init_iirf_t(m_filterGtR);
    chebyshev(m_filterIirfL, m_filterFirstL, 2, IIR_STAGE_LOWPASS, m_filterUfc, 0.5f);
    chebyshev(m_filterIirfR, m_filterFirstR, 2, IIR_STAGE_LOWPASS, m_filterUfc, 0.5f);
    chebyshev(m_filterIirfL, m_filterSecondR, 2, IIR_STAGE_HIGHPASS, m_filterLfc, 0.5f);
    chebyshev(m_filterIirfL, m_filterSecondR, 2, IIR_STAGE_HIGHPASS, m_filterLfc, 0.5f);
    combine_iir_stages(IIR_STAGE_BANDPASS, m_filterGtL, m_filterFirstL, m_filterSecondL, 0, 0);
    combine_iir_stages(IIR_STAGE_BANDPASS, m_filterGtR, m_filterFirstR, m_filterSecondR, 0, 0);

    //reverb
    m_gverb = gverb_new(m_sampleRate, 300.0f, 50.0f, 7.0f, 0.5f, 15.0f, 0.5f, 0.5f, 0.5f);

    m_staticAnalysed=false;
}


void SampleNode::unload()
{
    
}


void SampleNode::process(timeval &curTime, std::vector<std::vector<double> >& tempBuffer, std::vector<std::vector<double> >& outputBuffer)
{
    Node::process(curTime, tempBuffer, outputBuffer);

    //first create sample node events from new audio events
    if(m_newEventsWaiting) {
        m_events = m_newEvents;
        m_newEventsWaiting=false;
        for(unsigned int evID=0; evID<m_events.size(); evID++) { 
            m_sampleNodeEvents.push_back(SampleNodeEvent());
            SampleNodeEvent &snEvent=m_sampleNodeEvents[m_sampleNodeEvents.size()-1];
            //if it is a play event with a non null velocity
            if(m_events[evID].parameter==NODE_EXCITE_INST && m_events[evID].values[0]>0) {
                snEvent.type = m_events[evID].parameter;
                snEvent.sampleID = int(double(m_samples.size()) * m_events[evID].values[0]);
                if(snEvent.sampleID==int(m_samples.size())){
                    snEvent.sampleID=m_samples.size()-1;
                }
                snEvent.volume = m_events[evID].values[0]*m_volume;
                snEvent.offset = 0;
                snEvent.fading=false;
                snEvent.fade=1.0;
            }
            else { //if it is a stop event
                for(unsigned int snEvID = 0; snEvID<m_sampleNodeEvents.size(); ++snEvID) {
                    m_sampleNodeEvents[snEvID].fade=1.0;
                    m_sampleNodeEvents[snEvID].fading=0;
                }
            }
            

        }
        m_events.clear();
    }

    //reset the temp buffer
    for(unsigned int c=0; c<tempBuffer.size(); ++c) {
        tempBuffer[c].assign(tempBuffer[c].size(), 0.00001);
    }

    double fadeStep = 1.0 / double(tempBuffer.size()); 
    //go through each of the sample Node Events
    for(vector<SampleNodeEvent>::iterator itEvent=m_sampleNodeEvents.begin(); itEvent!=m_sampleNodeEvents.end();) {
        SampleNodeEvent &event = (*itEvent);
        Sample *currentSample = m_samples[event.sampleID];

        //get remaining length of the audioevent
        int length = int(tempBuffer[0].size());
        double leftLength = double(currentSample->getFramesCount() - event.offset)/(2.0*m_parametersValues[NODE_PITCH]);
        if(int(leftLength) < length) {
            length = leftLength;
        }
    
        //get the values
        for(int i=0;i<length;++i) {
            for(unsigned int c=0;c<tempBuffer.size();++c) {
                tempBuffer[c][i] += currentSample->getFrame(long(event.offset))*event.volume*event.fade;
            }
            event.offset+=(2.0*m_parametersValues[NODE_PITCH]);
            if(event.fading) {
                event.fade-= fadeStep;
            }
        }

        //if event is finished or completely faded out
        if(length<int(tempBuffer[0].size()) || event.fade<=0.0) {
            //remove the event
            m_sampleNodeEvents.erase(itEvent);
        }
        else {
            ++itEvent;
        }

    }

    //process effects
    if(m_parametersChanged) { //crossfade 
        timeval timeDiff;
        timersub(&curTime, &m_lastParameterFadeTime, &timeDiff);
        //only if enough time passed since the last crossfade
        if(timeDiff.tv_sec>0 || timeDiff.tv_usec>50000) {
            m_parametersChanged=false;
            std::vector<std::vector<double> >tempBuffer2=tempBuffer;
            //process the parameters before in fade out
            processFadedEffects(tempBuffer2, -m_fadeStep);
            //update parent parameters
            memcpy(m_parentParametersValues,m_parentParametersNewValues,sizeof(double)*NODE_PARAMETERS_NB);
            //update parameters
            for(int p=0; p<NODE_PARAMETERS_NB; ++p) {
                m_parametersValues[p]=m_parametersNewValues[p]*(2.0*m_parentParametersValues[p]);
            }
            if(m_parametersValues[NODE_PITCH]<=0.05) {
                m_parametersValues[NODE_PITCH]=0.05;
            }
            //process after in fade in
            processFadedEffects(tempBuffer, m_fadeStep);
            //combine the two
            for(unsigned int c=0;c<tempBuffer.size();++c) {
                for(unsigned int f=0;f<tempBuffer[c].size();++f) {
                    tempBuffer[c][f]+=tempBuffer2[c][f];
                }
            }
            m_lastParameterFadeTime=curTime;
        }
        else {
            processFadedEffects(tempBuffer, 0);
        }
    }
    else {
        processFadedEffects(tempBuffer, 0);
    }

    //analyse the loudness
    dynamicAnalyse(tempBuffer);

    //add the resulting buffer 
    for(unsigned int c=0;c<tempBuffer.size();++c) {
        for(unsigned int f=0;f<tempBuffer[c].size();++f) {
            outputBuffer[c][f]+=tempBuffer[c][f];
        }
    }
}

void SampleNode::processFadedEffects(std::vector<std::vector<double> >& buffer, double fadeStep)
{

    //apply reverb
    gverb_set_roomsize(m_gverb, m_parametersValues[NODE_RESONANCE]*70.0+30.0);
    gverb_set_revtime(m_gverb, m_parametersValues[NODE_RESONANCE]*7.0);
    double l1,l2,r1,r2;
    for(unsigned int c=0; c<buffer.size(); c+=2) {
        for(unsigned int i=0; i<buffer[c].size(); i++) {
            gverb_do(m_gverb, buffer[c][i], &l1, &r1);
            gverb_do(m_gverb, buffer[c+1][i], &l2, &r2);
            buffer[c][i] = ((l1+l2)/2.0 *m_parametersValues[NODE_RESONANCE]) + buffer[c][i]*(1.0-m_parametersValues[NODE_RESONANCE]);
            buffer[c+1][i] = ((r1+r2)/2.0 *m_parametersValues[NODE_RESONANCE]) + buffer[c+1][i]*(1.0-m_parametersValues[NODE_RESONANCE]);
        }
    }
    //apply diode processor
    unsigned int pos;
    if (m_parametersValues[NODE_NOISINESS]*2.0 > 0.1f && m_parametersValues[NODE_NOISINESS]*2.0 < 1.0f) {
        for(unsigned int c=0; c<buffer.size(); ++c) {
            for (pos = 0; pos < buffer[c].size(); pos++) {
                buffer[c][pos] = (((1.0f-m_parametersValues[NODE_NOISINESS]*2.0) * buffer[c][pos]) + (m_parametersValues[NODE_NOISINESS]*2.0 * (buffer[c][pos] > 0.0f ? buffer[c][pos] : 0.0f)));
            }
        }
    } else if (m_parametersValues[NODE_NOISINESS]*2.0 >= 1.0f && m_parametersValues[NODE_NOISINESS]*2.0 < 2.0f) {
        double fac = m_parametersValues[NODE_NOISINESS]*2.0 - 1.0f;
        for(unsigned int c=0; c<buffer.size(); ++c) {
            for (pos = 0; pos < buffer[c].size(); pos++) {
                buffer[c][pos] = (((1.0f-fac) * (buffer[c][pos] > 0 ? buffer[c][pos] : 0.0)) + (fac * fabs(buffer[c][pos])));
            }
        }
    } else if (m_parametersValues[NODE_NOISINESS]*2.0 >= 2) {
        double fac = m_parametersValues[NODE_NOISINESS]*2.0 < 3 ? m_parametersValues[NODE_NOISINESS]*2.0 - 2 : 1.0;
        for(unsigned int c=0; c<buffer.size(); ++c) {
            for (pos = 0; pos < buffer[c].size(); pos++) {
                buffer[c][pos] = (1.0-fac) * fabs(buffer[c][pos]);
            }
        }
    }
    //apply filter
    m_filterUfc = (m_parameters[0] + m_parameters[1]*0.5f)/m_sampleRate;
    m_filterLfc = (m_parameters[0] - m_parameters[1]*0.5f)/m_sampleRate;
    combine_iir_stages(IIR_STAGE_BANDPASS, m_filterGtL, m_filterFirstL, m_filterSecondL, 
        chebyshev(m_filterIirfL, m_filterFirstL,  2, IIR_STAGE_LOWPASS,  m_filterUfc, 0.5f),
        chebyshev(m_filterIirfL, m_filterSecondL, 2, IIR_STAGE_HIGHPASS, m_filterLfc, 0.5f));
    iir_process_buffer_ns_5(m_filterIirfL, m_filterGtL, &outputBuffer[0][0], &outputBuffer[0][0], framesPerBuffer);
    combine_iir_stages(IIR_STAGE_BANDPASS, m_filterGtR, m_filterFirstR, m_filterSecondR, 
        chebyshev(m_filterIirfR, m_filterFirstR,  2, IIR_STAGE_LOWPASS,  m_filterUfc, 0.5f),
        chebyshev(m_filterIirfR, m_filterSecondR, 2, IIR_STAGE_HIGHPASS, m_filterLfc, 0.5f));
    iir_process_buffer_ns_5(m_filterIirfR, m_filterGtR, &outputBuffer[1][0], &outputBuffer[1][0], framesPerBuffer);
    
    double fade=1.0;
    if(fadeStep>0) {
        fade=0;
    }
    for(unsigned int i=0;i<framesPerBuffer;++i, fade+=fadeStep) {
        outputBuffer[0][i]*=fade;
        outputBuffer[1][i]*=fade;
    }

    //apply volume and eventually fade
    double fade=1.0;
    if(fadeStep>0) {
        fade=0;
    }
    for(unsigned int i=0; i<buffer[0].size(); i++, fade+=fadeStep) {
        for(unsigned int c=0; c<buffer.size(); ++c) {
            buffer[c][i]*=m_parametersValues[NODE_VOLUME]*fade;    
        }
    }
    
}



void SampleNode::analyse(timeval& curTime, std::vector<AudioEvent>& events)
{
    //dynamic analysis
    Node::analyse(curTime, events);

    if(!m_staticAnalysed) {

        AudioManager *audioManager = AudioManager::getInstance();


        double argf[4];
        argf[0] = (double)m_sampleRate/(double)AudioManager::FFT_SIZE;
        argf[1] = XTRACT_MAGNITUDE_SPECTRUM;
        argf[2] = 0.f;
        argf[3] = 0.f;

        std::vector<double> tempBuffer(AudioManager::FFT_SIZE, 0);
        std::vector<double> spectrumBuffer(AudioManager::FFT_SIZE*2, 0);
        std::vector<double> tempBarkBuffer(AudioManager::NB_BARK_BANDS, 0);
        std::vector<double> barkBuffer(AudioManager::NB_BARK_BANDS, 0);

        //go through all the samples and add all the bark spectrums
        double spectrumBuffersCount=0;
        for(unsigned int s=0; s<m_samples.size(); ++s) {
            Sample* sample = m_samples[s];    
            if(sample->isLoaded()) {
                bool sampleAnalysed=false;
                long offset=0;
                while(!sampleAnalysed) {
                    //cout<<offset <<" "<<sample->getFramesCount()<<" "<<sample->getChannelsCount()<<endl;
                    int length = AudioManager::FFT_SIZE;
                    if(sample->getFramesCount() - offset < length) {
                        length = sample->getFramesCount() - offset;
                    }
                    for(int i=0;i<length;++i) {
                        tempBuffer[i]=0;
                        for(int c=0;c<sample->getChannelsCount();++c) {
                            tempBuffer[i] += sample->getFrame(offset)/double(sample->getChannelsCount());
                        }
                        offset++;
                    }
                    //if finished
                    if(length<=AudioManager::FFT_SIZE) {
                        sampleAnalysed=true;
                    }
                    //analyse the buffer
                    xtract_spectrum(&(tempBuffer[0]), AudioManager::FFT_SIZE, &argf[0], &(spectrumBuffer[0]));
                    xtract_bark_coefficients(&(spectrumBuffer[0]), AudioManager::FFT_SIZE, audioManager->getBarkBands(), &(tempBarkBuffer[0]));
                    for(unsigned int i=0; i<AudioManager::NB_BARK_BANDS; ++i) {
                        barkBuffer[i]+=tempBarkBuffer[i];
                    }
                    spectrumBuffersCount++;
                }
            }
        }

        for(unsigned int i=0;i<AudioManager::NB_BARK_BANDS;++i) {
            barkBuffer[i]/=spectrumBuffersCount;
        }

        //static analysis
        events.push_back(AudioEvent());
        AudioEvent& newEvent = events[events.size()-1];
        newEvent.type=AUDIO_NODE;
        newEvent.node=m_id;
        newEvent.parameter=NODE_ANALYSIS_STATIC;
        newEvent.values=barkBuffer;
        m_staticAnalysed=true;
    }
}

*/
