/***************************************************************************
 *  SasNode.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
#include "SasNode.hpp"
#include "AudioManager.hpp"

using namespace std;

static void
update_callback (sas_synthesizer_t s,
         sas_source_t source,
         sas_frame_t * frame,
         sas_position_t * pos,
         void * call_data)
{
  struct source_data_s * sd;
  sd = (struct source_data_s *) call_data;
  *frame = sd->frame;
  *pos = &sd->pos;
}

SasNode::SasNode(const int& id):Node(id), m_volume(1.0), m_polyphony(3), m_grainSize(64), m_resoStep(-0.0001), m_grainStep(1), m_grainDistance(64)
{
    gettimeofday(&m_lastParameterFadeTime, NULL);
    m_grainFadeStep = 1.0/(float(m_grainSize)/2.0);
    srand ( time(NULL) );    
}

SasNode::SasNode(const int& id, const SasNode& copiedSasNode): Node(id,copiedSasNode)
{

}

SasNode::~SasNode()
{

}

void SasNode::parse(xmlNodePtr node)
{

    char* value=NULL;

    value=NULL;
    value = (char*)xmlGetProp(node,(xmlChar*)"volume");
    if(value!=NULL) {
        m_volume = atof(value);
    }

}


void SasNode::load()
{
    Node::load();    

    //Initialize analysis
    m_barkBands = new int[NB_BARK_BANDS];
    xtract_init_bark(AudioManager::FFT_SIZE, m_sampleRate, m_barkBands);

    m_staticAnalysed=false;

    //create the sas synth and source
    m_synth = sas_synthesizer_make ();

    m_amp=0.5;
    m_freq=440;    

    for (int i = 0; i < ENVELOPE_SIZE; i++) {
        m_color[i] = 1.0;
    }
    m_colorEnvelope = sas_envelope_make(ENVELOPE_BASE, ENVELOPE_SIZE, m_color);
    sas_envelope_adjust_for_color (m_colorEnvelope);

    for (int i = 0; i < ENVELOPE_SIZE; i++) {
        m_warp[i]=ENVELOPE_BASE*float(i+1);
        m_warpIdentity[i]=ENVELOPE_BASE*float(i+1);
    }
    m_warpEnvelope = sas_envelope_make(ENVELOPE_BASE, ENVELOPE_SIZE, m_warp);
    sas_envelope_adjust_for_warp (m_warpEnvelope);

    m_sourceData.frame = sas_frame_make ();
    sas_frame_set_amplitude (m_sourceData.frame, m_amp);
    sas_frame_set_frequency (m_sourceData.frame, m_freq);
    sas_frame_set_color (m_sourceData.frame, m_colorEnvelope);
    sas_frame_set_warp (m_sourceData.frame, m_warpEnvelope);
    m_sourceData.pos.x = m_sourceData.pos.y = m_sourceData.pos.z = 0.0;
    
    m_sourceData.source = sas_synthesizer_source_make (m_synth, &m_sourceData.pos, update_callback, &m_sourceData);

    sas_synthesizer_synthesize (m_synth, m_outputBuffer);

    m_counter=0;
    


}


void SasNode::unload()
{
    sas_synthesizer_free (m_synth);
    sas_frame_free (m_sourceData.frame);
}

void SasNode::setParameter(const NODE_PARAMETER& param, const std::vector<float>& values, bool valueFromLoop)
{
    Node::setParameter(param, values, valueFromLoop);

    switch(param) {
        case NODE_NOISINESS: {
            float step=1.0/float(ENVELOPE_SIZE);
            float counter=0;
            for (int i = 0; i < ENVELOPE_SIZE; i++, counter+=step) {
                m_warp[i]=(1.0-(values[0]*0.5*counter))*m_warpIdentity[i];
            }
            m_warpEnvelope = sas_envelope_make(ENVELOPE_BASE, ENVELOPE_SIZE, m_warp);
            sas_envelope_adjust_for_warp (m_warpEnvelope);
        }
        break;
        case NODE_PITCH: {
            m_freq = 440.0*pow(2, (values[0]*100.0-57.0)/12.0);
        }break;
        default:break;
    }


}

void SasNode::process(timeval &curTime, std::vector<std::vector<float> >& tempBuffer, std::vector<std::vector<float> >& outputBuffer)
{
    Node::process(curTime, tempBuffer, outputBuffer);

    //first create sample node events from new audio events
    if(m_newEventsWaiting) {
        m_events = m_newEvents;
        m_newEventsWaiting=false;
        for(unsigned int event=0; event<m_events.size(); event++) { 
            switch(NODE_PARAMETER(m_events[event].parameter)) {
                case NODE_EXCITE_INST:{
                    //fade out the previous events
                    for(unsigned int e=0; e<m_instEvents.size(); ++e) {
                        m_instEvents[e].fadeStep=-0.01;
                    }
                    //add the new one
                    m_instEvents.push_back(SasNodeEvent());
                    SasNodeEvent& newEvent = m_instEvents[m_instEvents.size()-1];
                    newEvent.fade=0.0001;    
                    newEvent.fadeStep=m_events[event].values[0]/100.0; // Attack
                }break;
                case NODE_EXCITE_CONT:{
                    
                }break;
                case NODE_EXCITE_CONT_RELEASE:{
                    
                }break;
                case NODE_STOP: {
                    //stop every event
                    for(unsigned int i=0; i<m_instEvents.size(); ++i) {
                        m_instEvents[i].fadeStep=-0.01;
                    }
                }break;
                default:break;
            }

        }
        m_events.clear();
    }

    //reset the temp buffer
    for(unsigned int channel=0; channel<tempBuffer.size(); ++channel) {
        tempBuffer[channel].assign(tempBuffer[channel].size(), 0.00001);
    }

    //process the events
    for(vector<SasNodeEvent>::iterator itEvent=m_instEvents.begin(); itEvent!=m_instEvents.end();++itEvent) {
        //for each frame
        for(unsigned int frame=0; frame<tempBuffer[0].size(); ++frame) {
            //get the enveloppe
            for(unsigned int channel=0; channel<tempBuffer.size(); ++channel) {
                tempBuffer[channel][frame]+=(*itEvent).fade;
            }
            (*itEvent).fade+=(*itEvent).fadeStep;
            //if finished the attack, start decreasing
            if((*itEvent).fade>=1) {
                (*itEvent).fadeStep=-(((1.0-m_parametersValues[NODE_RESONANCE])*0.000009)+0.000001);
            }//if event finished
            else if((*itEvent).fade<=0) {
                m_instEvents.erase(itEvent);
                itEvent--;    
            }
            else if((*itEvent).fadeStep<0){ // if fading out and duration has changed
                if(m_parametersChanged) {                
                    m_instEvents[m_eventCounter].fadeStep=-(((1.0-m_parametersValues[NODE_DURATION])*0.000009)+0.000001);
                    m_parametersChanged=false;
                }
            }
        }
    }
    
    
    //get the synthetised sound and apply the envelope coming from the events
    for(unsigned int frame=0; frame<tempBuffer[0].size(); ++frame) {
        tempBuffer[0][frame]*=float(m_outputBuffer[m_counter]);
        m_counter++;
        tempBuffer[1][frame]*=float(m_outputBuffer[m_counter]);
        m_counter++;

        if(m_counter>=SAS_SAMPLES*2) {
            sas_frame_set_amplitude (m_sourceData.frame, m_parametersValues[NODE_VOLUME]*0.5);
            sas_frame_set_frequency (m_sourceData.frame, m_freq);
            sas_frame_set_color (m_sourceData.frame, m_colorEnvelope);
            sas_frame_set_warp (m_sourceData.frame, m_warpEnvelope);
            sas_synthesizer_synthesize (m_synth, m_outputBuffer);
            m_counter=0;
        }
    }

    //analyse the loudness
    dynamicAnalyse(tempBuffer);

    //add the resulting buffer 
    for(unsigned int channel=0; channel<tempBuffer.size();++channel) {
        for(unsigned int frame=0;frame<tempBuffer[channel].size();++frame) {
            outputBuffer[channel][frame]+=tempBuffer[channel][frame];
        }
    }
}

void SasNode::analyse(timeval& curTime, std::vector<AudioEvent>& events)
{
    //dynamic analysis
    Node::analyse(curTime, events);

    if(!m_staticAnalysed) {

        float argf[4];
        argf[0] = (float)m_sampleRate/(float)AudioManager::FFT_SIZE;
        argf[1] = XTRACT_MAGNITUDE_SPECTRUM;
        argf[2] = 0.f;
        argf[3] = 0.f;

        std::vector<float> tempBuffer(AudioManager::FFT_SIZE, 0);
        std::vector<float> spectrumBuffer(AudioManager::FFT_SIZE*2, 0);
        std::vector<float> tempBarkBuffer(NB_BARK_BANDS, 0);
        std::vector<float> barkBuffer(NB_BARK_BANDS, 0);

        //go through all the samples and add all the bark spectrums
        float spectrumBuffersCount=0;
        if(m_sample->isLoaded()) {
            bool sampleAnalysed=false;
            long offset=0;
            while(!sampleAnalysed) {
                //cout<<offset <<" "<<sample->getFramesCount()<<" "<<sample->getChannelsCount()<<endl;
                int length = AudioManager::FFT_SIZE;
                if(m_sample->getFramesCount() - offset < length) {
                    length = m_sample->getFramesCount() - offset;
                }
                for(int i=0;i<length;++i) {
                    tempBuffer[i]=0;
                    for(int c=0;c<m_sample->getChannelsCount();++c) {
                        tempBuffer[i] += m_sample->getFrame(c,offset)/float(m_sample->getChannelsCount());
                    }
                    offset++;
                }
                //if finished
                if(length<=AudioManager::FFT_SIZE) {
                    sampleAnalysed=true;
                }
                //analyse the buffer
                xtract_spectrum(&(tempBuffer[0]), AudioManager::FFT_SIZE, &argf[0], &(spectrumBuffer[0]));
                xtract_bark_coefficients(&(spectrumBuffer[0]), AudioManager::FFT_SIZE, m_barkBands, &(tempBarkBuffer[0]));
                for(unsigned int i=0;i<NB_BARK_BANDS;++i) {
                    barkBuffer[i]+=tempBarkBuffer[i];
                }
                spectrumBuffersCount++;
            }
        }

        for(int i=0;i<NB_BARK_BANDS;++i) {
            barkBuffer[i]/=spectrumBuffersCount;
        }

        //static analysis
        events.push_back(AudioEvent());
        AudioEvent& newEvent = events[events.size()-1];
        newEvent.type=AUDIO_NODE;
        newEvent.node=m_id;
        newEvent.parameter=NODE_ANALYSIS_STATIC;
        newEvent.values=barkBuffer;
        m_staticAnalysed=true;
    }
}

*/

