/***************************************************************************
 *  HllNode.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "HllNode.hpp"
#include "AudioManager.hpp"

using namespace std;

HllNode::HllNode(const int& id): HllElement(id), m_isPlaying(true) {}

HllNode::HllNode(const int& id, const HllNode& copiedHllNode): HllElement(id) {}

HllNode::~HllNode() {}

void HllNode::process(const double& timeDiff, 
                        std::vector<std::vector<double> >& tempBuffer, 
                            std::vector<std::vector<double> >& outputBuffer, 
                                std::vector<Event>& outputEvents) {
    if(m_parametersChanged) {
        //update parent parameters
        processParentNewValues();
        //set the parameters of children
        for(unsigned int i=0; i<m_children.size();++i) {
            m_children[i]->setParentParameters(m_parametersValues);
        } 
        m_parametersChanged=false;
    }

    //process the children
    for(unsigned int n=0; n<m_children.size(); n++) {
        m_children[n]->process(timeDiff, tempBuffer, 
                                outputBuffer, outputEvents);
    }

    if(m_dynamicAnalysisCounter>=m_dynamicAnalysisPeriod) {
        //get analyses from children
        m_dynamicAnalysis=0;
        vector<HllElement*>::iterator itNode = m_children.begin();
        for(;itNode!=m_children.end(); itNode++) {
            m_dynamicAnalysis+=(*itNode)->getDynamicAnalysis();
        }
        m_dynamicAnalysis/=double(m_children.size());
    }

    HllElement::process(timeDiff, tempBuffer, outputBuffer, outputEvents);
}

void HllNode::loopStop(const int& param) {
    m_isPlaying=false;
    for(unsigned int c=0;c<m_children.size();++c) {
        m_children[c]->loopStop();
    }
}

void HllNode::print(const std::string& prefix) {
    HllElement::print(prefix);
    for(unsigned int c=0; c<m_children.size(); ++c) {
        std::string pre = std::string(" ")+prefix;
        m_children[c]->print(pre);
    }
}

