/***************************************************************************
 *  HllNode.hpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _HLL_NODE_H
#define _HLL_NODE_H


#include "HllElement.hpp"


class HllNode: public HllElement
{
    public:
        HllNode(const int&);
        HllNode(const int&,const HllNode&);
        virtual ~HllNode();

        virtual void duplicate(std::vector<HllElement*>& createElements, 
                        int& elementsIdCounter)=0;

        virtual void process(const double&, std::vector<std::vector<double> >&, 
                std::vector<std::vector<double> >&, std::vector<Event>&);

        inline virtual void setExcitationLoop(std::vector<Event>&){}
        inline virtual void setModulationLoop(const unsigned int&, 
                                            std::vector<Event>&){}

        virtual void addChild(HllElement*)=0;
        virtual void removeChild(HllElement*)=0;
        virtual unsigned int getNbChildren()=0;
        virtual HllElement* getChild(unsigned int)=0;

        inline virtual void loopPlay(double){}
        virtual void loopStop(const int& param=-1);

        std::vector<HllElement*> getChildren(){return m_children;}

        inline const bool& isPlaying(){return m_isPlaying;}

        virtual void print(const std::string& prefix="");

    protected:

        bool m_isPlaying;


};



#endif
