/***************************************************************************
 *  HllNodeLoop.hpp
 *  Part of Drile
 *  2010- Florent Berthaut.
 *  hitmuri.net 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _HLL_NODE_LOOP_H
#define _HLL_NODE_LOOP_H

#include <iostream>
#include <cstring>
#include <cmath>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <vector>
#include <map>

#include "HllNode.hpp"

class HllNodeLoop: public HllNode
{
    public:
        HllNodeLoop(const int&);
        ~HllNodeLoop();
        virtual void duplicate(std::vector<HllElement*>& createElements, 
                        int& elementsIdCounter);

        void parse(xmlNodePtr);

        void addChild(HllElement*);
        void removeChild(HllElement*);
        inline unsigned int getNbChildren(){return m_children.size();}
        inline HllElement* getChild(unsigned int c){return m_children[0];}

        void setExcitationLoop(std::vector<Event>&);
        void setModulationLoop(const unsigned int&, 
                                        std::vector<Event>&);


        void process(const double& timeDiff, 
                std::vector<std::vector<double> >& tempBuffer, 
                    std::vector<std::vector<double> >& outputBuffer, 
                        std::vector<Event>& outputEvents);

        void processLoop(std::vector<Event>& loopEvents, 
                            unsigned int& loopIndex, 
                            double& loopCurrentPosition, 
                            double& playDiffWithSpeed);

        void loopPlay(double diff=0);
        void loopStop(const int& param=-1);

        inline const double& getLoopLength(){return m_excitationLoopLength;}
        inline const double& getLoopPosition(){
            return m_excitationLoopCurrentPosition;}
        inline const double& getLoopElapsedTime(){
            return m_excitationLoopElapsedTime;}

    protected:


        std::vector<Event> m_excitationLoopEvents;
        unsigned int m_excitationLoopIndex;
//        timeval m_excitationLoopPlayTime;
        double m_excitationLoopLength;
        double m_excitationLoopCurrentPosition;
        double m_excitationLoopElapsedTime;
        bool m_excitationLoopPlaying;
        
        std::vector<Event> m_modulationLoopsEvents[NODE_PARAMETERS_NB];
        unsigned int m_modulationLoopsIndex[NODE_PARAMETERS_NB];
//        timeval m_modulationLoopsPlayTime[NODE_PARAMETERS_NB];
        double m_modulationLoopsCurrentPosition[NODE_PARAMETERS_NB];
        double m_modulationLoopsLength[NODE_PARAMETERS_NB];
        bool m_modulationLoopsPlaying[NODE_PARAMETERS_NB];

        timeval m_loopPreviousTime;

        double m_loopSpeed;
        bool m_sentStopPosition;

        void synchronizeModulationLoop(const unsigned int& param);
};



#endif /* _NODE_H */
