/***************************************************************************
 *  HllNodeSync.hpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _HLL_NODE_SYNC_H
#define _HLL_NODE_SYNC_H


#include "HllNode.hpp"
#include "HllNodeLoop.hpp"

struct SyncedChildData{
/*
    SyncedChildData(): m_loopPulseChanged(true), m_loopOffsetRatio(0),
                        m_loopOffset(0), m_loopPulseRatio(1),
                        m_loopPulse(1), m_loopTime(0) {}
*/

//    bool m_loopPulseChanged;
//    double m_loopOffsetRatio;
//    double m_loopOffset;
    std::vector<double> loopPulses;
    unsigned int loopCounter;
//    double loopTime;
};


class HllNodeSync: public HllNode
{
    public:
        HllNodeSync(const int&);
        ~HllNodeSync();

        virtual void duplicate(std::vector<HllElement*>& createElements, 
                        int& elementsIdCounter);

        void parse(xmlNodePtr);

        inline unsigned int getNbChildren(){return m_children.size();}
        inline HllElement* getChild(unsigned int c){return m_children[c];}

        void addChild(HllElement* el);
        void removeChild(HllElement*);

        void loopStop(const int& param=-1);
        void processEvent(const Event& event, bool eventFromLoop);
        void process(const double& timeDiff, 
                std::vector<std::vector<double> >& tempBuffer, 
                    std::vector<std::vector<double> >& outputBuffer, 
                        std::vector<Event>& outputEvents);

        void setPulseNode(HllNodeLoop*);
        inline HllNodeLoop* getPulseNode(){return m_pulseNode;}
        
    protected:

        double m_pulseDryLength;
        double m_pulseRatio;
        double m_pulseLength;
        double m_pulseCounter;
        HllNodeLoop* m_pulseNode;
    
        std::vector<SyncedChildData> m_childrenData;

        void synchronizeChild(const unsigned int& child);

};



#endif
