/***************************************************************************
 *  Sample.hpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _Sample_H
#define _Sample_H

#include <iostream>
#include <stdio.h>
#include <string.h>
#include <vector>
#include <stdexcept>
#include <cmath>

#include <sndfile.h>
#include <samplerate.h>

class Sample {
    public:
        Sample(const std::string&);    
        ~Sample();    
        
        inline std::string getName(){return m_name;}
        
        void load();
        void unload();
    
        inline void addUser(){m_nbUsers++;}
        inline int getNbUsers(){return m_nbUsers;}
        inline void removeUser(){if(m_nbUsers>0){m_nbUsers--;}}

        const double& getFrame(const long&);
        const double& getLinInterpFrame(const double& offset);
        inline const int& getChannelsCount(){return m_channelsCount;};
        inline const long& getFramesCount(){return m_framesCount;};
        inline const double& getFloatFramesCount(){return m_doubleFramesCount;}
        inline bool isLoaded(){return m_loaded;};    
        const std::vector<double>& getBuffer() const;
        void setSampleRate(int);

    private:
        std::string m_name;
        std::vector<double> m_buffer;
        int m_channelsCount;
        int m_sampleRate;
        long m_framesCount; 
        double m_doubleFramesCount; 
        bool m_loaded;
        int m_nbUsers;
        double m_defaultFrame, m_computedFrame;
        
        static const int INTERP_SIZE=256;
        double m_interpCoeffs[INTERP_SIZE][4];
};



#endif 
