/***************************************************************************
 *  Mapping.cpp
 *  Part of Drile
 *  2010-2013 Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Mapping.hpp"
#include "MappingID.hpp"
#include "../env/Piivert.hpp"

using namespace std;

Mapping::Mapping(const int& id, const HAND& hand): m_hand(hand), m_id(id), 
                                                m_loopState(LOOP_NONE), 
                                                m_grabToggle(false) {
    //resize to the number of fingers and set to none selected
    m_selectedWorms.resize(5,-1);
    m_newSelectedWorms.resize(5,-1);
}

Mapping::~Mapping() {}


void Mapping::processOutput(const OutputEvent&) {}

void Mapping::getMappingsIDs(std::vector<MappingID>& mappingsIDs){
    vector<MappingID>::iterator itMap;
    for(itMap=m_ids.begin(); itMap!=m_ids.end(); ++itMap) {
        mappingsIDs.push_back(*itMap);
    }
}

void Mapping::startRecordingSelectedWorms(timeval &recordingStartTime, 
                                std::vector<Event>& outputAudioEvents, 
                                std::vector<Event>& outputEnvEvents) {
    m_recordingStartTime=recordingStartTime;

    for(unsigned int i=0; i<m_selectedWorms.size(); ++i) {
        if(m_selectedWorms[i]>=0) {
            startRecordingWorm(m_selectedWorms[i], outputAudioEvents, 
                                outputEnvEvents);
        }
    }
    m_loopState=LOOP_RECORDING;
}

void Mapping::startRecordingWorm(double& wormID, 
                                std::vector<Event>& outputAudioEvents, 
                                std::vector<Event>& outputEnvEvents) {
    //see if this worm was already in the recording list
    bool found=false;
    for(unsigned int i=0;i<m_recordedWorms.size();++i) {
        if(m_recordedWorms[i]==wormID) {
            found=true;
        }
    }
    //if not already in the list, add it to the recorded worms, and send the loop start 
    if(!found) {
        m_recordedWorms.push_back(wormID);

        outputEnvEvents.push_back(Event());
        Event &eEv = outputEnvEvents.back();
        eEv.type=EVENT_NODE;
        eEv.parameter=NODE_LOOP_RECORDING;
        eEv.node=wormID;
        eEv.values[0]=1;

        outputAudioEvents.push_back(Event());
        Event &aEv = outputAudioEvents.back();
        aEv.type=EVENT_TREE;
        aEv.parameter=TREE_BUILD_RECORD;
        aEv.node=wormID;
        aEv.time=m_recordingStartTime;
    }
}

void Mapping::stopRecordingWorms(timeval& recordingStopTime, 
                                std::vector<Event>& outputAudioEvents, 
                                std::vector<Event>& outputEnvEvents) {
    m_recordingStartTime=recordingStopTime;
    for(unsigned int i=0;i<m_recordedWorms.size();++i) {
        outputEnvEvents.push_back(Event());
        Event &eEv = outputEnvEvents.back();
        eEv.type=EVENT_NODE;
        eEv.parameter=NODE_LOOP_RECORDING;
        eEv.node=m_recordedWorms[i];
        eEv.values[0]=0;    

        outputAudioEvents.push_back(Event());
        Event &aEv = outputAudioEvents.back();
        aEv.type=EVENT_TREE;
        aEv.parameter=TREE_BUILD_CREATE;
        aEv.node=m_recordedWorms[i];
        aEv.time=m_recordingStartTime;    
    }
    m_loopState=LOOP_NONE;
    m_recordedWorms.clear();
}

void Mapping::cancelRecordingWorms(std::vector<Event>& outputAudioEvents, 
                                    std::vector<Event>& outputEnvEvents) {
    for(unsigned int i=0;i<m_recordedWorms.size();++i) {
        outputEnvEvents.push_back(Event());
        Event &eEv = outputEnvEvents.back();
        eEv.type=EVENT_NODE;
        eEv.parameter=NODE_LOOP_RECORDING;
        eEv.node=m_recordedWorms[i];
        eEv.values[0]=0;    
        outputAudioEvents.push_back(Event());
        Event &aEv = outputAudioEvents.back();
        aEv.type=EVENT_TREE;
        aEv.parameter=TREE_BUILD_CANCEL;
        aEv.node=m_recordedWorms[i];
        aEv.time=m_recordingStartTime;    
    }
    m_recordedWorms.clear();
    m_loopState=LOOP_NONE;
}

void Mapping::eraseRecordingWorms() {
    m_recordedWorms.clear();
    m_loopState=LOOP_NONE;
}

void Mapping::release(std::vector<Event>& outputAudioEvents, 
                                    std::vector<Event>& outputEnvEvents) {
    if(m_grabToggle) {
        m_grabToggle=!m_grabToggle;
        outputEnvEvents.push_back(Event());
        Event& eEv = outputEnvEvents.back();
        eEv.type=EVENT_PIIVERT;
        eEv.parameter=PIIVERT_GRAB;
        eEv.piivert=m_id;
        eEv.hand=m_hand;
        eEv.values[0]=m_grabToggle;
    }
}

