/***************************************************************************
 *            PiivertInput.hpp
 *            Part of Drile
 *
 *  2010  Tardigrade Inc.
 *  http://tardigrade-inc.com 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _MAPPING_H
#define _MAPPING_H

#include <iostream>
#include <cstring>
#include <sstream>
#include <cmath>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <vector>
#include <map>
#include <libxml/parser.h>
#include <libxml/tree.h>

#include <OpenSG/OSGQuaternion.h>

#include "../Event.hpp"
#include "../HllEnums.hpp"
#include "OutputEvent.hpp"
#include "MappingID.hpp"

enum HAND{LEFT_HAND, RIGHT_HAND};

class Mapping {

    public:
        Mapping(const int&, const HAND& hand=LEFT_HAND);
        virtual ~Mapping();

        virtual void process(InputEvent&, std::vector<Event>&, std::vector<Event>&)=0;
        virtual void processOutput(const OutputEvent&);
        virtual void update(std::vector<Event>&, std::vector<Event>&){}

        inline void setSelectedWorms(double* wormsIDs){
                    m_newSelectedWorms.assign(wormsIDs, wormsIDs+5);
                    m_selectedWormsHaveChanged=true;
        }
        
        void startRecordingSelectedWorms(timeval&, std::vector<Event>&, std::vector<Event>&);
        void startRecordingWorm(double&, std::vector<Event>&, std::vector<Event>&);
        void stopRecordingWorms(timeval&, std::vector<Event>&, std::vector<Event>&);
        void cancelRecordingWorms(std::vector<Event>&, std::vector<Event>&);
        void eraseRecordingWorms();

        void getMappingsIDs(std::vector<MappingID>&);

        void release(std::vector<Event>&, std::vector<Event>&);
    
    protected:

        HAND m_hand;
        int m_id;

        bool m_selectedWormsHaveChanged;
        std::vector<double> m_newSelectedWorms;
        std::vector<double> m_selectedWorms;

        timeval m_recordingStartTime;
        std::vector<double> m_recordedWorms;
        LOOP_STATE m_loopState;

        OSG::Vec3f m_initPosition;
        OSG::Quaternion m_initRotation;
        OSG::Vec3f m_initScale;

        std::vector<MappingID> m_ids;

        bool m_grabToggle;
};



#endif
