/***************************************************************************
 *            HandDevice.cpp
 *            Part of Drile
 *
 *  2010  Tardigrade Inc.
 *  http://tardigrade-inc.com 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "HandDevice.hpp"
#include "../../Drile.hpp"

using namespace std;

HandDevice::HandDevice(const int& id):Device(id)
{
    ostringstream oss;
    oss<<m_id;
    m_name = "Hand"+oss.str();

}

HandDevice::~HandDevice()
{

}

void HandDevice::parse(xmlNodePtr node)
{
    xmlNodePtr piivSubNode;
    for (piivSubNode = node->children; piivSubNode; piivSubNode = piivSubNode->next) {
        if (piivSubNode->type == XML_ELEMENT_NODE && !xmlStrcmp(piivSubNode->name, (const xmlChar *) "Input")) {
            char* value=NULL;
            value = (char*)xmlGetProp(piivSubNode,(xmlChar*)"type");
            if(value!=NULL) {
                std::string typeStr = value;

                std::string leftHandControls, rightHandControls;
                value = NULL;
                value = (char*)xmlGetProp(piivSubNode,(xmlChar*)"right_hand");
                if(value!=NULL) {
                    rightHandControls=value;
                }
                value = NULL;
                value = (char*)xmlGetProp(piivSubNode,(xmlChar*)"left_hand");
                if(value!=NULL) {
                    leftHandControls=value;
                }
                
                //create new input according to type
                if(typeStr.compare("tracking_vrpn")==0) {
                    m_mappings[RIGHT_HAND].push_back(
                        new HandTrackingVrpn(m_id, rightHandControls, 
                        RIGHT_HAND, this));
                    m_mappings[LEFT_HAND].push_back(
                        new HandTrackingVrpn(m_id, leftHandControls, 
                        LEFT_HAND, this));
                }
                else if(typeStr.compare("grab_vrpn")==0) {
                    m_mappings[RIGHT_HAND].push_back(
                        new HandGrabVrpn(m_id, rightHandControls, 
                            RIGHT_HAND, this));
                    m_mappings[LEFT_HAND].push_back(
                        new HandGrabVrpn(m_id, leftHandControls, 
                            LEFT_HAND, this));
                }
                else {
                    throw(std::logic_error(
                            "HandDevice: undefined mapping "+typeStr));
                }
            }
        }
        else if (piivSubNode->type == XML_ELEMENT_NODE && !xmlStrcmp(piivSubNode->name, (const xmlChar *) "Output")) {
            char* value=NULL;
            value = (char*)xmlGetProp(piivSubNode,(xmlChar*)"type");
            if(value!=NULL) {
                std::string typeStr = value;
                std::string param;

                value = NULL;
                value = (char*)xmlGetProp(piivSubNode,(xmlChar*)"param");
                if(value!=NULL) {
                    param=value;
                }
            }
        }
    }

}

void HandDevice::getMappingsIDs(std::vector<MappingID>&) {


}

void HandDevice::setSelectedWorms(const HAND& hand, double* values)
{
    //set input mappings
    for(unsigned int i=0; i<m_mappings[hand].size(); ++i) {
        m_mappings[hand][i]->setSelectedWorms(values);
    }

/*
    //send to outputMappings
    OutputEvent ev;
    ev.type=OUTPUT_PIIVERT;
    ev.parameter=PIIVERT_SELECT_WORM;
    ev.id=m_id;
    ev.strId=m_name;
    ev.hand=hand;
    ev.values=wormsIDs;
    for(unsigned int i=0; i<m_outputMappings.size(); ++i) {
        m_outputMappings[i]->processOutput(ev);
    }
*/
}

void HandDevice::setSpeed(const HAND& hand, double* values) 
{
/*
    //send to outputMappings
    OutputEvent ev;
    ev.type=OUTPUT_PIIVERT;
    ev.parameter=PIIVERT_SPEED;
    ev.id=m_id;
    ev.strId=m_name;
    ev.hand=hand;
    ev.values=values;
    for(unsigned int i=0; i<m_outputMappings.size(); ++i) {
        m_outputMappings[i]->processOutput(ev);
    }
*/
}

void HandDevice::setGrab(const HAND& hand, const double& press) 
{
    //send to outputMappings
    OutputEvent ev;
    ev.type=OUTPUT_PIIVERT;
    ev.parameter=PIIVERT_GRAB;
    ev.hand=hand;
    ev.strId=m_name;
    ev.id=m_id;
    ev.values[0]=press;
    for(unsigned int i=0; i<m_outputMappings.size(); ++i) {
        m_outputMappings[i]->processOutput(ev);
    }
}



