/***************************************************************************
 *            HandGrabVrpn.cpp
 *            Part of Drile
 *
 *  2010  Tardigrade Inc.
 *  http://tardigrade-inc.com 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "HandGrabVrpn.hpp"
#include "../../env/Piivert.hpp"
#include "HandDevice.hpp"
#include "../../input/InputManager.hpp"

using namespace std;

HandGrabVrpn::HandGrabVrpn(const int& id, const std::string& param, 
                            const HAND& hand, HandDevice* handDevice): 
                                Mapping(id, hand), m_handDevice(handDevice),
                                m_duration(30), m_durationF(30.0),
                                m_thresholdMinOpen(0.2), 
                                m_thresholdMaxClose(0.8),
                                m_indexBC(0), m_sum(0), m_popBut(1.0), 
                                m_closed(false){

    std::string device = param;
    size_t pos=device.find_first_of(';');

    //get adress
    std::string adressStr = device.substr(0,pos);
    InputManager *inputManager = InputManager::getInstance();
    int vrpnTrackerID = inputManager->addVrpnButton(adressStr);

    //get sensor id
    device = device.substr(pos+1);
    pos=device.find_first_of(';');
    std::istringstream is;
    is.str(device.substr(0, pos));
    int sensor; 
    is>>sensor;

    m_initPosition[0]=0;    
    m_initPosition[1]=0;    
    m_initPosition[2]=0;    
    m_initScale[0]=1;    
    m_initScale[1]=1;    
    m_initScale[2]=1;    


    m_ids.push_back(MappingID(INPUT_VRPN_BUTTON, this, 
                        vrpnTrackerID*100+sensor));

    m_bufferClose = new double[m_duration];
    for (int i =0; i< m_duration; i++){ //Fill the buffer with close information
        m_bufferClose[i] = 1.0;
        m_sum += m_bufferClose[i];
    }
    


}

HandGrabVrpn::~HandGrabVrpn()
{

}

void HandGrabVrpn::process(InputEvent& event, 
                                std::vector<Event>& outputAudioEvents, 
                                std::vector<Event>& outputEnvEvents) {
    if(m_selectedWormsHaveChanged) {
        m_selectedWorms = m_newSelectedWorms;
        m_selectedWormsHaveChanged=false;
    }


    m_popBut = m_bufferClose[m_indexBC];
    m_sum -= m_popBut;
    m_sum += event.values[0];
    m_bufferClose[m_indexBC] = event.values[0];
    m_indexBC = (m_indexBC +1)%m_duration;

    bool isStable = m_handDevice->getLocationStable(m_hand);
    
    if(m_sum/m_durationF < m_thresholdMinOpen && isStable && m_closed) {
        outputEnvEvents.push_back(Event());
        Event& eEv = outputEnvEvents.back();
        eEv.type=EVENT_PIIVERT;
        eEv.parameter=PIIVERT_GRAB;
        eEv.piivert=m_id;
        eEv.hand=m_hand;
        eEv.finger=event.parameter;
        eEv.values[0]=0;
        m_closed=false;
       // cout<<"Open hand"<<endl;
    }

    if(m_sum/m_durationF > m_thresholdMaxClose && isStable && !m_closed) {
        outputEnvEvents.push_back(Event());
        Event& eEv = outputEnvEvents.back();
        eEv.type=EVENT_PIIVERT;
        eEv.parameter=PIIVERT_GRAB;
        eEv.piivert=m_id;
        eEv.hand=m_hand;
        eEv.finger=event.parameter;
        eEv.values[0]=1;
        m_closed=true;
      //  cout<<"Close hand"<<endl;
    }

/*
    if(location.stable())
    {
    if (sum/DURATION_F < THRESHOLD_MIN_OPEN) //BLA


    if (sum/DURATION_F > THRESHOLD_MAX_CLOSE) //BLA


    }
*/



/*

    //finger: 11= left hand index, 12=left hand middle, 21=right hand index ...
    int finger = (m_hand+1)*10 + event.parameter+1;
    if(event.values[0]==1) {

    //test gestures
    m_piivertDevice->processGesture(finger);

    //send a Hit to audio if index and a grab to Env if middle finger
    switch(finger) {
        case 1514 :
        case 2425: //flam reverse one hand
        case 1525 :
        case 2414:{ //flam reverse two hands
            if(m_loopState==LOOP_RECORDING) {
                m_piivertDevice->cancelRecordingBothHands(outputAudioEvents, outputEnvEvents);
                m_piivertDevice->eraseRecordingBothHands();
            }
            else {
                if(m_selectedWorms[1]>=0) {
                    outputEnvEvents.push_back(EnvEvent());
                    EnvEvent &eEv = outputEnvEvents[outputEnvEvents.size()-1];
                    eEv.type=ENV_TREE;
                    eEv.parameter=TREE_EXTRACT;
                    eEv.worm=m_selectedWorms[1];
                    eEv.time=m_recordingStartTime;    

                    outputAudioEvents.push_back(AudioEvent());
                    AudioEvent &aEv = outputAudioEvents[outputAudioEvents.size()-1];
                    aEv.type=AUDIO_TREE;
                    aEv.parameter=TREE_EXTRACT;
                    aEv.node=m_selectedWorms[1];
                    aEv.time=m_recordingStartTime;    
                }
            }
        }break;
        case 1415 :
        case 2524:{ //record one hand 
            //set velocity depending on timing
            //m_hitVelocity = m_piivertDevice->getLastGestureDurationParameter();
            if(m_loopState==LOOP_RECORDING) {
                timeval recordingStopTime;
                gettimeofday(&recordingStopTime, NULL);
                stopRecordingWorms(recordingStopTime, outputAudioEvents, outputEnvEvents);
            }
            else { 
                if(m_loopState==LOOP_NONE) {
                    timeval recordingStartTime;
                    gettimeofday(&recordingStartTime, NULL);
                    startRecordingSelectedWorms(recordingStartTime, outputAudioEvents, outputEnvEvents);
                }
                if(m_selectedWorms[1]>=0) {
                    //excite
                    outputAudioEvents.push_back(AudioEvent());
                    AudioEvent &aEv = outputAudioEvents[outputAudioEvents.size()-1];
                    aEv.type=AUDIO_NODE;
                    aEv.parameter=NODE_EXCITE_INST;
                    aEv.node=m_selectedWorms[1];
                    aEv.values[0]=m_hitVelocity;

                    outputAudioEvents.push_back(AudioEvent());
                    AudioEvent &a2Ev = outputAudioEvents[outputAudioEvents.size()-1];
                    a2Ev.type=AUDIO_NODE;
                    a2Ev.parameter=NODE_EXCITE_CONT;
                    a2Ev.node=m_selectedWorms[1];
                    a2Ev.values[0]=m_hitVelocity/2.0;
                }
            }
            outputEnvEvents.push_back(EnvEvent());
            EnvEvent &eEv = outputEnvEvents[outputEnvEvents.size()-1];
            eEv.type=ENV_PIIVERT;
            eEv.parameter=PIIVERT_HIT;
            eEv.piivert=m_id;
            eEv.hand=m_hand;
            eEv.finger=event.parameter;
            eEv.values[0]=m_hitVelocity;
        }break;
        case 2515 :
        case 1424:{ //flam two hands
            //record
            switch(m_loopState) {
                case LOOP_NONE: {
                    m_piivertDevice->startRecordingBothHands(outputAudioEvents, outputEnvEvents);
                    //also send a hit
                    if(m_selectedWorms[1]>=0) {
                        outputAudioEvents.push_back(AudioEvent());
                        AudioEvent &aEv = outputAudioEvents[outputAudioEvents.size()-1];
                        aEv.type=AUDIO_NODE;
                        aEv.parameter=NODE_EXCITE_INST;
                        aEv.node=m_selectedWorms[1];
                        aEv.values[0]=m_hitVelocity;

                        outputAudioEvents.push_back(AudioEvent());
                        AudioEvent &a2Ev = outputAudioEvents[outputAudioEvents.size()-1];
                        a2Ev.type=AUDIO_NODE;
                        a2Ev.parameter=NODE_EXCITE_CONT;
                        a2Ev.node=m_selectedWorms[1];
                        a2Ev.values[0]=m_hitVelocity/2.0;
                    }
                    outputEnvEvents.push_back(EnvEvent());
                    EnvEvent &eEv = outputEnvEvents[outputEnvEvents.size()-1];
                    eEv.type=ENV_PIIVERT;
                    eEv.parameter=PIIVERT_HIT;
                    eEv.piivert=m_id;
                    eEv.hand=m_hand;
                    eEv.finger=event.parameter;
                    eEv.values[0]=m_hitVelocity;
                }break;
                case LOOP_RECORDING: {
                    m_piivertDevice->stopRecordingBothHands(outputAudioEvents, outputEnvEvents);
                    m_piivertDevice->eraseRecordingBothHands();
                }break;
                default:break;
            }
        }break;
        case 1524 :
        case 2415 :
        case 15 :
        case 24:{
            //hit or flam hit hit 
            //play
            if(m_selectedWorms[1]>=0) {
                //start recording
                if(m_loopState==LOOP_RECORDING) {
                    startRecordingWorm(m_selectedWorms[1], outputAudioEvents, outputEnvEvents);
                }
                //excite
                outputAudioEvents.push_back(AudioEvent());
                AudioEvent &aEv = outputAudioEvents[outputAudioEvents.size()-1];
                aEv.type=AUDIO_NODE;
                aEv.parameter=NODE_EXCITE_INST;
                aEv.node=m_selectedWorms[1];
                aEv.values[0]=m_hitVelocity;

                outputAudioEvents.push_back(AudioEvent());
                AudioEvent &a2Ev = outputAudioEvents[outputAudioEvents.size()-1];
                a2Ev.type=AUDIO_NODE;
                a2Ev.parameter=NODE_EXCITE_CONT;
                a2Ev.node=m_selectedWorms[1];
                a2Ev.values[0]=m_hitVelocity/2.0;
            }
            outputEnvEvents.push_back(EnvEvent());
            EnvEvent &eEv = outputEnvEvents[outputEnvEvents.size()-1];
            eEv.type=ENV_PIIVERT;
            eEv.parameter=PIIVERT_HIT;
            eEv.piivert=m_id;
            eEv.hand=m_hand;
            eEv.finger=event.parameter;
            eEv.values[0]=m_hitVelocity;
        }break;
        case 1425 :
        case 2514 :{ //two hands grab/grab flam
            outputEnvEvents.push_back(EnvEvent());
            EnvEvent& eEv = outputEnvEvents[outputEnvEvents.size()-1];
            eEv.type=ENV_PIIVERT;
            eEv.parameter=PIIVERT_STORE;
            eEv.piivert=m_id;
            eEv.hand=m_hand;
            eEv.finger=event.parameter;
            eEv.values[0]=0.5;
        }break;
        case 14:
        case 25:{ //grab
            if(m_selectedWorms[1]>=0) {
                if(m_loopState==LOOP_RECORDING) {
                    startRecordingWorm(m_selectedWorms[1], outputAudioEvents, outputEnvEvents);
                }
                outputAudioEvents.push_back(AudioEvent());
                AudioEvent &aEv = outputAudioEvents[outputAudioEvents.size()-1];
                aEv.type=AUDIO_NODE;
                aEv.parameter=NODE_STOP;
                aEv.node=m_selectedWorms[1];
                aEv.values[0]=1;

                outputEnvEvents.push_back(EnvEvent());
                EnvEvent &eEv2 = outputEnvEvents[outputEnvEvents.size()-1];
                eEv2.type=ENV_WORM;
                eEv2.parameter=WORM_STOP;
                eEv2.worm=m_selectedWorms[1];
                eEv2.values[0]=1;
            }
            outputEnvEvents.push_back(EnvEvent());
            EnvEvent& eEv = outputEnvEvents[outputEnvEvents.size()-1];
            eEv.type=ENV_PIIVERT;
            eEv.parameter=PIIVERT_GRAB;
            eEv.piivert=m_id;
            eEv.hand=m_hand;
            eEv.finger=event.parameter;
            eEv.values[0]=1;
        }break;
        case 26:
        case 16: { //middle button, grab and lock
            m_middleReleased=false;
            switch(m_slipGrabRelease) {
                case 0: {
                    outputEnvEvents.push_back(EnvEvent());
                    EnvEvent& eEv = outputEnvEvents[outputEnvEvents.size()-1];
                    eEv.type=ENV_PIIVERT;
                    eEv.parameter=PIIVERT_GRAB;
                    eEv.piivert=m_id;
                    eEv.hand=m_hand;
                    eEv.finger=event.parameter;
                    eEv.values[0]=0.5;
                    m_slipGrabRelease=1;
                }break;
                case 1: {
                    outputEnvEvents.push_back(EnvEvent());
                    EnvEvent& eEv = outputEnvEvents[outputEnvEvents.size()-1];
                    eEv.type=ENV_PIIVERT;
                    eEv.parameter=PIIVERT_GRAB;
                    eEv.piivert=m_id;
                    eEv.hand=m_hand;
                    eEv.finger=event.parameter;
                    eEv.values[0]=1;
                    m_slipGrabRelease=2;
                }break;
                case 2: {
                    outputEnvEvents.push_back(EnvEvent());
                    EnvEvent &eEv = outputEnvEvents[outputEnvEvents.size()-1];
                    eEv.type=ENV_PIIVERT;
                    eEv.parameter=PIIVERT_GRAB;
                    eEv.piivert=m_id;
                    eEv.hand=m_hand;
                    eEv.finger=event.parameter;
                    eEv.values[0]=0;
                    m_slipGrabRelease=0;
                }break;
                default:break;
            }
        }break;
        default:break;
        }
    }
    else {
        //send release object to Env
        switch(finger) {
            case 14:
            case 25:{
                outputEnvEvents.push_back(EnvEvent());
                EnvEvent &eEv = outputEnvEvents[outputEnvEvents.size()-1];
                eEv.type=ENV_PIIVERT;
                eEv.parameter=PIIVERT_GRAB;
                eEv.piivert=m_id;
                eEv.hand=m_hand;
                eEv.finger=event.parameter;
                eEv.values[0]=0;
            }break;
            case 24:
            case 15:{
                //excite release
                outputAudioEvents.push_back(AudioEvent());
                AudioEvent &aEv = outputAudioEvents[outputAudioEvents.size()-1];
                aEv.type=AUDIO_NODE;
                aEv.parameter=NODE_EXCITE_CONT_RELEASE;
                aEv.node=m_selectedWorms[1];
                aEv.values[0]=m_hitVelocity;
            }
            case 26:
            case 16: {
                m_middleReleased=true;
            }break;
            default:break;
        }
    }
*/

}

