/***************************************************************************
 *            HandTrackingVrpn.cpp
 *            Part of Drile
 *
 *  2010  Tardigrade Inc.
 *  http://tardigrade-inc.com 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "HandTrackingVrpn.hpp"
#include "HandDevice.hpp"
#include "../../input/InputManager.hpp"
#include "../../env/Piivert.hpp"

using namespace std;

HandTrackingVrpn::HandTrackingVrpn(const int& id, const std::string& param, 
                                    const HAND& hand, HandDevice* handDevice): 
                                    Mapping(id, hand), m_handDevice(handDevice),
                                    m_speedThreshold(0.005), 
                                    m_freezeThreshold(0.03), m_indexBL(0),
                                    m_duration(30), m_durationF(30){

    std::string device = param;
    size_t pos=device.find_first_of(';');

    //get adress
    std::string adressStr = device.substr(0,pos);
    InputManager *inputManager = InputManager::getInstance();
    int vrpnTrackerID = inputManager->addVrpnTracker(adressStr);

    //get sensor id
    device = device.substr(pos+1);
    pos=device.find_first_of(';');
    std::istringstream is;
    is.str(device.substr(0, pos));
    int sensor; 
    is>>sensor;

    m_initPosition[0]=0;    
    m_initPosition[1]=0;    
    m_initPosition[2]=0;    
    m_initScale[0]=1;    
    m_initScale[1]=1;    
    m_initScale[2]=1;    

    m_ids.push_back(MappingID(INPUT_VRPN_TRACKER, this, 
                                vrpnTrackerID*100+sensor));

    //if init pos
    device = device.substr(pos+1);
    pos=device.find_first_of(';');
    if(pos!=string::npos) {
        std::istringstream is1,is2,is3;
        std::string posStr = device.substr(0, pos);
        size_t pos2 = posStr.find_first_of('/');
        is1.str(posStr.substr(0, pos2));
        is1>>m_initPosition[0];
        posStr = posStr.substr(pos2+1);
        pos2 = posStr.find_first_of('/');
        is2.str(posStr.substr(0, pos2));
        is2>>m_initPosition[1];
        posStr = posStr.substr(pos2+1);
        pos2 = posStr.find_first_of(';');
        is3.str(posStr.substr(0, pos2));
        is3>>m_initPosition[2];
    }

    //if init rot
    device = device.substr(pos+1);
    pos=device.find_first_of(';');
    if(pos!=string::npos) {
        std::istringstream is1,is2,is3;
        double rotX,rotY,rotZ;
        std::string posStr = device.substr(0, pos);
        size_t pos2 = posStr.find_first_of('/');
        is1.str(posStr.substr(0, pos2));
        is1>>rotX;
        posStr = posStr.substr(pos2+1);
        pos2 = posStr.find_first_of('/');
        is2.str(posStr.substr(0, pos2));
        is2>>rotY;
        posStr = posStr.substr(pos2+1);
        pos2 = posStr.find_first_of(';');
        is3.str(posStr.substr(0, pos2));
        is3>>rotZ;
        m_initRotation.setValue(rotX/180.0*M_PI, rotY/180.0*M_PI, rotZ/180.0*M_PI);
    }
    //if init scale
    device = device.substr(pos+1);
    pos=device.find_first_of(';');
    if(pos!=string::npos) {
        std::istringstream is1,is2,is3;
        std::string posStr = device.substr(0, pos);
        size_t pos2 = posStr.find_first_of('/');
        is1.str(posStr.substr(0, pos2));
        is1>>m_initScale[0];
        posStr = posStr.substr(pos2+1);
        pos2 = posStr.find_first_of('/');
        is2.str(posStr.substr(0, pos2));
        is2>>m_initScale[1];
        posStr = posStr.substr(pos2+1);
        pos2 = posStr.find_first_of(';');
        is3.str(posStr.substr(0, pos2));
        is3>>m_initScale[2];
    }
    

    for(unsigned int i=0;i<3;++i) {
        m_bufferLocation[i] = new double[m_duration];
        for(int d=0;d<m_duration;++d) {
            m_bufferLocation[i][d]=0;
        }
        m_sum[i]=0;
        m_var[i]=0;
        m_popLoc[i]=1.0;
        m_previousPosition[i]=0;
    }
}

HandTrackingVrpn::~HandTrackingVrpn()
{

}

void HandTrackingVrpn::process(InputEvent& event, 
                                    std::vector<Event>& outputAudioEvents, 
                                    std::vector<Event>& outputEnvEvents)
{
    //values: x/y/z/quat(x,y,z,w)
    outputEnvEvents.push_back(Event());
    Event& eEv = outputEnvEvents.back();
    eEv.type=EVENT_PIIVERT;
    eEv.parameter=PIIVERT_POSQUAT;
    eEv.piivert = m_id;
    eEv.hand = m_hand;

    eEv.values[0]=event.values[0];
    eEv.values[1]=event.values[1];
    eEv.values[2]=event.values[2];
    eEv.values[3]=event.values[3];
    eEv.values[4]=event.values[4];
    eEv.values[5]=event.values[5];
    eEv.values[6]=event.values[6];
    
    //compute speed
    m_currentSpeed=sqrt(pow(event.values[0]-m_previousPosition[0],2)+
                            pow(event.values[1]-m_previousPosition[1],2)+
                            pow(event.values[2]-m_previousPosition[2],2));


   

    for(unsigned int i=0;i<3;++i) {
        //record previous position
        m_previousPosition[i]=event.values[i];

        //compute mean location
        m_popLoc[i] = m_bufferLocation[i][m_indexBL];
        m_sum[i] -= m_popLoc[i];
        m_sum[i] += event.values[i];
        m_bufferLocation[i][m_indexBL] = event.values[i];
    }

  //  if(m_hand==LEFT_HAND) {
  //      cout<<"Speed "<<m_currentSpeed<<endl;
  //  }

    if(m_currentSpeed < m_speedThreshold) {
        for(unsigned int c=0;c<3;++c) {
            m_var[c] = 0;
            for(int i = 0; i < m_duration; i++) {
                m_var[c]+=abs(m_bufferLocation[c][i] - m_sum[c] / m_durationF);
            }
            m_var[c] = m_var[c] / m_durationF;
        }

     //   if(m_hand==LEFT_HAND) {
     //      cout<<"Sum "<<m_var[0] + m_var[1] + m_var[2]<<endl;
     //   }
        if (m_var[0] + m_var[1] + m_var[2] < m_freezeThreshold) {   
           m_handDevice->setLocationStable(m_hand,true);
        }
        else {
           m_handDevice->setLocationStable(m_hand,false);
        }
    }
    else {
       m_handDevice->setLocationStable(m_hand,false);
    }


    
    m_indexBL = (m_indexBL +1)%m_duration;

}

