/***************************************************************************
 *            HeadTrackingVRPNHydra.cpp
 *            Part of Drile
 *
 *  2010  Tardigrade Inc.
 *  http://tardigrade-inc.com 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "HeadTrackingVRPNHydra.hpp"
#include "../../env/Camera.hpp"
#include "../../input/InputManager.hpp"
#include <OpenSG/OSGQuaternion.h>

using namespace std;

HeadTrackingVRPNHydra::HeadTrackingVRPNHydra(const int& id, const std::string& dev): Mapping(id)
{
    std::string device = dev;
    size_t pos=device.find_first_of(';');

    //get adress
    std::string adressStr = device.substr(0,pos);
    InputManager *inputManager = InputManager::getInstance();
    int vrpnTrackerID = inputManager->addVrpnTracker(adressStr);

    //get sensor id
    device = device.substr(pos+1);
    pos=device.find_first_of(';');
    std::istringstream is;
    is.str(device.substr(0, pos));
    int sensor; 
    is>>sensor;

    m_ids.push_back(MappingID(INPUT_VRPN_TRACKER, 
                        this, vrpnTrackerID*100+sensor));
    m_initPosition[0]=0;    
    m_initPosition[1]=0;    
    m_initPosition[2]=0;    

    //if init pos
    device = device.substr(pos+1);
    pos=device.find_first_of(';');
    if(pos!=string::npos) {
        std::istringstream is1,is2,is3;
        std::string posStr = device.substr(0, pos);
        size_t pos2 = posStr.find_first_of('/');
        is1.str(posStr.substr(0, pos2));
        is1>>m_initPosition[0];
        posStr = posStr.substr(pos2+1);
        pos2 = posStr.find_first_of('/');
        is2.str(posStr.substr(0, pos2));
        is2>>m_initPosition[1];
        posStr = posStr.substr(pos2+1);
        pos2 = posStr.find_first_of(';');
        is3.str(posStr.substr(0, pos2));
        is3>>m_initPosition[2];
    }
}

HeadTrackingVRPNHydra::~HeadTrackingVRPNHydra()
{

}

void HeadTrackingVRPNHydra::process(InputEvent& event, 
                        std::vector<Event>& outputAudioEvents, 
                        std::vector<Event>& outputEnvEvents) {
    //values: x/y/z/quat(x,y,z,w)
    outputEnvEvents.push_back(Event());
    Event& eEv = outputEnvEvents.back();
    eEv.type=EVENT_CAMERA;
    eEv.parameter=CAMERA_HEAD_POSQUAT;
    eEv.camera = m_id;

    eEv.values[0]=event.values[0]+m_initPosition[0];
    eEv.values[1]=-event.values[2]+m_initPosition[1];
    eEv.values[2]=event.values[1]+m_initPosition[2];

}

