/***************************************************************************
 *  PiivertDevice.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "PiivertDevice.hpp"
#include "../../Drile.hpp"

#include "PiivertFingersMouse.hpp"
#include "PiivertTrackingMouse.hpp"
#include "PiivertFingersFSRMidi.hpp"
#include "PiivertTrackingVRPN.hpp"
#include "PiivertTrackingVRPNGameTrak.hpp"
#include "PiivertTrackingVRPNHydra.hpp"
#include "PiivertFingersVrpn.hpp"
#include "PiivertTrackingOscPos.hpp"
#include "PiivertVibroOsc.hpp"
#ifdef WITH_SERIAL
#include "PiivertFingersSerial.hpp"
#endif

using namespace std;

PiivertDevice::PiivertDevice(const int& id): Device(id), m_lastGesture(0), 
                                        PIIVERT_GESTURE_TIME_THRESHOLD(100000), 
                                        m_lastGestureDurationParameter(1) {
    ostringstream oss;
    oss<<m_id;
    m_name = "Piivert"+oss.str();

    gettimeofday(&m_lastGestureTime,NULL);
}

PiivertDevice::~PiivertDevice() {}

void PiivertDevice::parse(xmlNodePtr node) {
    xmlNodePtr piivSubNode;
    for (piivSubNode = node->children; piivSubNode; piivSubNode = piivSubNode->next) {
        if (piivSubNode->type == XML_ELEMENT_NODE && !xmlStrcmp(piivSubNode->name, (const xmlChar *) "Input")) {
            char* value=NULL;
            value = (char*)xmlGetProp(piivSubNode,(xmlChar*)"type");
            if(value!=NULL) {
                std::string typeStr = value;

                std::string leftHandControls, rightHandControls;
                value = NULL;
                value = (char*)xmlGetProp(piivSubNode,(xmlChar*)"right_hand");
                if(value!=NULL) {
                    rightHandControls=value;
                }
                value = NULL;
                value = (char*)xmlGetProp(piivSubNode,(xmlChar*)"left_hand");
                if(value!=NULL) {
                    leftHandControls=value;
                }
                
                //create new input according to type
                if(typeStr.compare("fingers_interfacez_midi")==0) {
                    m_mappings[RIGHT_HAND].push_back(
                            new PiivertFingersFSRMidi(m_id, 
                                rightHandControls, RIGHT_HAND, this));
                    m_mappings[LEFT_HAND].push_back(
                            new PiivertFingersFSRMidi(m_id, 
                                leftHandControls, LEFT_HAND, this));
                }
                else if(typeStr.compare("tracking_vrpn")==0) {
                    m_mappings[RIGHT_HAND].push_back(new PiivertTrackingVRPN(m_id, rightHandControls, RIGHT_HAND));
                    m_mappings[LEFT_HAND].push_back(new PiivertTrackingVRPN(m_id, leftHandControls, LEFT_HAND));
                }
                else if(typeStr.compare("tracking_vrpn_gametrak")==0) {
                    m_mappings[RIGHT_HAND].push_back(new PiivertTrackingVRPNGameTrak(m_id, rightHandControls, RIGHT_HAND));
                    m_mappings[LEFT_HAND].push_back(new PiivertTrackingVRPNGameTrak(m_id, leftHandControls, LEFT_HAND));
                }
                else if(typeStr.compare("tracking_vrpn_hydra")==0) {
                    m_mappings[RIGHT_HAND].push_back(new PiivertTrackingVRPNHydra(m_id, rightHandControls, RIGHT_HAND));
                    m_mappings[LEFT_HAND].push_back(new PiivertTrackingVRPNHydra(m_id, leftHandControls, LEFT_HAND));
                }
                else if(typeStr.compare("tracking_osc_pos")==0) {
                    m_mappings[RIGHT_HAND].push_back(new PiivertTrackingOscPos(m_id, rightHandControls, RIGHT_HAND));
                    m_mappings[LEFT_HAND].push_back(new PiivertTrackingOscPos(m_id, leftHandControls, LEFT_HAND));
                }
                else if(typeStr.compare("fingers_mice")==0) {
                    m_mappings[RIGHT_HAND].push_back(
                        new PiivertFingersMouse(m_id, rightHandControls, 
                            RIGHT_HAND, this));

                    m_mappings[LEFT_HAND].push_back(
                        new PiivertFingersMouse(m_id, 
                            leftHandControls, LEFT_HAND, this));
                }
                else if(typeStr.compare("tracking_mice")==0) {
                    m_mappings[RIGHT_HAND].push_back(
                        new PiivertTrackingMouse(m_id, rightHandControls, 
                            RIGHT_HAND));
                    m_mappings[LEFT_HAND].push_back(
                        new PiivertTrackingMouse(m_id, leftHandControls, 
                        LEFT_HAND));
                }
                #ifdef WITH_SERIAL
                else if(typeStr.compare("fingers_serial")==0) {
                    m_mappings[RIGHT_HAND].push_back(
                        new PiivertFingersSerial(m_id, rightHandControls, 
                            RIGHT_HAND, this));
                    m_mappings[LEFT_HAND].push_back(
                        new PiivertFingersSerial(m_id, leftHandControls, 
                            LEFT_HAND, this));
                }
                #endif
                else if(typeStr.compare("fingers_vrpn")==0) {
                    m_mappings[RIGHT_HAND].push_back(
                        new PiivertFingersVrpn(m_id, rightHandControls, 
                            RIGHT_HAND, this));
                    m_mappings[LEFT_HAND].push_back(
                        new PiivertFingersVrpn(m_id, leftHandControls, 
                            LEFT_HAND, this));
                }
            }
        }
        else if (piivSubNode->type == XML_ELEMENT_NODE 
                && !xmlStrcmp(piivSubNode->name, (const xmlChar *) "Output")) {
            char* value=NULL;
            value = (char*)xmlGetProp(piivSubNode,(xmlChar*)"type");
            if(value!=NULL) {
                std::string typeStr = value;
                std::string param;

                value = NULL;
                value = (char*)xmlGetProp(piivSubNode,(xmlChar*)"param");
                if(value!=NULL) {
                    param=value;
                }

                if(typeStr.compare("vibro_osc")==0) {
                    Mapping* newOutputMapping 
                                = new PiivertVibroOsc(m_id, param);
                    m_outputMappings.push_back(newOutputMapping);

                    
                }
            }
        }
    }

}

void PiivertDevice::getMappingsIDs(std::vector<MappingID>& mappingsIDs) {
    for(unsigned int i=0;i<m_mappings[LEFT_HAND].size();++i) {
        m_mappings[LEFT_HAND][i]->getMappingsIDs(mappingsIDs);
        m_mappings[RIGHT_HAND][i]->getMappingsIDs(mappingsIDs);
    }
}

void PiivertDevice::setSelectedWorms(const HAND& hand, double* values) {
    //set input mappings
    for(unsigned int i=0; i<m_mappings[hand].size(); ++i) {
        m_mappings[hand][i]->setSelectedWorms(values);
    }
}

void PiivertDevice::setSpeed(const HAND& hand, double* values) {
}

void PiivertDevice::setGrab(const HAND& hand, const double& press) {
    //send to outputMappings
    OutputEvent ev;
    ev.type=OUTPUT_PIIVERT;
    ev.parameter=PIIVERT_GRAB;
    ev.hand=hand;
    ev.strId=m_name;
    ev.id=m_id;
    ev.values[0]=press;
    for(unsigned int i=0; i<m_outputMappings.size(); ++i) {
        m_outputMappings[i]->processOutput(ev);
    }
}

void PiivertDevice::processGesture(int& gesture) {
    timeval curTime, timeDiff;
    gettimeofday(&curTime, NULL);
    timersub(&curTime, &m_lastGestureTime, &timeDiff);
    //if same gesture, and no roll for now
    if(timeDiff.tv_sec==0 && timeDiff.tv_usec<PIIVERT_GESTURE_TIME_THRESHOLD 
                            && m_lastGesture<100) { //FIXME no roll for now
        //redefine current gesture
        gesture=m_lastGesture*100+gesture;
        //store last gesture duration
        m_lastGestureDurationParameter = 
            double(timeDiff.tv_usec)/double(PIIVERT_GESTURE_TIME_THRESHOLD); 
    }
    m_lastGesture=gesture;
    m_lastGestureTime=curTime;
}

void PiivertDevice::startRecordingBothHands(std::vector<Event>& audioEvents, 
                                            std::vector<Event>& envEvents) {
    timeval recordingStartTime;
    gettimeofday(&recordingStartTime, NULL);
    for(int h=0; h<2; ++h) {
        for(unsigned int m=0; m<m_mappings[h].size(); ++m) {
            m_mappings[h][m]->startRecordingSelectedWorms(recordingStartTime, 
                                                    audioEvents, envEvents);    
        }
    }
}

void PiivertDevice::stopRecordingBothHands(std::vector<Event>& audioEvents,
                                             std::vector<Event>& envEvents) {
    timeval recordingStopTime;
    gettimeofday(&recordingStopTime, NULL);
    for(int h=0; h<2; ++h) {
        for(unsigned int m=0; m<m_mappings[h].size(); ++m) {
            m_mappings[h][m]->stopRecordingWorms(recordingStopTime, 
                                                    audioEvents, envEvents);
        }
    }
}

void PiivertDevice::cancelRecordingBothHands(std::vector<Event>& audioEvents, 
                                            std::vector<Event>& envEvents)
{
    for(int h=0; h<2; ++h) {
        for(unsigned int m=0; m<m_mappings[h].size(); ++m) {
            m_mappings[h][m]->cancelRecordingWorms(audioEvents, envEvents);    
        }
    }
}

void PiivertDevice::eraseRecordingBothHands()
{
    for(int h=0; h<2; ++h) {
        for(unsigned int m=0; m<m_mappings[h].size(); ++m) {
            m_mappings[h][m]->eraseRecordingWorms();
        }
    }
}

void PiivertDevice::releaseBothHands(std::vector<Event>& audioEvents, 
                                            std::vector<Event>& envEvents) {
    for(int h=0; h<2; ++h) {
        for(unsigned int m=0; m<m_mappings[h].size(); ++m) {
            m_mappings[h][m]->release(audioEvents, envEvents);    
        }
    }
}
