/***************************************************************************
 *  PiivertDevice.hpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _PIIVERT_DEVICE_H
#define _PIIVERT_DEVICE_H

#include <iostream>
#include <cstring>
#include <cmath>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <vector>
#include <map>
#include <libxml/parser.h>
#include <libxml/tree.h>


#include "../Device.hpp"

class PiivertDevice: public Device {
    public:
        PiivertDevice(const int&);
        ~PiivertDevice();
        void parse(xmlNodePtr);

        void getMappingsIDs(std::vector<MappingID>&);

        void setSelectedWorms(const HAND&, double*);
        void setGrab(const HAND&, const double&);
        void setSpeed(const HAND&, double*);;

        void processGesture(int&);
        inline double getLastGestureDurationParameter(){
                        return m_lastGestureDurationParameter;}

        void startRecordingBothHands(std::vector<Event>&, std::vector<Event>&);
        void stopRecordingBothHands(std::vector<Event>&, std::vector<Event>&);
        void cancelRecordingBothHands(std::vector<Event>&, std::vector<Event>&);
        void eraseRecordingBothHands();
        void releaseBothHands(std::vector<Event>&, std::vector<Event>&);
    
    private:
        timeval m_lastGestureTime;
        int m_lastGesture;
        int PIIVERT_GESTURE_TIME_THRESHOLD;

        double m_lastGestureDurationParameter;
};



#endif
