/***************************************************************************
 *  PiivertFingersFSRMidi.hpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _PIIVERT_FINGERS_FSR_MIDI_H
#define _PIIVERT_FINGERS_FSR_MIDI_H

#include <iostream>
#include <cstring>
#include <cmath>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <vector>
#include <map>
#include <libxml/parser.h>
#include <libxml/tree.h>

#include "../Mapping.hpp"

class PiivertDevice;

class PiivertFingersFSRMidi: public Mapping
{
    public:
        PiivertFingersFSRMidi(const int&, const std::string&, 
                                const HAND&, PiivertDevice*);
        ~PiivertFingersFSRMidi();

        void process(InputEvent&, std::vector<Event>&, 
                        std::vector<Event>&);
        void update(std::vector<Event>&, std::vector<Event>&);

    private:
        void parseFinger(std::string&, const int&);

    private:
        PiivertDevice* m_piivertDevice;
    
        std::map<int, int> m_fingers;

        int m_buttonSensorOffset, m_ccSensorOffset;

        std::vector<int> m_firstSensorValues;
        std::vector<int> m_fingersLastValues;
        std::vector<timeval> m_fingersLastTimes;
        std::vector<std::vector<float> > m_fingerValues;

        int m_hitValueThreshold;;
        int m_hitTimeThreshold;
        int m_releaseTimeThreshold;
        int m_releaseValueThreshold;

        enum FINGER_STATE {FINGER_RELEASED, FINGER_PRESSING, FINGER_PRESSED};
        std::vector<FINGER_STATE> m_fingersStates;
        std::vector<int> m_fingersPressMax;


};



#endif
