/***************************************************************************
 *  PiivertFingersMouse.cpp
 *  Part of Drile
 *  2010- Florent Berthaut
 *  hitmuri.net 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "PiivertFingersMouse.hpp"
#include "../../env/Piivert.hpp"
#include "PiivertDevice.hpp"

#include "../../input/MiceInput.hpp"

using namespace std;

PiivertFingersMouse::PiivertFingersMouse(const int& id, 
                                        const std::string& param, 
                                        const HAND& hand, 
                                        PiivertDevice* piivertDevice): 
                                        Mapping(id, hand), 
                                        m_piivertDevice(piivertDevice), 
                                        m_hitVelocity(1.0), 
                                        m_slipGrabRelease(0), 
                                        m_middleReleased(true) {
    istringstream is;
    is.str(param);
    int dev;
    is>>dev;
    m_ids.push_back(MappingID(INPUT_MICE, this, 100*dev+MOUSE_BUTTONS));
    m_ids.push_back(MappingID(INPUT_MICE, this, 100*dev+MOUSE_BUTTONS+1));
    m_ids.push_back(MappingID(INPUT_MICE, this, 100*dev+MOUSE_BUTTONS+2));
}

PiivertFingersMouse::~PiivertFingersMouse()
{

}

void PiivertFingersMouse::process(InputEvent& event, 
                                    std::vector<Event>& outputAudioEvents, 
                                    std::vector<Event>& outputEnvEvents) {
    if(m_selectedWormsHaveChanged) {
        m_selectedWorms = m_newSelectedWorms;
        m_selectedWormsHaveChanged=false;
        if(m_selectedWorms[1]>=0) {
            //start recording
            if(m_loopState==LOOP_RECORDING) {
                startRecordingWorm(m_selectedWorms[1], outputAudioEvents, 
                                    outputEnvEvents);
            }
        }
    }

    //finger: 11= left hand index, 12=left hand middle, 21=right hand index ...
    int finger = (m_hand+1)*10 + event.parameter+1;
    if(event.values[0]==1) {

    //test gestures
    m_piivertDevice->processGesture(finger);

    //send a Hit to audio if index and a grab to Env if middle finger
    switch(finger) {
        case 1514 :
        case 2425: //flam reverse one hand
        case 1525 :
        case 2414:{ //flam reverse two hands
            if(m_loopState==LOOP_RECORDING) {
                m_piivertDevice->cancelRecordingBothHands(outputAudioEvents, 
                                                            outputEnvEvents);
                m_piivertDevice->eraseRecordingBothHands();
                cout<<"cancel"<<endl;
            }
            else {
                if(m_selectedWorms[1]>=0) {
                    outputEnvEvents.push_back(Event());
                    Event &eEv = outputEnvEvents[outputEnvEvents.size()-1];
                    eEv.type=EVENT_TREE;
                    eEv.parameter=TREE_EXTRACT;
                    eEv.node=m_selectedWorms[1];
                    eEv.values[0]=m_selectedWorms[1];
                    eEv.time=m_recordingStartTime;    

                    outputAudioEvents.push_back(Event());
                    Event &aEv = outputAudioEvents.back();
                    aEv.type=EVENT_TREE;
                    aEv.parameter=TREE_EXTRACT;
                    aEv.node=m_selectedWorms[1];
                    aEv.values[0]=m_selectedWorms[1];
                    aEv.time=m_recordingStartTime;    
                    cout<<"sent extract for node "<<eEv.values[0]<<" from fingers mouse"<<endl;
                }
            }
            outputEnvEvents.push_back(Event());
            Event &eEv1 = outputEnvEvents.back();
            eEv1.type=EVENT_PIIVERT;
            eEv1.parameter=PIIVERT_RECORD_TWO_HANDS;
            eEv1.piivert=m_id;
            eEv1.hand=m_hand;
            eEv1.values[0]=0;

            //tunnel store
            /*
            outputEnvEvents.push_back(EnvEvent());
            EnvEvent& eEv = outputEnvEvents[outputEnvEvents.size()-1];
            eEv.type=ENV_PIIVERT;
            eEv.parameter=PIIVERT_STORE;
            eEv.piivert=m_id;
            eEv.hand=m_hand;
            eEv.finger=event.parameter;
            eEv.values.push_back(0.5);
            */
            //slipping
            /*
            outputEnvEvents.push_back(EnvEvent());
            EnvEvent& eEv2 = outputEnvEvents[outputEnvEvents.size()-1];
            eEv2.type=ENV_PIIVERT;
            eEv2.parameter=PIIVERT_GRAB;
            eEv2.piivert=m_id;
            eEv2.hand=m_hand;
            eEv2.finger=event.parameter;
            eEv2.values[0]=1;
            */
        }break;
        case 1415 :
        case 2524:{ //record one hand 
            if(m_loopState==LOOP_RECORDING) {
                timeval recordingStopTime;
                gettimeofday(&recordingStopTime, NULL);
                stopRecordingWorms(recordingStopTime, outputAudioEvents, 
                                        outputEnvEvents);
                outputEnvEvents.push_back(Event());
                Event &eEv1 = outputEnvEvents.back();
                eEv1.type=EVENT_PIIVERT;
                eEv1.parameter=PIIVERT_RECORD_ONE_HAND;
                eEv1.piivert=m_id;
                eEv1.hand=m_hand;
                eEv1.values[0]=0;
            }
            else { 
                if(m_loopState==LOOP_NONE) {
                    timeval recordingStartTime;
                    gettimeofday(&recordingStartTime, NULL);
                    startRecordingSelectedWorms(recordingStartTime, 
                                        outputAudioEvents, outputEnvEvents);
                    outputEnvEvents.push_back(Event());
                    Event &eEv1 = outputEnvEvents.back();
                    eEv1.type=EVENT_PIIVERT;
                    eEv1.parameter=PIIVERT_RECORD_ONE_HAND;
                    eEv1.piivert=m_id;
                    eEv1.hand=m_hand;
                    eEv1.values[0]=1;
                }
                if(m_selectedWorms[1]>=0) {
                    //excite
                    outputAudioEvents.push_back(Event());
                    Event &aEv = outputAudioEvents.back();
                    aEv.type=EVENT_NODE;
                    aEv.parameter=NODE_EXCITE_INST;
                    aEv.node=m_selectedWorms[1];
                    aEv.values[0]=m_hitVelocity;

                    outputAudioEvents.push_back(Event());
                    Event &a2Ev = outputAudioEvents.back();
                    a2Ev.type=EVENT_NODE;
                    a2Ev.parameter=NODE_EXCITE_CONT;
                    a2Ev.node=m_selectedWorms[1];
                    a2Ev.values[0]=m_hitVelocity/2.0;
                }
            }
            outputEnvEvents.push_back(Event());
            Event &eEv = outputEnvEvents.back();
            eEv.type=EVENT_PIIVERT;
            eEv.parameter=PIIVERT_HIT;
            eEv.piivert=m_id;
            eEv.hand=m_hand;
            eEv.finger=event.parameter;
            eEv.values[0]=m_hitVelocity;
        }break;
        case 2515 :
        case 1424:{ //flam two hands
            //record
            switch(m_loopState) {
                case LOOP_NONE: {
                    m_piivertDevice->startRecordingBothHands(outputAudioEvents,
                                                             outputEnvEvents);
                    outputEnvEvents.push_back(Event());
                    Event &eEv1 = outputEnvEvents.back();
                    eEv1.type=EVENT_PIIVERT;
                    eEv1.parameter=PIIVERT_RECORD_TWO_HANDS;
                    eEv1.piivert=m_id;
                    eEv1.hand=m_hand;
                    eEv1.values[0]=1;
                    //also send a hit
                    if(m_selectedWorms[1]>=0) {
                        outputAudioEvents.push_back(Event());
                        Event &aEv = outputAudioEvents.back();
                        aEv.type=EVENT_NODE;
                        aEv.parameter=NODE_EXCITE_INST;
                        aEv.node=m_selectedWorms[1];
                        aEv.values[0]=m_hitVelocity;

                        outputAudioEvents.push_back(Event());
                        Event &a2Ev = outputAudioEvents.back();
                        a2Ev.type=EVENT_NODE;
                        a2Ev.parameter=NODE_EXCITE_CONT;
                        a2Ev.node=m_selectedWorms[1];
                        a2Ev.values[0]=m_hitVelocity/2.0;
                    }
                    outputEnvEvents.push_back(Event());
                    Event &eEv = outputEnvEvents.back();
                    eEv.type=EVENT_PIIVERT;
                    eEv.parameter=PIIVERT_HIT;
                    eEv.piivert=m_id;
                    eEv.hand=m_hand;
                    eEv.finger=event.parameter;
                    eEv.values[0]=m_hitVelocity;
                }break;
                case LOOP_RECORDING: {
                    m_piivertDevice->stopRecordingBothHands(outputAudioEvents,
                                                            outputEnvEvents);
                    m_piivertDevice->eraseRecordingBothHands();
                    Event &eEv1 = outputEnvEvents.back();
                    eEv1.type=EVENT_PIIVERT;
                    eEv1.parameter=PIIVERT_RECORD_TWO_HANDS;
                    eEv1.piivert=m_id;
                    eEv1.hand=m_hand;
                    eEv1.values[0]=0;
                }break;
                default:break;
            }
        }break;
        case 1524 :
        case 2415 :
        case 15 :
        case 24:{
            //hit or flam hit hit 
            //play
            if(m_selectedWorms[1]>=0) {
                //start recording
                if(m_loopState==LOOP_RECORDING) {
                    startRecordingWorm(m_selectedWorms[1], outputAudioEvents, 
                                        outputEnvEvents);
                }
                //excite
                outputAudioEvents.push_back(Event());
                Event &aEv = outputAudioEvents.back();
                aEv.type=EVENT_NODE;
                aEv.parameter=NODE_EXCITE_INST;
                aEv.node=m_selectedWorms[1];
                aEv.values[0]=m_hitVelocity;

                outputAudioEvents.push_back(Event());
                Event &a2Ev = outputAudioEvents.back();
                a2Ev.type=EVENT_NODE;
                a2Ev.parameter=NODE_EXCITE_CONT;
                a2Ev.node=m_selectedWorms[1];
                a2Ev.values[0]=m_hitVelocity/2.0;
            }
            outputEnvEvents.push_back(Event());
            Event &eEv = outputEnvEvents.back();
            eEv.type=EVENT_PIIVERT;
            eEv.parameter=PIIVERT_HIT;
            eEv.piivert=m_id;
            eEv.hand=m_hand;
            eEv.finger=event.parameter;
            eEv.values[0]=m_hitVelocity;
        }break;
        case 1425 :
        case 2514 :{ //two hands grab/grab flam
            outputEnvEvents.push_back(Event());
            Event& eEv = outputEnvEvents.back();
            eEv.type=EVENT_PIIVERT;
            eEv.parameter=PIIVERT_STORE;
            eEv.piivert=m_id;
            eEv.hand=m_hand;
            eEv.finger=event.parameter;
            eEv.values[0]=1;
        }break;
        case 14:
        case 25:{ //grab
            if(m_selectedWorms[1]>=0) {
                if(m_loopState==LOOP_RECORDING) {
                    startRecordingWorm(m_selectedWorms[1], outputAudioEvents, 
                                        outputEnvEvents);
                }
                outputAudioEvents.push_back(Event());
                Event &aEv = outputAudioEvents.back();
                aEv.type=EVENT_NODE;
                aEv.parameter=NODE_STOP;
                aEv.node=m_selectedWorms[1];
                aEv.values[0]=1;

            }
            outputEnvEvents.push_back(Event());
            Event& eEv = outputEnvEvents.back();
            eEv.type=EVENT_PIIVERT;
            eEv.parameter=PIIVERT_GRAB;
            eEv.piivert=m_id;
            eEv.hand=m_hand;
            eEv.finger=event.parameter;
            eEv.values[0]=1;
        }break;
        case 26:
        case 16: { //middle button, grab and lock
            m_middleReleased=false;
            switch(m_slipGrabRelease) {
                case 0: {
                    outputEnvEvents.push_back(Event());
                    Event& eEv = outputEnvEvents.back();
                    eEv.type=EVENT_PIIVERT;
                    eEv.parameter=PIIVERT_GRAB;
                    eEv.piivert=m_id;
                    eEv.hand=m_hand;
                    eEv.finger=event.parameter;
                    eEv.values[0]=1;
                    m_slipGrabRelease=1;
                }break;
                case 1: {
                    outputEnvEvents.push_back(Event());
                    Event& eEv = outputEnvEvents.back();
                    eEv.type=EVENT_PIIVERT;
                    eEv.parameter=PIIVERT_GRAB;
                    eEv.piivert=m_id;
                    eEv.hand=m_hand;
                    eEv.finger=event.parameter;
                    eEv.values[0]=1;
                    m_slipGrabRelease=2;
                }break;
                case 2: {
                    outputEnvEvents.push_back(Event());
                    Event &eEv = outputEnvEvents.back();
                    eEv.type=EVENT_PIIVERT;
                    eEv.parameter=PIIVERT_GRAB;
                    eEv.piivert=m_id;
                    eEv.hand=m_hand;
                    eEv.finger=event.parameter;
                    eEv.values[0]=0;
                    m_slipGrabRelease=0;
                }break;
                default:break;
            }
        }break;
        default:break;
        }
    }
    else {
        //send release object to Env
        switch(finger) {
            case 14:
            case 25:{
                outputEnvEvents.push_back(Event());
                Event &eEv = outputEnvEvents.back();
                eEv.type=EVENT_PIIVERT;
                eEv.parameter=PIIVERT_GRAB;
                eEv.piivert=m_id;
                eEv.hand=m_hand;
                eEv.finger=event.parameter;
                eEv.values[0]=0;
            }break;
            case 24:
            case 15:{
                //excite release
                outputAudioEvents.push_back(Event());
                Event &aEv = outputAudioEvents.back();
                aEv.type=EVENT_NODE;
                aEv.parameter=NODE_EXCITE_CONT_RELEASE;
                aEv.node=m_selectedWorms[1];
                aEv.values[0]=m_hitVelocity;
            }
            case 26:
            case 16: {
                m_middleReleased=true;
            }break;
            default:break;
        }
    }

}

