/***************************************************************************
 *  PiivertFingersSerial.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef WITH_SERIAL

#include "PiivertFingersSerial.hpp"
#include "../../env/Piivert.hpp"
#include "PiivertDevice.hpp"
#include "../../input/InputManager.hpp"

using namespace std;

PiivertFingersSerial::PiivertFingersSerial(const int& id, 
                                            const std::string& param, 
                                            const HAND& hand, 
                                            PiivertDevice* piivertDevice): 
                                            Mapping(id, hand), 
                                            m_piivertDevice(piivertDevice),
                                            m_hitValueThreshold(100), 
                                            m_hitTimeThreshold(100), 
                                            m_releaseValueThreshold(90){
    std::string device = param;
    size_t pos=device.find_first_of(';');

    //get device
    std::string deviceStr = device.substr(0,pos);
    //get resolution and baud rate 
    device = device.substr(pos+1);
    pos=device.find_first_of('/');
    std::string resStr = device.substr(0,pos);
    std::istringstream is1;
    is1.str(resStr);
    is1>>m_resolution;
    device = device.substr(pos+1);

    pos=device.find_first_of(';');
    std::string baudStr = device.substr(0,pos);
    std::istringstream is2;
    is2.str(baudStr);
    int baudRate;
    is2>>baudRate;

    //get the device id from the input manager
    InputManager *inputManager = InputManager::getInstance();
    int serialID = inputManager->addSerialDevice(deviceStr, baudRate);
    cout<<"added serial device with id "<<serialID<<endl;

    timeval curTime;
    gettimeofday(&curTime,NULL);

    device = device.substr(pos+1);
    pos=device.find_first_of(';');
    int finger=0;
    //get sensors ids
    while(pos!=std::string::npos) {
        std::istringstream is3;
        std::string sensorStr = device.substr(0,pos);
        is3.str(sensorStr);
        int sensor; 
        is3>>sensor;
        sensor+=serialID*100;
        m_fingers[sensor]=finger;
        m_fingersStates.push_back(FINGER_RELEASED);
        m_fingersLastValues.push_back(0);
        m_fingersLastTimes.push_back(curTime);
        m_fingersPressMax.push_back(0);
        finger++;
        m_ids.push_back(MappingID(INPUT_SERIAL, this, sensor));
        device = device.substr(pos+1);
        pos=device.find_first_of(';');
    }
}

PiivertFingersSerial::~PiivertFingersSerial(){}

void PiivertFingersSerial::process(InputEvent& event, 
                                    std::vector<Event>& outputAudioEvents, 
                                    std::vector<Event>& outputEnvEvents) {

    if(m_selectedWormsHaveChanged) {
        //check all fingers for pressing state and send stop to corresponding 
        //selected nodes
        if(m_newSelectedWorms[1]!=m_selectedWorms[1] && m_selectedWorms[1]>=0) {
            for(unsigned int f=0;f<m_fingersStates.size();++f) {
                if(m_fingersStates[f]==FINGER_PRESSED) {
                    outputAudioEvents.push_back(Event());
                    Event &aEv = outputAudioEvents.back();
                    aEv.type=EVENT_NODE;
                    aEv.parameter=NODE_EXCITE_CONT_RELEASE;
                    aEv.node=m_selectedWorms[1];
                    aEv.finger=0;
                    aEv.values[0]=1;
                    outputAudioEvents.push_back(Event());
                    Event &aEv2 = outputAudioEvents.back();
                    aEv2.type=EVENT_NODE;
                    aEv2.parameter=NODE_EXCITE_CONT_RELEASE;
                    aEv2.finger=1;
                    aEv2.node=m_selectedWorms[1];
                    aEv2.values[0]=1;
                }
            }
        }
        m_selectedWorms = m_newSelectedWorms;
        m_selectedWormsHaveChanged=false;
    }


    //SENSORS
    bool gotHit=false;
    double hitValue=0;

    int finger=m_fingers[event.parameter];
    //cout<<"got id "<<event.id<<" finger "<<event.parameter<<" -> "<<finger<<endl;
    switch(finger) { 
        case 0: {//thumb , grab
            if(event.values[0]>0) {
                m_grabToggle=!m_grabToggle;
                outputEnvEvents.push_back(Event());
                Event& eEv = outputEnvEvents.back();
                eEv.type=EVENT_PIIVERT;
                eEv.parameter=PIIVERT_GRAB;
                eEv.piivert=m_id;
                eEv.hand=m_hand;
                eEv.finger=event.parameter;
                eEv.values[0]=m_grabToggle;
                if(m_loopState==LOOP_RECORDING && m_selectedWorms[1]>=0) {
                    startRecordingWorm(m_selectedWorms[1], 
                        outputAudioEvents, outputEnvEvents);
                }
            }
        }break;
        case 3: {  //ring, attenuate and record 
            if(event.values[0]>0) {
                //compute gesture
                gotHit=true;
                hitValue=1.0;
                if(m_selectedWorms[1]>=0) {
                    if(m_loopState==LOOP_RECORDING) {
                        startRecordingWorm(m_selectedWorms[1], 
                            outputAudioEvents, outputEnvEvents);
                    }

                    outputAudioEvents.push_back(Event());
                    Event &aEv = outputAudioEvents.back();
                    aEv.type=EVENT_NODE;
                    aEv.parameter=NODE_STOP;
                    aEv.node=m_selectedWorms[1];
                    aEv.values[0]=1;
                }
            }
        }break;
        default: { // index and middle , excite 
            int resultValue = 1023 - (int)(event.values[0]);
            //cout<<"resval "<<resultValue<<endl;
            resultValue=(resultValue < 0)?0:(resultValue>1023)?1023:resultValue;
            if(m_fingersStates[finger] == FINGER_RELEASED) {
                if(resultValue>0) {
                    m_fingersStates[finger]=FINGER_PRESSING;
                    gettimeofday(&m_fingersLastTimes[finger], NULL);
                    m_fingersPressMax[finger]=resultValue;
              //      cout<<"got more than 0"<<endl;
                }
            }
            if(m_fingersStates[finger] == FINGER_PRESSING) {
                timeval curTime;
                gettimeofday(&curTime,NULL);
                timeval timeDiff;
                timersub(&curTime,&m_fingersLastTimes[finger],&timeDiff);
                //if time is over for a hit
                if(timeDiff.tv_sec>0 || timeDiff.tv_usec>m_hitTimeThreshold) {
                //    cout<<"pressing time is up"<<endl;
                    hitValue = double(m_fingersPressMax[finger])/1023.0;
                    //if value high enough, hit
                    if(m_fingersPressMax[finger]>m_hitValueThreshold) {
                        //cout<<"hit "<<hitValue<<endl;
                        gotHit=true;
                        if(m_selectedWorms[1]>=0) {
                            outputAudioEvents.push_back(Event());
                            Event &aEv = outputAudioEvents.back();
                            aEv.type=EVENT_NODE;
                            aEv.parameter=NODE_EXCITE_INST;
                            aEv.node=m_selectedWorms[1];
                            aEv.finger=finger;
                            aEv.values[0]=hitValue;
                        }
                    } //else press
                    else {
                    //    cout<<"press"<<endl;
                        gotHit=true;
                        if(m_selectedWorms[1]>=0) {
                            outputAudioEvents.push_back(Event());
                            Event &aEv = outputAudioEvents.back();
                            aEv.type=EVENT_NODE;
                            aEv.parameter=NODE_EXCITE_CONT;
                            aEv.node=m_selectedWorms[1];
                            aEv.finger=finger;
                            aEv.values[0]=hitValue;
                        }
                    }
                    outputEnvEvents.push_back(Event());
                    Event &eEv = outputEnvEvents.back();
                    eEv.type=EVENT_PIIVERT;
                    eEv.parameter=PIIVERT_PRESS;
                    eEv.piivert=m_id;
                    eEv.hand=m_hand;
                    eEv.finger=finger;
                    eEv.values[0]=hitValue;
                    m_fingersStates[finger]=FINGER_PRESSED;
                }
                else if(resultValue>m_fingersPressMax[finger]) {
                    //cout<<"max = "<<resultValue<<endl;
                    m_fingersPressMax[finger]=resultValue;
                }
            }
            if(m_fingersStates[finger] == FINGER_PRESSED) {
                if(resultValue<m_releaseValueThreshold) {
                    hitValue = 0; 
                    m_fingersStates[finger]=FINGER_RELEASED;
                }
                else {
                    hitValue = double(resultValue)/1023.0;
                }
                if(m_selectedWorms[1]>=0) {
                    outputAudioEvents.push_back(Event());
                    Event &aEv = outputAudioEvents.back();
                    aEv.type=EVENT_NODE;
                    aEv.parameter=NODE_EXCITE_CONT;
                    aEv.node=m_selectedWorms[1];
                    aEv.finger=finger;
                    aEv.values[0]=hitValue;
                }
                outputEnvEvents.push_back(Event());
                Event &eEv = outputEnvEvents.back();
                eEv.type=EVENT_PIIVERT;
                eEv.parameter=PIIVERT_PRESS;
                eEv.piivert=m_id;
                eEv.hand=m_hand;
                eEv.finger=finger;
                eEv.values[0]=hitValue;
            }

            m_fingersLastValues[finger]=resultValue;
        }break;
    }




    //GESTURES
    if(gotHit) {
        //process gesture
        finger = (m_hand+1)*10 + finger;
        m_piivertDevice->processGesture(finger);

        //according to gesture, send audio and env events
        switch(finger) {
            case 1320:
            case 1321:
            case 2311:
            case 2310:{ //two hands flam 
                switch(m_loopState) {
                    case LOOP_NONE: {
                        m_piivertDevice->startRecordingBothHands(
                            outputAudioEvents, outputEnvEvents);
                        outputEnvEvents.push_back(Event());
                        Event &eEv1 = outputEnvEvents.back();
                        eEv1.type=EVENT_PIIVERT;
                        eEv1.parameter=PIIVERT_RECORD_TWO_HANDS;
                        eEv1.piivert=m_id;
                        eEv1.hand=m_hand;
                        eEv1.values[0]=1;
                    }break;
                    case LOOP_RECORDING: {
                        m_piivertDevice->stopRecordingBothHands(
                            outputAudioEvents, outputEnvEvents);
                        m_piivertDevice->eraseRecordingBothHands();
                        outputEnvEvents.push_back(Event());
                        Event &eEv1 = outputEnvEvents.back();
                        eEv1.type=EVENT_PIIVERT;
                        eEv1.parameter=PIIVERT_RECORD_TWO_HANDS;
                        eEv1.piivert=m_id;
                        eEv1.hand=m_hand;
                        eEv1.values[0]=0;
                        m_piivertDevice->releaseBothHands(
                            outputAudioEvents, outputEnvEvents);
                    }break;
                    default:break;
                }
                if(m_selectedWorms[1]>=0) {
                    //excite
                    outputAudioEvents.push_back(Event());
                    Event &aEv = outputAudioEvents.back();
                    aEv.type=EVENT_NODE;
                    aEv.parameter=NODE_EXCITE_INST;
                    aEv.node=m_selectedWorms[1];
                    aEv.finger=finger;
                    aEv.values[0]=hitValue;
                }
                outputEnvEvents.push_back(Event());
                Event &eEv = outputEnvEvents.back();
                eEv.type=EVENT_PIIVERT;
                eEv.parameter=PIIVERT_HIT;
                eEv.piivert=m_id;
                eEv.hand=m_hand;
                eEv.finger=event.parameter;
                eEv.values[0]=hitValue;
            }break;
            case 1023:
            case 1123:
            case 2013:
            case 2113:{ //two hands reverse flam 
                if(m_loopState==LOOP_RECORDING) {
                    m_piivertDevice->cancelRecordingBothHands(
                        outputAudioEvents, outputEnvEvents);
                    m_piivertDevice->eraseRecordingBothHands();
                }
                outputEnvEvents.push_back(Event());
                Event &eEv1 = outputEnvEvents.back();
                eEv1.type=EVENT_PIIVERT;
                eEv1.parameter=PIIVERT_RECORD_TWO_HANDS;
                eEv1.piivert=m_id;
                eEv1.hand=m_hand;
                eEv1.values[0]=0;
            }break;
            case 1310:
            case 1311:
            case 2321:
            case 2320:{ //one hand flam 
                switch(m_loopState) {
                    case LOOP_RECORDING: {
                        timeval recordingStopTime;
                        gettimeofday(&recordingStopTime, NULL);
                        stopRecordingWorms(recordingStopTime, 
                                    outputAudioEvents, outputEnvEvents);
                        outputEnvEvents.push_back(Event());
                        Event &eEv1 = outputEnvEvents.back();
                        eEv1.type=EVENT_PIIVERT;
                        eEv1.parameter=PIIVERT_RECORD_ONE_HAND;
                        eEv1.piivert=m_id;
                        eEv1.hand=m_hand;
                        eEv1.values[0]=0;

                        //release
            release(outputAudioEvents, outputEnvEvents);
                    }break;
                    case LOOP_NONE: {
                        timeval recordingStartTime;
                        gettimeofday(&recordingStartTime, NULL);
                        startRecordingSelectedWorms(recordingStartTime, 
                            outputAudioEvents, outputEnvEvents);
                        outputEnvEvents.push_back(Event());
                        Event &eEv1 = outputEnvEvents.back();
                        eEv1.type=EVENT_PIIVERT;
                        eEv1.parameter=PIIVERT_RECORD_ONE_HAND;
                        eEv1.piivert=m_id;
                        eEv1.hand=m_hand;
                        eEv1.values[0]=1;
                    }break;
                    default:break;
                }
                if(m_selectedWorms[1]>=0) {
                    //excite
                    outputAudioEvents.push_back(Event());
                    Event &aEv = outputAudioEvents.back();
                    aEv.type=EVENT_NODE;
                    aEv.parameter=NODE_EXCITE_INST;
                    aEv.node=m_selectedWorms[1];
                    aEv.finger=finger;
                    aEv.values[0]=hitValue;
                }
                outputEnvEvents.push_back(Event());
                Event &eEv = outputEnvEvents.back();
                eEv.type=EVENT_PIIVERT;
                eEv.parameter=PIIVERT_HIT;
                eEv.piivert=m_id;
                eEv.hand=m_hand;
                eEv.finger=event.parameter;
                eEv.values[0]=hitValue;
            }break;
            case 1013:
            case 1113:
            case 2023:
            case 2123:{ //one hand reverse flam 
                if(m_loopState==LOOP_RECORDING) {
                    cancelRecordingWorms(outputAudioEvents, outputEnvEvents);
                    eraseRecordingWorms();
                }
                else {
                    if(m_selectedWorms[1]>=0) {
                        outputEnvEvents.push_back(Event());
                        Event &eEv = outputEnvEvents.back();
                        eEv.type=EVENT_TREE;
                        eEv.parameter=TREE_EXTRACT;
                        eEv.node=m_selectedWorms[1];
                        eEv.values[0]=m_selectedWorms[1];
                        eEv.time=m_recordingStartTime;    

                        outputAudioEvents.push_back(Event());
                        Event &aEv = outputAudioEvents.back();
                        aEv.type=EVENT_TREE;
                        aEv.parameter=TREE_EXTRACT;
                        aEv.node=m_selectedWorms[1];
                        aEv.values[0]=m_selectedWorms[1];
                        aEv.time=m_recordingStartTime;    
                        //cout<<"sent extract for node "<<eEv.values[0]<<" from fingers mouse"<<endl;
                    }
                }
                outputEnvEvents.push_back(Event());
                Event &eEv1 = outputEnvEvents.back();
                eEv1.type=EVENT_PIIVERT;
                eEv1.parameter=PIIVERT_RECORD_ONE_HAND;
                eEv1.piivert=m_id;
                eEv1.hand=m_hand;
                eEv1.values[0]=0;

            }break;
            case 2021:
            case 2120:
            case 20:
            case 21:
            case 1011:
            case 1110:
            case 10:
            case 11: 
            case 2011:
            case 2110:
            case 1021:
            case 1120:{ //one finger hit
                if(m_selectedWorms[1]>=0) {
                    //start recording
                    if(m_loopState==LOOP_RECORDING) {
                        startRecordingWorm(m_selectedWorms[1], 
                            outputAudioEvents, outputEnvEvents);
                    }
                    //excite
                    outputAudioEvents.push_back(Event());
                    Event &aEv = outputAudioEvents.back();
                    aEv.type=EVENT_NODE;
                    aEv.parameter=NODE_EXCITE_INST;
                    aEv.finger=finger;
                    aEv.node=m_selectedWorms[1];
                    aEv.values[0]=hitValue;
                }
                outputEnvEvents.push_back(Event());
                Event &eEv = outputEnvEvents.back();
                eEv.type=EVENT_PIIVERT;
                eEv.parameter=PIIVERT_HIT;
                eEv.piivert=m_id;
                eEv.hand=m_hand;
                eEv.finger=event.parameter;
                eEv.values[0]=hitValue;
            }
            default:break;
        }
        

    }


}

#endif

