/***************************************************************************
 *  PiivertFingersVrpn.cpp
 *  Part of Drile
 *  2010- Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "PiivertFingersVrpn.hpp"
#include "../../env/Piivert.hpp"
#include "PiivertDevice.hpp"
#include "../../input/InputManager.hpp"

using namespace std;

PiivertFingersVrpn::PiivertFingersVrpn(const int& id, const std::string& param, 
                                        const HAND& hand, 
                                        PiivertDevice* piivertDevice): 
                                        Mapping(id, hand), 
                                        m_piivertDevice(piivertDevice), 
                                        m_hitVelocity(1.0), 
                                        m_slipGrabRelease(0), 
                                        m_middleReleased(true) {
    std::string device = param;
    size_t pos=device.find_first_of(';');

    //get adress
    std::string adressStr = device.substr(0,pos);
    InputManager *inputManager = InputManager::getInstance();
    int vrpnTrackerID = inputManager->addVrpnButton(adressStr);

    //get fingers ids 
    std::string fingerStr = device.substr(pos+1);
    pos=fingerStr.find_first_of(';');
    int finger=0;
    //get sensors ids
    while(pos!=std::string::npos) {
        std::istringstream is3;
        std::string sensorStr = fingerStr.substr(0,pos);
        is3.str(sensorStr);
        int sensor; 
        is3>>sensor;
        sensor+=vrpnTrackerID*100;
        m_fingers[sensor]=finger;
        finger++;
        m_ids.push_back(MappingID(INPUT_VRPN_BUTTON, this, sensor));
        fingerStr = fingerStr.substr(pos+1);
        pos=fingerStr.find_first_of(';');
    }
}

PiivertFingersVrpn::~PiivertFingersVrpn(){}

void PiivertFingersVrpn::process(InputEvent& event, 
                                std::vector<Event>& outputAudioEvents, 
                                std::vector<Event>& outputEnvEvents) {
    if(m_selectedWormsHaveChanged) {
        m_selectedWorms = m_newSelectedWorms;
        m_selectedWormsHaveChanged=false;
        if(m_selectedWorms[1]>=0) {
            //start recording
            if(m_loopState==LOOP_RECORDING) {
                startRecordingWorm(m_selectedWorms[1], outputAudioEvents, 
                                    outputEnvEvents);
            }
        }
    }

    //finger: 10= left hand thumb, 11=left hand index, 20=right hand thumb ...
    int finger = (m_hand+1)*10 + m_fingers[event.parameter];
    if(event.values[0]>0) {

    //test gestures
    m_piivertDevice->processGesture(finger);

    //send a Hit to audio if index and a grab to Env if middle finger
    switch(finger) {
        case 1113 :
        case 1213 :
        case 2123 :
        case 2223 : //flam reverse one hand
        case 1223 :
        case 1123 :
        case 2213 :
        case 2113:{ //flam reverse two hands
            if(m_loopState==LOOP_RECORDING) {
                m_piivertDevice->cancelRecordingBothHands(outputAudioEvents, 
                                                            outputEnvEvents);
                m_piivertDevice->eraseRecordingBothHands();
            }
            else {
                if(m_selectedWorms[1]>=0) {
                    outputEnvEvents.push_back(Event());
                    Event &eEv = outputEnvEvents[outputEnvEvents.size()-1];
                    eEv.type=EVENT_TREE;
                    eEv.parameter=TREE_EXTRACT;
                    eEv.node=m_selectedWorms[1];
                    eEv.values[0]=m_selectedWorms[1];
                    eEv.time=m_recordingStartTime;    

                    outputAudioEvents.push_back(Event());
                    Event &aEv = outputAudioEvents.back();
                    aEv.type=EVENT_TREE;
                    aEv.parameter=TREE_EXTRACT;
                    aEv.node=m_selectedWorms[1];
                    aEv.values[0]=m_selectedWorms[1];
                    aEv.time=m_recordingStartTime;    
                }
            }
            outputEnvEvents.push_back(Event());
            Event &eEv1 = outputEnvEvents.back();
            eEv1.type=EVENT_PIIVERT;
            eEv1.parameter=PIIVERT_RECORD_TWO_HANDS;
            eEv1.piivert=m_id;
            eEv1.hand=m_hand;
            eEv1.values[0]=0;

        }break;
        case 1312 :
        case 1311 :
        case 2321 :
        case 2322:{ //record one hand 
            if(m_loopState==LOOP_RECORDING) {
                timeval recordingStopTime;
                gettimeofday(&recordingStopTime, NULL);
                stopRecordingWorms(recordingStopTime, outputAudioEvents, 
                                        outputEnvEvents);
                outputEnvEvents.push_back(Event());
                Event &eEv1 = outputEnvEvents.back();
                eEv1.type=EVENT_PIIVERT;
                eEv1.parameter=PIIVERT_RECORD_ONE_HAND;
                eEv1.piivert=m_id;
                eEv1.hand=m_hand;
                eEv1.values[0]=0;
            }
            else { 
                if(m_loopState==LOOP_NONE) {
                    timeval recordingStartTime;
                    gettimeofday(&recordingStartTime, NULL);
                    startRecordingSelectedWorms(recordingStartTime, 
                                        outputAudioEvents, outputEnvEvents);
                    outputEnvEvents.push_back(Event());
                    Event &eEv1 = outputEnvEvents.back();
                    eEv1.type=EVENT_PIIVERT;
                    eEv1.parameter=PIIVERT_RECORD_ONE_HAND;
                    eEv1.piivert=m_id;
                    eEv1.hand=m_hand;
                    eEv1.values[0]=1;
                }
                if(m_selectedWorms[1]>=0) {
                    //excite
                    outputAudioEvents.push_back(Event());
                    Event &aEv = outputAudioEvents.back();
                    aEv.type=EVENT_NODE;
                    aEv.parameter=NODE_EXCITE_INST;
                    aEv.node=m_selectedWorms[1];
                    aEv.values[0]=m_hitVelocity;

                    outputAudioEvents.push_back(Event());
                    Event &a2Ev = outputAudioEvents.back();
                    a2Ev.type=EVENT_NODE;
                    a2Ev.parameter=NODE_EXCITE_CONT;
                    a2Ev.node=m_selectedWorms[1];
                    a2Ev.values[0]=m_hitVelocity/2.0;
                }
            }
            outputEnvEvents.push_back(Event());
            Event &eEv = outputEnvEvents.back();
            eEv.type=EVENT_PIIVERT;
            eEv.parameter=PIIVERT_HIT;
            eEv.piivert=m_id;
            eEv.hand=m_hand;
            eEv.finger=event.parameter;
            eEv.values[0]=m_hitVelocity;
        }break;
        case 2312 :
        case 2311 :
        case 1321 :
        case 1322 :{ //flam two hands
            //record
            switch(m_loopState) {
                case LOOP_NONE: {
                    m_piivertDevice->startRecordingBothHands(outputAudioEvents,
                                                             outputEnvEvents);
                    outputEnvEvents.push_back(Event());
                    Event &eEv1 = outputEnvEvents.back();
                    eEv1.type=EVENT_PIIVERT;
                    eEv1.parameter=PIIVERT_RECORD_TWO_HANDS;
                    eEv1.piivert=m_id;
                    eEv1.hand=m_hand;
                    eEv1.values[0]=1;
                    //also send a hit
                    if(m_selectedWorms[1]>=0) {
                        outputAudioEvents.push_back(Event());
                        Event &aEv = outputAudioEvents.back();
                        aEv.type=EVENT_NODE;
                        aEv.parameter=NODE_EXCITE_INST;
                        aEv.node=m_selectedWorms[1];
                        aEv.values[0]=m_hitVelocity;

                        outputAudioEvents.push_back(Event());
                        Event &a2Ev = outputAudioEvents.back();
                        a2Ev.type=EVENT_NODE;
                        a2Ev.parameter=NODE_EXCITE_CONT;
                        a2Ev.node=m_selectedWorms[1];
                        a2Ev.values[0]=m_hitVelocity/2.0;
                    }
                    outputEnvEvents.push_back(Event());
                    Event &eEv = outputEnvEvents.back();
                    eEv.type=EVENT_PIIVERT;
                    eEv.parameter=PIIVERT_HIT;
                    eEv.piivert=m_id;
                    eEv.hand=m_hand;
                    eEv.finger=event.parameter;
                    eEv.values[0]=m_hitVelocity;
                }break;
                case LOOP_RECORDING: {
                    m_piivertDevice->stopRecordingBothHands(outputAudioEvents,
                                                            outputEnvEvents);
                    m_piivertDevice->eraseRecordingBothHands();
                    Event &eEv1 = outputEnvEvents.back();
                    eEv1.type=EVENT_PIIVERT;
                    eEv1.parameter=PIIVERT_RECORD_TWO_HANDS;
                    eEv1.piivert=m_id;
                    eEv1.hand=m_hand;
                    eEv1.values[0]=0;
                }break;
                default:break;
            }
        }break;
        case 1121 :
        case 1221 :
        case 1122 :
        case 1222 :
        case 2111 :
        case 2211 :
        case 2112 :
        case 2212 :
        case 11 :
        case 12 :
        case 21 :
        case 22:{
            //hit or flam hit hit 
            //play
            if(m_selectedWorms[1]>=0) {
                //start recording
                if(m_loopState==LOOP_RECORDING) {
                    startRecordingWorm(m_selectedWorms[1], outputAudioEvents, 
                                        outputEnvEvents);
                }
                //excite
                outputAudioEvents.push_back(Event());
                Event &aEv = outputAudioEvents.back();
                aEv.type=EVENT_NODE;
                aEv.parameter=NODE_EXCITE_INST;
                aEv.node=m_selectedWorms[1];
                aEv.values[0]=m_hitVelocity;

                outputAudioEvents.push_back(Event());
                Event &a2Ev = outputAudioEvents.back();
                a2Ev.type=EVENT_NODE;
                a2Ev.parameter=NODE_EXCITE_CONT;
                a2Ev.node=m_selectedWorms[1];
                a2Ev.values[0]=m_hitVelocity/2.0;
            }
            outputEnvEvents.push_back(Event());
            Event &eEv = outputEnvEvents.back();
            eEv.type=EVENT_PIIVERT;
            eEv.parameter=PIIVERT_HIT;
            eEv.piivert=m_id;
            eEv.hand=m_hand;
            eEv.finger=event.parameter;
            eEv.values[0]=m_hitVelocity;
        }break;
        case 10:
        case 20:{ //grab
            m_grabToggle=!m_grabToggle;
            if(m_selectedWorms[1]>=0) {
                if(m_loopState==LOOP_RECORDING) {
                    startRecordingWorm(m_selectedWorms[1], outputAudioEvents, 
                                        outputEnvEvents);
                }
            }
            outputEnvEvents.push_back(Event());
            Event& eEv = outputEnvEvents.back();
            eEv.type=EVENT_PIIVERT;
            eEv.parameter=PIIVERT_GRAB;
            eEv.piivert=m_id;
            eEv.hand=m_hand;
            eEv.finger=event.parameter;
            eEv.values[0]=m_grabToggle;
        }break;
        case 13:
        case 23:{ //mute
            if(m_selectedWorms[1]>=0) {
                if(m_loopState==LOOP_RECORDING) {
                    startRecordingWorm(m_selectedWorms[1], outputAudioEvents, 
                                        outputEnvEvents);
                }
                outputAudioEvents.push_back(Event());
                Event &aEv = outputAudioEvents.back();
                aEv.type=EVENT_NODE;
                aEv.parameter=NODE_STOP;
                aEv.node=m_selectedWorms[1];
                aEv.values[0]=m_grabToggle;
            }
        }break;
        default:break;
        }
    }
    else {
        //send release object to Env
        switch(finger) {
            case 12:
            case 11:
            case 21:
            case 22:{
                //excite release
                outputAudioEvents.push_back(Event());
                Event &aEv = outputAudioEvents.back();
                aEv.type=EVENT_NODE;
                aEv.parameter=NODE_EXCITE_CONT_RELEASE;
                aEv.node=m_selectedWorms[1];
                aEv.values[0]=m_hitVelocity;
            }
            default:break;
        }
    }

}

