/***************************************************************************
 *  PiivertTrackingMouse.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "PiivertTrackingMouse.hpp"
#include "../../env/Piivert.hpp"

using namespace std;

PiivertTrackingMouse::PiivertTrackingMouse(const int& id, 
                                const std::string& param, const HAND& hand): 
                                Mapping(id, hand), m_posX(0.2), m_posY(1.0), 
                                m_posZ(1), m_mouseSpeed(0.01), m_maxPosX(1),
                                m_maxPosY(2), m_maxPosZ(2), m_minPosX(-1),
                                m_minPosY(0), m_minPosZ(0){

    istringstream is;
    is.str(param);
    int dev;
    is>>dev;
    m_ids.push_back(MappingID(INPUT_MICE, this, 100*dev+MOUSE_X));
    m_ids.push_back(MappingID(INPUT_MICE, this, 100*dev+MOUSE_Y));
    m_ids.push_back(MappingID(INPUT_MICE, this, 100*dev+MOUSE_SCROLL));
}

PiivertTrackingMouse::~PiivertTrackingMouse()
{

}

void PiivertTrackingMouse::process(InputEvent& event, 
                                    std::vector<Event>& outputAudioEvents, 
                                    std::vector<Event>& outputEnvEvents) {
    switch(event.parameter) {
        case MOUSE_X: {
            if(m_posX+(event.values[0]*m_mouseSpeed)<=m_maxPosX && m_posX+(event.values[0]*m_mouseSpeed)>=m_minPosX) {
                m_posX+=event.values[0]*m_mouseSpeed;
            }
        }break;
        case MOUSE_Y: {
            if(m_posY-(event.values[0]*m_mouseSpeed)<=m_maxPosY && m_posY-(event.values[0]*m_mouseSpeed)>=m_minPosY) {
                m_posY-=event.values[0]*m_mouseSpeed;
            }
        }break;
        case MOUSE_SCROLL: {
            if(m_posZ-(event.values[0]*m_mouseSpeed)<=m_maxPosZ && m_posZ-(event.values[0]*m_mouseSpeed)>=m_minPosZ) {
                m_posZ-=event.values[0]*m_mouseSpeed;
            }

        }break;
    }

    outputEnvEvents.push_back(Event());
    Event &eEv = outputEnvEvents.back();
    eEv.type=EVENT_PIIVERT;
    eEv.parameter=PIIVERT_POSQUAT;
    eEv.piivert = m_id;
    eEv.hand=m_hand;
    eEv.values[0]=m_posX;
    eEv.values[1]=m_posY;
    eEv.values[2]=m_posZ;
    eEv.values[3]=0;
    eEv.values[4]=0;
    eEv.values[5]=0;
    eEv.values[6]=0;
}

