/***************************************************************************
 *            PiivertVibroOsc.cpp
 *            Part of Drile
 *
 *  2010  Tardigrade Inc.
 *  http://tardigrade-inc.com 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "PiivertVibroOsc.hpp"
#include "../../env/Piivert.hpp"
#include "PiivertDevice.hpp"

using namespace std;

PiivertVibroOsc::PiivertVibroOsc(const int& id, const std::string& params): Mapping(id), m_adress("127.0.0.1"), m_port(7000)
{
    size_t pos = params.find_first_of(";");
    if(pos!=string::npos) {
        istringstream is;
        m_adress=params.substr(0,pos);
        is.str(params.substr(pos+1));
        is>>m_port;
    }
    m_socket = new UdpTransmitSocket( IpEndpointName( m_adress.c_str(), m_port ) );
}

PiivertVibroOsc::~PiivertVibroOsc()
{

}

void PiivertVibroOsc::process(InputEvent&, std::vector<Event>&, 
                                            std::vector<Event>&)
{

}

void PiivertVibroOsc::processOutput(const OutputEvent &ev)
{
    char buffer[IP_MTU_SIZE];
    osc::OutboundPacketStream p( buffer, IP_MTU_SIZE );
    std::string mess = "/drile/output/"+ev.strId+"/";

    if(HAND(ev.hand)==LEFT_HAND) {
        mess+="L/";
    }
    else {
        mess+="R/";
    }

    switch(PIIVERT_PARAMETER(ev.parameter)) {
        case PIIVERT_SELECT_WORM: {
            mess+="select/";
        }break;
        case PIIVERT_GRAB: {
            mess+="grab/";
        }break;
        case PIIVERT_SPEED: {
            mess+="speed/";
        }break;
        default:break;
    }


    p << osc::BeginMessage(mess.c_str());
    for(unsigned int i=0; i<ev.nbValues; ++i) {
        p<<ev.values[i];
    }
    
    p << osc::EndMessage;   
    m_socket->Send( p.Data(), p.Size() );
}

