/***************************************************************************
 *            Grid.cpp
 *            Part of Drile
 *
 *  2010  Tardigrade Inc.
 *  http://tardigrade-inc.com 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Grid.hpp"

using namespace std;

Grid::Grid(const int& id):m_id(id), m_nbUsers(0), m_grabbingToolID(-1), 
                            m_isGrabbedWithBothHands(false), m_zoneID(0), 
                            m_currentScale(0), m_nbElements(20) {
    m_position = OSG::Vec3f(0, 0, 0);
    {

        double gridElementsWidth = (Worm::WORM_SIZE * 5.0)/m_nbElements;

        m_grid = OSG::Node::create();
        std::ostringstream oss;
        oss<<m_id+1000;
        OSG::setName(m_grid, oss.str());
        m_gridTrans = OSG::ComponentTransform::create();
        m_gridTrans->setTranslation(m_position);
        m_grid->setCore(m_gridTrans);

        OSG::GeometryRefPtr geo = OSG::makeBoxGeo(gridElementsWidth*m_nbElements, gridElementsWidth*m_nbElements, Worm::WORM_SIZE, 1, 1, 1);
        m_highlightMaterial = OSG::SimpleMaterial::create();
        m_highlightMaterial->setLit(true);
        m_highlightMaterial->setColorMaterial(GL_NONE);
        m_highlightMaterial->setDiffuse(OSG::Color3f(0.5, 0.5, 0.5));
        m_highlightMaterial->setTransparency(1);
        OSG::PolygonChunkRefPtr chunk = OSG::PolygonChunk::create();
        chunk->setFrontMode(GL_LINE);
        chunk->setBackMode(GL_LINE);
        m_highlightMaterial->addChunk(chunk);
        geo->setMaterial(m_highlightMaterial);
        m_highlight = OSG::Node::create();
        OSG::setName(m_highlight, oss.str());
        m_highlight->setCore(geo);
        m_grid->addChild(m_highlight);

        //OSG::Quaternion quat(OSG::Vec3f(0,0,1), -M_PI/2.0);
        for(double i=0; i<m_nbElements; ++i) {
            m_elements.push_back(std::vector<OSG::NodeRecPtr>());
            m_elementsTrans.push_back(std::vector<OSG::ComponentTransformRecPtr>());
            m_elementsMaterial.push_back(std::vector<OSG::SimpleMaterialRefPtr>());
            m_elementsSpeed.push_back(std::vector<double>());
            m_elementsRot.push_back(std::vector<double>());
            for(double j=0; j<m_nbElements; ++j) {
                //OSG::GeometryRefPtr geo = OSG::makeCylinderGeo(gridElementsWidth, Worm::WORM_SIZE/2.0, GRIDS_ELEMENTS_SIDES, true, false, false);
                OSG::GeometryRefPtr geo2 = OSG::makeBoxGeo(gridElementsWidth, gridElementsWidth, gridElementsWidth, 1, 1, 1);
                OSG::setName(geo2, oss.str());

                OSG::SimpleMaterialRefPtr material = OSG::SimpleMaterial::create();
                material->setLit(true);
                material->setColorMaterial(GL_AMBIENT_AND_DIFFUSE);
                material->setDiffuse(OSG::Color3f(0.5,0.5,0.5));
                material->setAmbient(OSG::Color3f(0.5,0.5,0.5));
                material->setSpecular(OSG::Color3f(0.5,0.5,0.5));
                material->setTransparency(0.4);

                geo2->setMaterial(material);

                OSG::NodeRefPtr element = OSG::Node::create();
                OSG::setName(element, oss.str());
                element->setCore(geo2);

                OSG::NodeRefPtr tempTransNode = OSG::Node::create();
                OSG::ComponentTransformRecPtr tempTrans = OSG::ComponentTransform::create();
                //tempTrans->setRotation(quat);
                tempTransNode->addChild(element);
                tempTransNode->setCore(tempTrans);
                OSG::setName(tempTransNode, oss.str());

                OSG::NodeRecPtr transNode = OSG::Node::create();
                OSG::setName(transNode, oss.str());
                OSG::ComponentTransformRecPtr trans = OSG::ComponentTransform::create();
                trans->setTranslation(OSG::Vec3f((i - m_nbElements/2.0)*gridElementsWidth, (j - m_nbElements/2.0)*gridElementsWidth, 0));
                trans->setRotation(OSG::Quaternion(1,0,0,0));
                transNode->addChild(tempTransNode);
                transNode->setCore(trans);
                m_grid->addChild(transNode);

                m_elements[i].push_back(element);
                m_elementsTrans[i].push_back(trans);
                m_elementsMaterial[i].push_back(material);
                m_elementsSpeed[i].push_back(0);
                m_elementsRot[i].push_back(0);

            }
        }
        setScale(0);
        m_length = gridElementsWidth*m_nbElements;
        OSG::commitChanges();
    }
}

Grid::~Grid()
{

}

void Grid::parse(xmlNodePtr node)
{
    char* value=NULL;
    value = (char*)xmlGetProp(node,(xmlChar*)"position");
    if(value!=NULL) {
        string valueStr=value;
        istringstream is,is2,is3;
        size_t pos;
        pos=valueStr.find_first_of("/");
        if(pos!=std::string::npos) {
            is.str(valueStr.substr(0,pos));
            is>>m_position[0];
        }
        is2.str(valueStr.substr(pos+1));
        is2>>m_position[1];
    }

    xmlNodePtr paramNode;
    for (paramNode = node->children; paramNode; paramNode = paramNode->next) {
        if (paramNode->type == XML_ELEMENT_NODE && (!xmlStrcmp(paramNode->name, (const xmlChar *) "GridXParameter") || !xmlStrcmp(paramNode->name, (const xmlChar *) "GridYParameter"))) {
            int axis=0;
            if(!xmlStrcmp(paramNode->name, (const xmlChar *) "GridYParameter")) {
                axis=1;
            }
            char* value=NULL;
            value = (char*)xmlGetProp(paramNode,(xmlChar*)"parameter");
            if(value!=NULL) {
                m_gridParameters[axis].push_back(GridParameter());
                GridParameter &newParameter = m_gridParameters[axis][m_gridParameters[axis].size()-1];
                
                if(std::string(value).compare("color")==0) {
                    newParameter.m_param=Worm::WORM_COLOR;
                }
                else if(std::string(value).compare("transparency")==0) {
                    newParameter.m_param=Worm::WORM_TRANSPARENCY;
                }
                else if(std::string(value).compare("distortion")==0) {
                    newParameter.m_param=Worm::WORM_DISTORTION;
                }
                else if(std::string(value).compare("scattering")==0) {
                    newParameter.m_param=Worm::WORM_SCATTERING;                }
                else if(std::string(value).compare("speed")==0) {
                    newParameter.m_param=Worm::WORM_SPEED;
                }
                else {
                    newParameter.m_param=Worm::WORM_HEIGHT;
                }

                xmlNodePtr scaleNode;
                for (scaleNode = paramNode->children; scaleNode; scaleNode = scaleNode->next) {
                    if (scaleNode->type == XML_ELEMENT_NODE && !xmlStrcmp(scaleNode->name, (const xmlChar *) "GridScale")) {
                        newParameter.m_scales.push_back(GridScale());
                        GridScale &newScale = newParameter.m_scales[newParameter.m_scales.size()-1];
                        value=NULL;
                        value = (char*)xmlGetProp(scaleNode,(xmlChar*)"type");
                        if(value!=NULL) {
                            if(std::string(value).compare("array")==0) {
                                newScale.m_type=GRID_SCALE_ARRAY;
                            }
                            else {
                                newScale.m_type=GRID_SCALE_LINEAR;
                            }
                        }
                        value=NULL;
                        value = (char*)xmlGetProp(scaleNode,(xmlChar*)"values");
                        if(value!=NULL) {
                            std::string valuesStr = std::string(value);
                            size_t pos = valuesStr.find_first_of(";");
                            while(pos!=string::npos) {
                                istringstream is;
                                is.str(valuesStr.substr(0,pos));
                                double val;
                                is>>val;
                                newScale.m_values.push_back(val);
                                valuesStr = valuesStr.substr(pos+1);    
                                pos = valuesStr.find_first_of(";");
                            }
                        }
                    }
                }
            }
        }
    }

}

void Grid::setParameter(const string& param, const string& value)
{
}

void Grid::addParameter(const string& param)
{
    /*
    GridParameter newParameter;
    if(param.compare("color")==0) {
        newParameter.m_param=WORM_COLOR;
    }
    else if(param.compare("height")==0) {
        newParameter.m_param=WORM_HEIGHT;
    }
    else if(param.compare("transparency")==0) {
        newParameter.m_param=WORM_TRANSPARENCY;
    }
    else if(param.compare("distortion")==0) {
        newParameter.m_param=WORM_DISTORTION;
    }
    else if(param.compare("scattering")==0) {
        newParameter.m_param=WORM_SCATTERING;
    }
    else if(param.compare("speed")==0) {
        newParameter.m_param=WORM_SPEED;
    }

    m_gridParameters.push_back(newParameter);
    */
}

void Grid::changeScale()
{
    bool reset=true;    
    for(unsigned int a=0;a<2;++a) {
        for(unsigned int p=0;p<m_gridParameters[a].size();++p) {
            GridParameter &parameter=m_gridParameters[a][p];
            if(m_currentScale+1<int(parameter.m_scales.size())) {
                reset=false;
            }
        }
    }
    if(reset) {
        m_currentScale=0;
    }
    else {
        m_currentScale++;
    }
    
    setScale(m_currentScale);
}

void Grid::setScale(const int& scaleID)
{
    double value;
    for(unsigned int a=0; a<2;++a) {
        for(vector<GridParameter>::iterator itParam=m_gridParameters[a].begin();itParam!=m_gridParameters[a].end();++itParam) {
            if(scaleID<int((*itParam).m_scales.size())) {
                (*itParam).m_currentScale=scaleID;
                for(int elAxe1=0; elAxe1<m_nbElements; ++elAxe1) {
                    computeValueFromRatio(value, *itParam, double(elAxe1)/m_nbElements);
                    for(int elAxe2=0; elAxe2<m_nbElements; ++elAxe2) {
                        switch((*itParam).m_param) {
                            case Worm::WORM_COLOR: {
                                OSG::Color3f &color = (a==0)? m_elementsMaterial[elAxe1][elAxe2]->editDiffuse(): m_elementsMaterial[elAxe2][elAxe1]->editDiffuse();
                                color.setValuesHSV(fmod(value*5.0,1.0)*360.0, 0.6, value*0.9+0.1); 
                            }break;
                            case Worm::WORM_TRANSPARENCY: {
                                if(a==0) {
                                    m_elementsMaterial[elAxe1][elAxe2]->setTransparency(0.2+value*0.7);
                                }
                                else {
                                    m_elementsMaterial[elAxe2][elAxe1]->setTransparency(0.2+value*0.7);
                                }
                            }break;
                            case Worm::WORM_HEIGHT: {
                                OSG::Vec3f &scale = (a==0)? m_elementsTrans[elAxe1][elAxe2]->editScale() : m_elementsTrans[elAxe2][elAxe1]->editScale();
                                scale[2]=value;
                            }break;
                            case Worm::WORM_DISTORTION: {

                            }break;
                            case Worm::WORM_SCATTERING: {
                                OSG::Vec3f &scale = (a==0)? m_elementsTrans[elAxe1][elAxe2]->editScale(): m_elementsTrans[elAxe2][elAxe1]->editScale();
                                scale[0]=1.0-(value*0.9);
                                scale[1]=1.0-(value*0.9);
                            }break;
                            case Worm::WORM_SPEED: {
                                if(a==0) {
                                    m_elementsRot[elAxe1][elAxe2]=0;
                                    m_elementsSpeed[elAxe1][elAxe2]=value*1.9+0.1;
                                }
                                else {
                                    m_elementsRot[elAxe2][elAxe1]=0;
                                    m_elementsSpeed[elAxe2][elAxe1]=value*1.9+0.1;
                                }

                            }break;
                            default:break;
                        }
                    }
                }
            }    
        }    
    }
}

void Grid::setStretch(const double& stretch) {
    OSG::Vec3f& gridScale = m_gridTrans->editScale();
    if(gridScale[0]+stretch>0.01) {
        gridScale[0]+=stretch;
    }
}

void Grid::computeValueFromRatio(double & value, GridParameter& parameter, const double& ratio)
{
    GridScale &scale = parameter.m_scales[parameter.m_currentScale];
    if(scale.m_type == GRID_SCALE_ARRAY) {
        if(ratio<1) {
            value = scale.m_values[int(ratio*scale.m_values.size())];
        }
        else {
            value = scale.m_values[scale.m_values.size()-1];
        }
    }
    else {
        value=ratio*(scale.m_values[1]-scale.m_values[0])+scale.m_values[0];
    }
    if(value<0) {
        value=0;
    }
    else if(value>1) {
        value=1;
    }
}

void Grid::testCollision(OSG::IntersectAction* intersectAction, OSG::NodeRefPtr root)
{
/*

    OSG::Vec3f posGrid, scale, posWorm, beginGrid;
    OSG::Quaternion quat, scaleQuat;
    
    OSG::BoxVolume gridVolume;
    m_grid->getWorldVolume(gridVolume);
    m_grid->getToWorld().getTransform(posGrid, quat, scale, scaleQuat);

    EnvManager *envManager = EnvManager::getInstance();
    Room* currentRoom = envManager->getCurrentRoom();
    
    //for each worm
    for(unsigned int worm = 0; worm<currentRoom->getNbWorms(m_zoneID); ++worm) {
        int currentWormID = currentRoom->getWormID(m_zoneID, worm);
        Worm* currentWorm = envManager->getWorm(currentWormID);

        //test intersection with the grid
        OSG::BoxVolume currentWormVolume;
        currentWorm->getCollisionNode()->getWorldVolume(currentWormVolume);
        //if it intersects
        if(gridVolume.intersect(currentWormVolume)) {

            double stretch = m_gridTrans->getScale()[0]; 
            stretch*=m_length;
            
            OSG::Vec3f dir(1, 0, 0);
            quat.multVec(dir, dir);

            OSG::Vec3f dir2(0, 1, 0);
            quat.multVec(dir2, dir2);

            beginGrid = posGrid - (dir+dir2)*(stretch/2.0);
            currentWorm->getNode()->getToWorld().getTransform(posWorm, quat, scale, scaleQuat);
            //test if worm already in the grid
            int prevWorm=-1;
            for(unsigned int w=0; w<m_insideWormsIDs.size(); ++w) {
                if(m_insideWormsIDs[w]==currentWormID+1) {
                    prevWorm=w;
                }
            }
            if(prevWorm>=0) { //if it is
                //set the worm as being still in the grid
                m_insideWormsIDs[prevWorm]*=-1;
            }
            else {
                //if no, tell it to remember the values of controled parameters and keep the entering position (x,y,z)
                for(unsigned int a=0;a<2;++a) {
                    for(unsigned int p=0;p<m_gridParameters[a].size();++p) {
                        currentWorm->storeParameterValue(m_gridParameters[a][p].m_param);
                    }
                }
                OSG::Vec3f prevPos(1,1,1);
                for(int i=0;i<3;++i) {
                    if(posWorm[i]<posGrid[i]) {
                        prevPos[i]=-1;    
                    }
                }
                //add the worm
                m_insideWormsPositions.push_back(prevPos);
                m_insideWormsIDs.push_back((currentWormID+1)*-1);
                //bypass its parameters
                for(unsigned int a=0;a<2;++a) {
                    for(vector<GridParameter>::iterator itParam=m_gridParameters[a].begin(); itParam<m_gridParameters[a].end(); ++itParam) {
                        currentWorm->bypassAudioParameter((*itParam).m_param, true);
                    }
                }
            }

            //compute ratio according to position
            double ratio[2];
            ratio[0]=(cos(dir.enclosedAngle(posWorm-beginGrid))*(posWorm.addToZero().dist(beginGrid.addToZero()) ))/stretch;
            ratio[1]=(cos(90-dir.enclosedAngle(posWorm-beginGrid))*(posWorm.addToZero().dist(beginGrid.addToZero()) ))/stretch;
            for(unsigned int a=0;a<2;++a) {
                if(ratio[a]>1) {
                    ratio[a]=1;
                }
                else if(cos(dir.enclosedAngle(posWorm-beginGrid))<0) {
                    ratio[a]=0;
                }
                //for each parameter, get value from ratio and set the value
                double value=0;
                for(vector<GridParameter>::iterator itParam=m_gridParameters[a].begin(); itParam<m_gridParameters[a].end(); ++itParam) {
                    computeValueFromRatio(value, *itParam, ratio[a]);
                    //currentWorm->setParameter((*itParam).m_param, std::vector<double>(1, value));
                    currentWorm->setParameter((*itParam).m_param, &value, 1);
                }
            }
        }
    }

    //among the previously recorded worms
    vector<int>::iterator itWorm = m_insideWormsIDs.begin();
    vector<OSG::Vec3f>::iterator itWormPos = m_insideWormsPositions.begin();
    for(;itWorm!=m_insideWormsIDs.end() && itWormPos!=m_insideWormsPositions.end();) {
        //if not there anymore
        if((*itWorm)>0) {
            OSG::Vec3f posPrevWorm;
            Worm* prevWorm = envManager->getWorm((*itWorm)-1);
            //unbypass its parameters
            for(unsigned int a=0;a<2;++a) {
                for(vector<GridParameter>::iterator itParam=m_gridParameters[a].begin(); itParam<m_gridParameters[a].end(); ++itParam) {
                    prevWorm->bypassAudioParameter((*itParam).m_param, false);
                }
            }
            prevWorm->getNode()->getToWorld().getTransform(posPrevWorm, quat, scale, scaleQuat);
            //compute current position relative to Grid
            //FIXME
            OSG::Vec3f newPos(1,1,1);
            bool crossed=false;
            for(int i=0;i<3;++i) {
                if(posPrevWorm[i]<posGrid[i]) {
                    newPos[i]=-1;    
                }
            }

            //FIXME test crossing in several directions 
            if((*itWormPos)[1]+newPos[1]==0) {
                crossed=true;
            }
            
            //if crossed the grid, keep current values
            //else restore values
            if(!crossed && prevWorm->getNbUsers()!=0) {
                for(unsigned int a=0;a<2;++a) {
                    for(vector<GridParameter>::iterator itParam=m_gridParameters[a].begin();itParam!=m_gridParameters[a].end();++itParam) {
                        prevWorm->restoreParameterValue((*itParam).m_param);
                    }
                }
            }
            
            //remove the worm
            m_insideWormsIDs.erase(itWorm);        
            m_insideWormsPositions.erase(itWormPos);        
        }
        else {
            //reset the worms ids to positive values
            (*itWorm)*=-1;
            ++itWorm;
            ++itWormPos;
        }
    }


    //update rotation of the elements
    for(unsigned int i=0; i<m_elementsTrans.size(); ++i) {
        for(unsigned int j=0; j<m_elementsTrans[j].size(); ++j) {
            if(m_elementsTrans[i][j]>0) {
                OSG::Quaternion& quat = m_elementsTrans[i][j]->editRotation(); 
                m_elementsRot[i][j]=double(fmod(double(m_elementsRot[i][j]+m_elementsSpeed[i][j]/10.0), double(M_PI*2.0)));
                quat.setValue(0, m_elementsRot[i][j], 0);
            }
        }
    }
*/
}

void Grid::setHighlight(const bool& highlight, const double& hue)
{
    if((!highlight && m_nbUsers==0) || (highlight && m_nbUsers>0)) {
        m_highlightMaterial->setTransparency(!highlight);
        OSG::Color3f &color = m_highlightMaterial->editDiffuse();
        color.setValuesHSV(hue*360.0, 0.6, 0.8); 
    }
}

bool Grid::addGrabbingTool(const int& toolID)
{
    bool added=false;
    //if nothing grabbing
    if(m_grabbingToolID==-1) {
        m_grabbingToolID=toolID;
        added=true;
    }
    else if(m_grabbingToolID==toolID) {
        m_isGrabbedWithBothHands=true;    
        added=true;
    }

    return added;
}

void Grid::removeGrabbingTool(const int& toolID)
{
    if(m_grabbingToolID==toolID) {
        if(m_isGrabbedWithBothHands) {
            m_isGrabbedWithBothHands=false;
        }
        else {
            m_grabbingToolID=-1;
        }
    }
}

