/***************************************************************************
 *            Grid.hpp
 *            Part of Drile
 *
 *  2010  Tardigrade Inc.
 *  http://tardigrade-inc.com 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _GRID_H
#define _GRID_H

#include <iostream>
#include <cstring>
#include <cmath>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <vector>
#include <map>

#include <OpenSG/OSGSimpleGeometry.h>
#include <OpenSG/OSGComponentTransform.h>
#include <OpenSG/OSGMaterialGroup.h>
#include <OpenSG/OSGSimpleMaterial.h>
#include <OpenSG/OSGIntersectAction.h>
#include <OpenSG/OSGBoxVolume.h>

#include "Worm.hpp"

enum GRID_SCALE_TYPE {GRID_SCALE_LINEAR, GRID_SCALE_ARRAY};

struct GridScale
{
    GRID_SCALE_TYPE m_type;
    std::vector<double> m_values;
};

struct GridParameter
{
    Worm::WORM_PARAMETER m_param;
    int m_currentScale;
    std::vector<GridScale> m_scales;
};

class Grid
{
    public:
        Grid(const int&);
        ~Grid();

        void parse(xmlNodePtr);

        inline const int& getID(){return m_id;}
        void setParameter(const std::string&, const std::string&);
        
        void addParameter(const std::string&);

        inline OSG::NodeRecPtr getNode(){return m_grid;}
        inline OSG::ComponentTransformRecPtr getTrans(){return m_gridTrans;}

        void changeScale();
        void setScale(const int&);
        void computeValueFromRatio(double &, GridParameter&, const double&);
        void setStretch(const double&);
        inline void setDepth(const double& depth) {
            OSG::Vec3f& pos = m_gridTrans->editTranslation();
            pos[2]=depth;
        }

        void testCollision(OSG::IntersectAction*, OSG::NodeRefPtr); 

        void setHighlight(const bool&, const double& hue=0);

        inline void addUser(){m_nbUsers++;}
        inline void removeUser(){m_nbUsers--;if(m_nbUsers<0){m_nbUsers=0;}}

        bool addGrabbingTool(const int& toolID);
        void removeGrabbingTool(const int& toolID);
        inline bool isGrabbedWithBothHands(const int& toolID){return ((m_grabbingToolID==toolID)&&m_isGrabbedWithBothHands);}

        inline const unsigned int& getZoneID(){return m_zoneID;}
        inline void setZoneID(const unsigned int& zone){m_zoneID=zone;}

    private:

        int m_id;
        int m_nbUsers;

        int m_grabbingToolID;
        bool m_isGrabbedWithBothHands;

        unsigned int m_zoneID;

        std::vector<GridParameter> m_gridParameters[2];
        int m_currentScale;

        OSG::NodeRecPtr m_grid;
        OSG::ComponentTransformRecPtr m_gridTrans;
        OSG::Vec3f m_position;

        OSG::NodeRefPtr m_highlight;
        OSG::SimpleMaterialRefPtr m_highlightMaterial;

        std::vector<std::vector<OSG::NodeRecPtr> > m_elements;
        std::vector<std::vector<OSG::ComponentTransformRecPtr> > m_elementsTrans;
        std::vector<std::vector<OSG::SimpleMaterialRefPtr> > m_elementsMaterial;
        std::vector<std::vector<double> > m_elementsSpeed;
        std::vector<std::vector<double> > m_elementsRot;

        std::vector<int> m_insideWormsIDs;
        std::vector<OSG::Vec3f> m_insideWormsPositions;

        double m_length;
        double m_nbElements;

};



#endif 
