/***************************************************************************
 *  LeafWorm.cpp
 *  Part of Drile
 *  2010- Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "LeafWorm.hpp"
#include "../Drile.hpp"

using namespace std;

LeafWorm::LeafWorm(const int& id):Worm(id) {
    std::ostringstream oss;
    oss<<m_id;
    {
        m_nbElements=24;
        m_elementHeight = WORM_SIZE/m_nbElements;
        //create the shape
        for(float i=0;i<m_nbElements;++i){
            OSG::GeometryRefPtr geo = OSG::makeCylinderGeo(m_elementHeight, 
                                                            WORM_SIZE/2.0, 
                                                            10, true, true, 
                                                            true);
            geo->setMaterial(m_wormMaterial);

            OSG::NodeRefPtr node = OSG::Node::create();
            OSG::setName(node, oss.str());
            node->setCore(geo);

            OSG::NodeRecPtr transNode = OSG::Node::create();
            OSG::setName(transNode, oss.str());
            OSG::ComponentTransformRecPtr transCore = 
                    OSG::ComponentTransform::create();
            transCore->setTranslation(OSG::Vec3f(0, i*m_elementHeight 
                                    - (m_elementHeight*m_nbElements)/2.0, 0));
            transNode->addChild(node);
            transNode->setCore(transCore);
            m_elementsGroupTransNode->addChild(transNode);

            m_elementsTrans.push_back(transCore);
            m_elementsRotation.push_back(OSG::Vec3f(0, 0, 0));
            m_elementsWidth.push_back(0.2);
        }

        m_worm->updateVolume();

        OSG::commitChanges();
    }
}

LeafWorm::~LeafWorm() {}

void LeafWorm::update(std::vector<Event>& outputEvents) {
    Worm::update(outputEvents);
    if(m_staticShapeChanged) {
        updateStaticShape();
        m_staticShapeChanged=false;
    }
}

void LeafWorm::updateStaticShape() {
    double max=0;
    //get the max
    vector<double>::iterator itVal;
    int layer=0;
    for(itVal=m_staticShapeValues.begin(); itVal!=m_staticShapeValues.end() 
            && layer<m_nbElements; ++itVal, ++layer) {
        if(*itVal>max) {
            max=*itVal;
        }
    }
    //normalize and set the shape
    double ratio=0.8/max;
    layer=0;
    for(itVal=m_staticShapeValues.begin(); itVal!=m_staticShapeValues.end() 
            && layer<m_nbElements; ++itVal, ++layer) {
        OSG::Vec3f &scale = m_elementsTrans[layer]->editScale();
        scale[0]=0.2+(*itVal)*ratio;
        scale[2]=scale[0];
        m_elementsWidth[layer]=scale[0];
    }
}

void LeafWorm::setParameter(const std::string&, const std::string&){}


void LeafWorm::parse(xmlNodePtr node) {
    Worm::parse(node);
}

Ghostable* LeafWorm::createGhost() {
    LeafWorm* newGhost = new LeafWorm(m_id);
    newGhost->setLiving(this);
    newGhost->m_staticShapeValues.clear();
    //give it the static shape
    vector<double>::iterator itVal;
    for(itVal=m_staticShapeValues.begin(); 
            itVal!=m_staticShapeValues.end();++itVal) {
        newGhost->m_staticShapeValues.push_back(*itVal);
    }
    newGhost->updateStaticShape();

    //set the other parameters
    newGhost->setColor(m_parametersValues[WORM_COLOR]);
    newGhost->setHeight(m_parametersValues[WORM_HEIGHT]);
    newGhost->setTransparency(m_parametersValues[WORM_TRANSPARENCY]);
    newGhost->setScattering(m_parametersValues[WORM_SCATTERING]);
    //update

    m_ghosts.push_back(newGhost);
    return newGhost;
}


