/***************************************************************************
 *  LoopWorm.cpp
 *  Part of Drile
 *  2010- Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "LoopWorm.hpp"
#include "../Drile.hpp"

using namespace std;

LoopWorm::LoopWorm(const int& id):Worm(id), 
                                    m_rotationSpeed(0.5), m_rotation(0) {

    m_geomString = "o Cube\n"
        "v -0.000000 -0.408156 -1.414214\n"
        "v 1.414214 -0.408156 -0.000000\n" 
        "v 0.000000 -0.408156 4.025044\n"
        "v -1.414214 -0.408156 -0.000000\n" 
        "v 0.000000 0.408156 -1.414214\n"
        "v 1.414214 0.408156 0.000000\n"
        "v -0.000000 0.408156 4.025044\n"
        "v -1.414214 0.408156 0.000000\n"
        "s off\n"
        "f 1 2 3 4\n"
        "f 5 8 7 6\n"
        "f 1 5 6 2\n"
        "f 2 6 7 3\n"
        "f 3 7 8 4\n"
        "f 5 1 4 8\n";

    std::ostringstream oss;
    oss<<m_id;

    {

        m_nbElements=10;
        m_elementHeight = WORM_SIZE/ m_nbElements;
        //create the loop geometry
        OSG::GeometryRecPtr geo = createLoopGeo(m_elementHeight);

        //create elements
        for(float i=0;i<m_nbElements-1;++i) {
            OSG::NodeRecPtr node = OSG::Node::create();
            node->setCore(geo);
            geo->setMaterial(m_wormMaterial);
            OSG::setName(node, oss.str());

            OSG::NodeRecPtr transNode = OSG::Node::create();
            OSG::setName(transNode, oss.str());
            OSG::ComponentTransformRecPtr transCore = 
                    OSG::ComponentTransform::create();
            transCore->setTranslation(
                            OSG::Vec3f(0, i*m_elementHeight-
                                (m_elementHeight*m_nbElements)/2.0, 0));
            transNode->addChild(node);
            transNode->setCore(transCore);
            m_elementsGroupTransNode->addChild(transNode);
            m_elementsTrans.push_back(transCore);
            m_elementsRotation.push_back(OSG::Vec3f(0, 0, 0));
            m_elementsWidth.push_back(0.2);
        }

        OSG::NodeRecPtr tempScaleTransNode=OSG::Node::create();
        m_scaleTrans=OSG::ComponentTransform::create();
        tempScaleTransNode->setCore(m_scaleTrans);
        m_scaleTrans->setScale(OSG::Vec3f(1, 0.6, 1));

        //create the offset cursor node
        m_offsetNode=OSG::Node::create();
        m_offsetNode->setCore(geo);
        OSG::ComponentTransformRecPtr tempTrans2 = 
                                    OSG::ComponentTransform::create();
        tempTrans2->setTranslation(
            OSG::Vec3f(0,m_elementHeight*((m_nbElements/2.0)-1.0),0));
        tempTrans2->setScale(OSG::Vec3f(0.3,0.5,0.5));
        OSG::NodeRefPtr tempTransNode2 = OSG::Node::create();
        tempTransNode2->addChild(m_offsetNode);
        tempTransNode2->setCore(tempTrans2);
        m_offsetTrans = OSG::ComponentTransform::create();
        m_offsetTransNode = OSG::Node::create();
        m_offsetTransNode->setCore(m_offsetTrans);
        m_offsetTransNode->addChild(tempTransNode2);
        tempScaleTransNode->addChild(m_offsetTransNode);

        //speedwheel
        OSG::NodeRefPtr wheelNode= OSG::Node::create();
        OSG::GeometryRefPtr wheelGeo = 
            OSG::makeBoxGeo(Worm::WORM_SIZE/4.0, m_elementHeight/3.0, 
                                Worm::WORM_SIZE/4.0, true, true, true);
        wheelNode->setCore(wheelGeo);
        wheelGeo->setMaterial(m_wormMaterial);
        m_wheelTrans = OSG::ComponentTransform::create();
        OSG::NodeRefPtr wheelTransNode = OSG::Node::create();
        wheelTransNode->setCore(m_wheelTrans);
        wheelTransNode->addChild(wheelNode);
        OSG::ComponentTransformRecPtr wheelTempTrans = 
                                    OSG::ComponentTransform::create();
        wheelTempTrans->setTranslation(
            OSG::Vec3f(0,-m_elementHeight*(m_nbElements/2.0+1),0));
        OSG::NodeRefPtr wheelTempTransNode = OSG::Node::create();
        wheelTempTransNode->addChild(wheelTransNode);
        wheelTempTransNode->setCore(wheelTempTrans);
        tempScaleTransNode->addChild(wheelTempTransNode);
        m_worm->addChild(tempScaleTransNode);

        m_worm->updateVolume();

        OSG::commitChanges();
    }

}

LoopWorm::~LoopWorm() {}

void LoopWorm::parse(xmlNodePtr) {}

void LoopWorm::setRotationOffset(const double& value) {
    m_parametersValues[WORM_ROTATION_OFFSET]=value;
    OSG::Quaternion& quat = m_offsetTrans->editRotation();
    quat.setValue(0, -M_PI*2.0*value, 0);
}

void LoopWorm::setParameter(const WORM_PARAMETER& param, double* values, 
                                unsigned int nbValues, bool valueFromAudio) {
    switch(param) {
        case WORM_SPEED: {
            Worm::setSpeed(values[0]);                
            m_rotationSpeed=values[0];
            if(!valueFromAudio) {
                sendParameterValue(WORM_SPEED);
            }
        }break;
        case WORM_HEIGHT: {
            Worm::setHeight(values[0]);
            OSG::Vec3f& scale = m_scaleTrans->editScale();
            OSG::Vec3f& scaleW =  m_elementsGroupTrans->editScale();
            scale[1]=scaleW[1];
            if(!valueFromAudio) {
                sendParameterValue(WORM_HEIGHT);
            }
        }break;
        default:{
            Worm::setParameter(param, values, nbValues, valueFromAudio);
        }break;
    }
}

void LoopWorm::setParameter(const std::string&, const std::string&)
{

}

void LoopWorm::update(std::vector<Event>& outputEvents) {
    Worm::update(outputEvents);
    OSG::Quaternion& quat = m_wheelTrans->editRotation();
    m_rotation=fmod(m_rotation+m_rotationSpeed*0.5,360.0);
    quat.setValue(0,-m_rotation/M_PI*2.0,0);
}

Ghostable* LoopWorm::createGhost() {
    LoopWorm* newGhost = new LoopWorm(m_id);
    newGhost->setLiving(this);
    newGhost->setColor(m_parametersValues[WORM_COLOR]);
    newGhost->setHeight(m_parametersValues[WORM_HEIGHT]);
    newGhost->setTransparency(m_parametersValues[WORM_TRANSPARENCY]);
    newGhost->setScattering(m_parametersValues[WORM_SCATTERING]);
    m_ghosts.push_back(newGhost);
    return newGhost;
}


OSG::GeometryRecPtr LoopWorm::createLoopGeo(const double& layersHeight) {
    OSG::GeoUInt8PropertyRecPtr types = OSG::GeoUInt8Property::create();
    types->addValue(GL_QUADS);
    OSG::GeoUInt32PropertyRecPtr lengths = OSG::GeoUInt32Property::create();
    lengths->addValue(4 * 6);
    OSG::GeoPnt3fPropertyRecPtr pos = OSG::GeoPnt3fProperty::create();
    OSG::GeoVec3fPropertyRecPtr norms = OSG::GeoVec3fProperty::create();

    double size = WORM_SIZE/4.0;

    pos->addValue(OSG::Pnt3f(0, layersHeight/2.0, size/4.0*3.0));
    norms->addValue(OSG::Vec3f(0,1,0));
    pos->addValue(OSG::Pnt3f(size/2.0, layersHeight/2.0, 0));
    norms->addValue(OSG::Vec3f(0,1,0));
    pos->addValue(OSG::Pnt3f(0, layersHeight/2.0, -size/4.0));
    norms->addValue(OSG::Vec3f(0,1,0));
    pos->addValue(OSG::Pnt3f(-size/2.0, layersHeight/2.0, 0));
    norms->addValue(OSG::Vec3f(0,1,0));

    pos->addValue(OSG::Pnt3f(0, -layersHeight/2.0, size/4.0*3.0));
    norms->addValue(OSG::Vec3f(0,-1,0));
    pos->addValue(OSG::Pnt3f(size/2.0, -layersHeight/2.0, 0));
    norms->addValue(OSG::Vec3f(0,-1,0));
    pos->addValue(OSG::Pnt3f(0, -layersHeight/2.0, -size/4.0));
    norms->addValue(OSG::Vec3f(0,-1,0));
    pos->addValue(OSG::Pnt3f(-size/2.0, -layersHeight/2.0, 0));
    norms->addValue(OSG::Vec3f(0,-1,0));

    OSG::GeoUInt32PropertyRecPtr indices = OSG::GeoUInt32Property::create();
    indices->addValue(0); indices->addValue(1); 
    indices->addValue(2); indices->addValue(3); 
    indices->addValue(0); indices->addValue(4); 
    indices->addValue(5); indices->addValue(1); 
    indices->addValue(1); indices->addValue(5); 
    indices->addValue(6); indices->addValue(2); 
    indices->addValue(2); indices->addValue(6); 
    indices->addValue(7); indices->addValue(3); 
    indices->addValue(3); indices->addValue(7); 
    indices->addValue(4); indices->addValue(0); 
    indices->addValue(4); indices->addValue(7); 
    indices->addValue(6); indices->addValue(5); 


    OSG::GeometryRecPtr geo = OSG::Geometry::create();
    
    geo->setTypes(types);
    geo->setLengths(lengths);
    geo->setIndices(indices);
    geo->setPositions(pos);
    geo->setNormals(norms);

    calcVertexNormals(geo);
 
    return geo;
}

