/***************************************************************************
 *  Manipulable.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Manipulable.hpp"
#include "Piivert.hpp"

using namespace std;

Manipulable::Manipulable(const int& id): m_id(id), m_zone(NULL), m_nbUsers(0), 
                                            m_grabbingToolID(-1), 
                                            m_grabInitalised(false), 
                                            m_isGrabbedWithBothHands(false) {}

Manipulable::~Manipulable() {}

void Manipulable::grab(Piivert* tool, 
                        const int& hand, 
                            std::vector<Event>& outputEvents) {
    addGrabbingTool(tool->getID());
}

void Manipulable::release(Piivert* tool, 
                            const int& hand, 
                                std::vector<Event>& outputEvents) {
    removeGrabbingTool(tool->getID());
}

bool Manipulable::addGrabbingTool(const int& toolID) {
    bool added=false;
    //if nothing grabbing
    if(m_grabbingToolID==-1) {
        m_grabbingToolID=toolID;
        added=true;
    }
    else if(m_grabbingToolID==toolID) {
        m_isGrabbedWithBothHands=true;    
        added=true;
    }
    return added;
}

void Manipulable::removeGrabbingTool(const int& toolID) {
    if(m_grabbingToolID==toolID) {
        if(m_isGrabbedWithBothHands) {
            m_isGrabbedWithBothHands=false;
        }
        else {
            m_grabbingToolID=-1;
        }
    }
}

