/***************************************************************************
 *  Manipulable.hpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _Manipulable_H
#define _Manipulable_H

#include <OpenSG/OSGSimpleGeometry.h>

#include "../Event.hpp"

class Zone;
class Piivert;
class Musician;

class Manipulable {
    public:
        Manipulable(const int& id);
        virtual ~Manipulable()=0;

        inline const int& getID(){return m_id;}
        virtual void grab(Piivert* tool, 
                            const int& hand, 
                                std::vector<Event>& outputEvents)=0;
        virtual void release(Piivert* tool, 
                                const int& hand, 
                                    std::vector<Event>& outputEvents)=0;
        virtual void moveTo(Piivert* tool,
                                const int& hand, 
                                    const float& depth, 
                                        const OSG::Line& ray, 
                                            std::vector<Event>& outputEvents)=0;
        virtual void hit(std::vector<Event>& outputEvents)=0;
        virtual const OSG::Vec3f& getPosition()=0;
        virtual void setHighlight(const bool&, const double& hue=0, 
                                    const double& scale=1.0)=0;

        inline const int& getNbUsers(){return m_nbUsers;}
        inline void addUser(){m_nbUsers++;}
        inline void removeUser(){m_nbUsers--;}

        inline void setZone(Zone* zone){m_zone=zone;}
        virtual void removeFromZone(Zone*)=0;
        virtual void moveToZone(Zone*)=0;
        virtual void attachToMusician(Musician* mus)=0;
        virtual void detachFromMusician(Musician* mus)=0;

    protected:
        bool addGrabbingTool(const int& toolID);
        void removeGrabbingTool(const int& toolID);
        inline bool isGrabbedWithBothHands(const int& toolID) {
            return ((m_grabbingToolID==toolID)&&m_isGrabbedWithBothHands);
        }

    protected:
        int m_id;
        Zone* m_zone;
        int m_nbUsers;
        int m_grabbingToolID;
        bool m_grabInitalised;
        bool m_isGrabbedWithBothHands;
        OSG::Vec3f m_storedGrabPos;
        OSG::Vec3f m_grabOffset;
};



#endif
