/***************************************************************************
 *  Musician.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "Musician.hpp"
#include "Piivert.hpp"
#include "Zone.hpp"
#include "Room.hpp"
#include "Passage.hpp"
#include "Tunnel.hpp"
#include "Sign.hpp"

using namespace std;

Musician::Musician(int id, OSG::NodeRecPtr root, 
		    OSG::PassiveWindowRecPtr window):
                                        Camera(id,root,window), 
					m_color(0), 
                                        m_passagesHeight(0) {

    {


        //passages / signs / ghosts
        m_passagesRoot=OSG::Node::create();
        m_passagesRootTrans = OSG::ComponentTransform::create();
        m_passagesRoot->setCore(m_passagesRootTrans);
        m_camBeacon->addChild(m_passagesRoot);
        m_signsRoot=OSG::Node::create();
        m_signsRootTrans = OSG::ComponentTransform::create();
        m_signsRoot->setCore(m_signsRootTrans);
        m_camBeacon->addChild(m_signsRoot);
        m_ghostsRoot=OSG::Node::create();
        m_ghostsRootTrans = OSG::ComponentTransform::create();
        m_ghostsRoot->setCore(m_ghostsRootTrans);
        m_camBeacon->addChild(m_ghostsRoot);
/*
        //generate a stage for rendering of each zone in the passages
        float fboWidth=640, fboHeight=480;
        m_fboTexImg = OSG::Image::create();
        m_fboTex = OSG::TextureObjChunk::create();
        m_fbo = OSG::FrameBufferObject::create();
        m_texBuf = OSG::TextureBuffer::create();
        m_depthBuf = OSG::RenderBuffer::create();
        m_fboTexImg->set(OSG::Image::OSG_RGB_PF, fboWidth, fboHeight);
        m_fboTex->setImage(m_fboTexImg);
        m_texBuf->setTexture(m_fboTex);
        m_depthBuf->setInternalFormat(GL_DEPTH_COMPONENT24);
        m_fbo->setColorAttachment(m_texBuf, 0);
        m_fbo->setDepthAttachment(m_depthBuf);
        m_fbo->editMFDrawBuffers()->push_back(GL_COLOR_ATTACHMENT0_EXT);
        m_fbo->setWidth (fboWidth );
        m_fbo->setHeight(fboHeight);
        m_fbo->setPostProcessOnDeactivate(true);
        m_fboMaterial = OSG::SimpleMaterial::create();
        OSG::TextureEnvChunkRefPtr fboTexEnv = OSG::TextureEnvChunk::create();
        fboTexEnv->setEnvMode(GL_REPLACE);
        m_fboMaterial->addChunk(m_fboTex);
        m_fboMaterial->addChunk(fboTexEnv);
        m_fboMaterial->addChunk(
            OSG::StateChunkRefPtr(OSG::TwoSidedLightingChunk::create()));
        m_fboMaterial->setSpecular(OSG::Color3f(0.2f, 0.2f, 0.2f));
        m_fboMaterial->setDiffuse (OSG::Color3f(0.7f, 0.7f, 0.7f));
        m_stage  = OSG::SimpleStage::create();
        m_stageNode = OSG::makeNodeFor(m_stage);
        m_stageVisitCore = OSG::VisitSubTree::create();
        m_stageVisitNode = OSG::Node::create();
        m_stageVisitNode->setCore(m_stageVisitCore);
        m_stageVisitCore->setSubTreeRoot(m_zoneRoot);
        m_stage->setRenderTarget(m_fbo);
        m_stage->setSize(0.0f, 0.0f, 1.0f, 1.0f);
        m_stageCam = OSG::PerspectiveMusician::create();
        m_stageCamBeaconNode=OSG::Node::create();
        m_stageCamBeaconNodeTrans=OSG::ComponentTransform::create();
        m_stageCamBeaconNode->setCore(m_stageCamBeaconNodeTrans);
        m_stageCamBeaconNodeTrans->setTranslation(OSG::Vec3f(0,0,0));
        m_stageCam->setBeacon(m_stageCamBeaconNode);
        m_stageCam->setNear(0.5f);
        m_stageCam->setFar(8000.0);
        m_stageCam->setFov(OSG::osgDegree2Rad(60));
        m_stage->setMusician(m_stageCam);
        m_stageNode->addChild(m_stageVisitNode);
        OSG::GradientBackgroundRefPtr gb = OSG::GradientBackground::create();
        gb->addLine(OSG::Color3f(1, 1, 1), 0.0);
        gb->addLine(OSG::Color3f(0, 0, 0), 1.0);
        m_stage->setBackground(gb);
        m_zone->addChild(m_stageNode);
*/

        OSG::commitChanges();
    }
}

Musician::~Musician() {}

void Musician::parse(xmlNodePtr node) {
    Camera::parse(node);

    char* value=NULL;
    value = (char*)xmlGetProp(node,(xmlChar*)"color");
    if(value!=NULL) {
        setColor(atof(value));
    }

    value=NULL;
    value = (char*)xmlGetProp(node,(xmlChar*)"passages_height");
    if(value!=NULL) {
        setPassagesHeight(atof(value));;
    }

    value=NULL;
    value = (char*)xmlGetProp(node,(xmlChar*)"signs");
    if(value!=NULL) {
        setSigns(std::string(value));
    }
}

void Musician::reset() {
    Camera::reset();
    vector<Piivert*>::iterator itPiiv;
    for(itPiiv=m_piiverts.begin(); itPiiv!=m_piiverts.end(); ++itPiiv) {
        delete (*itPiiv);
    }
}

void Musician::setParameter(const CAMERA_PARAMETER& param, double* values, 
                                unsigned int nbValues) {
    m_parametersValues[param].assign(values, values+nbValues);
    m_parametersChanged[param]=true;
}

void Musician::setCurrentRoom(Room* currentRoom) {
    m_currentRoom=currentRoom;
    m_currentZone=currentRoom->getCreateZone(m_id);
}

void Musician::setColor(const double& color) {
    m_color=color;
    vector<Piivert*>::iterator itPiiv;
    for(itPiiv=m_piiverts.begin(); itPiiv!=m_piiverts.end(); ++itPiiv) {
        (*itPiiv)->setColor(m_color);
    }
    vector<Passage*>::iterator itPass;
    for(itPass=m_passages.begin();itPass!=m_passages.end();++itPass) {
        (*itPass)->getOtherPassage()->setColor(color);
    }
}

void Musician::addPiivert(Piivert* newPiivert) {
    m_piiverts.push_back(newPiivert);
    newPiivert->setColor(m_color);
    m_camBeacon->addChild(newPiivert->getNode());    
}

void Musician::setPassagesHeight(const float& h) {
    OSG::Vec3f& pos = m_passagesRootTrans->editTranslation();
    pos[1]=h;
}

void Musician::testTunnelPassagesCollision(Tunnel* tunnel, 
                                        std::vector<Event>& events, 
                                        const bool& released) {
    if(released) {
        //test collision with passages
        vector<Passage*>::iterator itPassage;
        for(itPassage=m_passages.begin(); 
                    itPassage!=m_passages.end(); ++itPassage) {
            Passage* passage = (*itPassage);
            passage->testCollisionWithTunnel(tunnel);
        }
    }
}


void Musician::testSignPassagesCollision(Sign* sign, 
                                        std::vector<Event>& events) {
    //test collision with passages
    vector<Passage*>::iterator itPassage;
    for(itPassage=m_passages.begin(); 
                itPassage!=m_passages.end(); ++itPassage) {
        Passage* passage = (*itPassage);
        passage->testCollisionWithSign(sign);
    }
}

void Musician::testWormPassagesCollision(Worm* worm, 
                                        std::vector<Event>& events, 
                                        const bool& released) {

    //test collision with passages and tunnels and worms inside passages
    vector<Passage*>::iterator itPassage;
    for(itPassage=m_passages.begin(); 
                itPassage!=m_passages.end(); ++itPassage) {
        Passage* passage = (*itPassage);
        if(released) {
            passage->testCollisionWithWorm(worm);
            if(passage->getGhostWorms().size()>0) {
                std::vector<Worm*>::const_iterator itGhost;
                for(itGhost=passage->getGhostWorms().begin(); 
                        itGhost!=passage->getGhostWorms().end();itGhost++) {
                    if(worm->getID()!=(*itGhost)->getID()) {
                        worm->testCollisionWithWorm((*itGhost),events,released);
                    }
                }
            }
        }
        if(passage->getGhostTunnels().size()>0) {
            std::vector<Tunnel*>::const_iterator itGhost;
            for(itGhost=passage->getGhostTunnels().begin(); 
                    itGhost!=passage->getGhostTunnels().end();itGhost++)
            (*itGhost)->testCollisionWithWorm(worm);
        }
    }
}

void Musician::setSigns(const std::string& signs) {

    m_signsStr=signs;

    vector<Sign*>::iterator itSign;
    for(itSign=m_signs.begin();itSign!=m_signs.end();++itSign) {
        delete (*itSign);
    }
    m_signs.clear();
    m_signsRoot->clearChildren();

    size_t pos = m_signsStr.find_first_of(';');    
    unsigned int c=0;
    while(pos!=m_signsStr.npos) {
        Sign* newSign=new Sign(c, m_signsStr.substr(0,pos));
        m_signs.push_back(newSign);
        newSign->setPosition(
                    OSG::Vec3f(-0.5, float(c)*0.1+m_passagesHeight, 0));
        m_signsRoot->addChild(newSign->getNode());
        m_signsStr = m_signsStr.substr(pos+1);
        pos = m_signsStr.find_first_of(';');    
        c++;
    }
}

void Musician::updatePassagesSignsPositions() {
    for(unsigned int s=0;s<m_signs.size();++s) {
        m_signs[s]->setPosition(
                    OSG::Vec3f(-0.5, float(s)*0.1+m_passagesHeight, 0));
    }
    if(m_passages.size()>0) {
        //reset positions of passages horizontally and centered
        double pos=-(m_passages[0]->getMaxDistanceBetweenPassages()
                                *(double)m_passages.size()) / 2.0;
        double step = m_passages[0]->getMaxDistanceBetweenPassages();
        vector<Passage*>::iterator itPass;
        for(itPass=m_passages.begin();itPass!=m_passages.end();++itPass) {
            (*itPass)->setPosition(
                        OSG::Vec3f(pos, m_passagesHeight, 0));
            pos+=step;
        }
    }
}

void Musician::addPassage(Passage* passage) {
    passage->setParentMusician(this);
    m_passages.push_back(passage);
    m_passagesRoot->addChild(passage->getNode());
    passage->setZone(m_currentZone);
    updatePassagesSignsPositions();
    updatePassagesGroups();
}

void Musician::updatePassagesGroups() {
    if(m_passages.size()>0) {
        //define groups
        float maxDist=m_passages[0]->getMaxDistanceBetweenPassages();
        unsigned int pas=0;
        while(pas<m_passages.size()) {
            std::vector<Passage*> group;
            float dist=0;
            while(dist<maxDist) {
                group.push_back(m_passages[pas]);
                if(pas+1<m_passages.size()) {
                    dist=m_passages[pas+1]->getPosition()[0] - 
                            m_passages[pas]->getPosition()[0];
                }
                else {
                    dist=maxDist;
                }
                pas++;
            }
            for(unsigned int p=0; p<group.size(); ++p) {
                group[p]->clearGroupPassages();
                for(unsigned int p2=0; p2<group.size(); ++p2) {
                    group[p]->addPassageToGroup(group[p2]);
                }
            }
        }

        //send all
        for(unsigned int p=0; p<m_passages.size(); ++p) {
            m_passages[p]->clearAllPassages();
            for(unsigned int p2=0;p<m_passages.size();++p) {
                m_passages[p]->addPassageToAll(m_passages[p2]);
            }
        }
    }
}

void Musician::addGhostNode(OSG::NodeRecPtr ghostNode) {
    m_ghostsRoot->addChild(ghostNode);
}

void Musician::removeGhostNode(OSG::NodeRecPtr ghostNode) {
    m_ghostsRoot->subChild(ghostNode);
}

void Musician::update() {
    //update passages and send musical activity to connected passages
    vector<Passage*>::iterator itPassage=m_passages.begin();
    for(; itPassage!=m_passages.end(); ++itPassage) {
        (*itPassage)->update();
        //(*itPassage)->getOtherPassage()->setActivity(activity);
    }

    Camera::update();
}



