/***************************************************************************
 *  Musician.hpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _Musician_H
#define _Musician_H


#include "Camera.hpp"

class Piivert;
class Zone;
class Sign;
class Tunnel;
class Passage;
class Worm;
class Room;

class Musician: public Camera {
    public:
        Musician(int, OSG::NodeRecPtr, OSG::PassiveWindowRecPtr);
        ~Musician();
        void parse(xmlNodePtr);
        void reset();

        void setParameter(const CAMERA_PARAMETER& param, double* values, 
                                unsigned int nbValues);

        inline const double& getColor(){return m_color;}
        void setColor(const double&);

        void addPiivert(Piivert*);

        void update();

        inline unsigned int getNbPiiverts(){return m_piiverts.size();}

        void addPassage(Passage* passage);
        void updatePassagesGroups();
        void setSigns(const std::string&);
        void updatePassagesSignsPositions();
        void addGhostNode(OSG::NodeRecPtr);
        void removeGhostNode(OSG::NodeRecPtr);
        void setPassagesHeight(const float&);
        void testTunnelPassagesCollision(Tunnel* tunnel, 
                                        std::vector<Event>& events, 
                                        const bool& released);
        void testSignPassagesCollision(Sign* sign, 
                                        std::vector<Event>& events);
        void testWormPassagesCollision(Worm* worm, 
                                        std::vector<Event>& events, 
                                        const bool& released);

        inline Zone* getCurrentZone(){return m_currentZone;}
        void setCurrentRoom(Room* currentRoom);

    private:
        double m_color;

        std::vector<Piivert*> m_piiverts;

        Room* m_currentRoom;
        Zone* m_currentZone;

        //passages
        float m_passagesHeight;
        std::vector<Passage*> m_passages;
        std::vector<Sign*> m_signs;
        OSG::NodeRecPtr m_passagesRoot;
        OSG::ComponentTransformRecPtr m_passagesRootTrans;
        OSG::NodeRecPtr m_signsRoot;
        OSG::ComponentTransformRecPtr m_signsRootTrans;
        OSG::NodeRecPtr m_ghostsRoot;
        OSG::ComponentTransformRecPtr m_ghostsRootTrans;
        OSG::SimpleMaterialRecPtr m_fboMaterial;    
        std::string m_signsStr;
};



#endif
