/***************************************************************************
 *  Passage.cpp
 *  Part of Drile
 *  2010- Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Passage.hpp"
#include "EnvManager.hpp"
#include "Room.hpp"
#include "Zone.hpp"
#include "Worm.hpp"
#include "Tunnel.hpp"
#include "Sign.hpp"


using namespace std;


Passage::Passage():m_nbUsers(0), 
                    m_diameterSend(0.2), 
                    m_diameterIndiv(0.2),
                    m_diameterGroup(0.4),
                    m_diameterAll(0.6),
                    m_heightSend(0.2), 
                    m_heightShare(0.2),
                    m_signalCounter(0) {

    m_passage = OSG::Node::create();
    m_passageTrans = OSG::ComponentTransform::create();
    m_passage->setCore(m_passageTrans);

    OSG::VoidPAttachmentRefPtr ptr = OSG::VoidPAttachment::create();
    ptr->editFieldPtr()->setValue(this);
    m_passage->addAttachment(ptr);
    

    //Send level
    makeSendNode();
    m_passage->addChild(m_sendNode);
    double height=m_heightSend/2.0+m_heightShare/2.0;
    m_shareIndivNode = OSG::Node::create();
    m_shareIndivMat = OSG::SimpleMaterial::create();
    makeShareNode(m_shareIndivNode, m_shareIndivMat, 
                        height, m_diameterSend, 
                        m_diameterIndiv, 0.4);
    m_passage->addChild(m_shareIndivNode);
    //group share level
    height+=m_heightShare;
    m_shareGroupNode = OSG::Node::create();
    m_shareGroupMat = OSG::SimpleMaterial::create();
    makeShareNode(m_shareGroupNode, m_shareGroupMat, 
                        height, m_diameterIndiv, 
                        m_diameterGroup, 0.7);
    m_passage->addChild(m_shareGroupNode);
    //all share level
    height+=m_heightShare;
    m_shareAllNode = OSG::Node::create();
    m_shareAllMat = OSG::SimpleMaterial::create();
    makeShareNode(m_shareAllNode, m_shareAllMat, 
                        height, m_diameterGroup, 
                        m_diameterAll, 0.9);
    m_passage->addChild(m_shareAllNode);

    OSG::GeometryRefPtr geo2 = OSG::makeBoxGeo(m_diameterIndiv, 
                (m_heightSend+m_heightShare*3), m_diameterIndiv, 1, 1, 1);
    m_highlightMaterial = OSG::SimpleMaterial::create();
    m_highlightMaterial->setLit(false);
    m_highlightMaterial->setColorMaterial(GL_NONE);
    m_highlightMaterial->setDiffuse(OSG::Color3f(0.5, 0.5, 0.5));
    m_highlightMaterial->setTransparency(1);
    OSG::PolygonChunkRefPtr chunk = OSG::PolygonChunk::create();
    chunk->setFrontMode(GL_LINE);
    chunk->setBackMode(GL_LINE);
    m_highlightMaterial->addChunk(chunk);
    geo2->setMaterial(m_highlightMaterial);
    m_highlight = OSG::Node::create();
    m_highlight->setCore(geo2);
    m_highlight->addAttachment(ptr);
    OSG::NodeRecPtr tempHlTransNode = OSG::Node::create();
    OSG::ComponentTransformRecPtr tempHlTrans = 
                                OSG::ComponentTransform::create();
    tempHlTrans->setTranslation(
                    OSG::Vec3f(0,(m_heightShare*3)/2.0,0));
    tempHlTransNode->setCore(tempHlTrans);
    tempHlTransNode->addChild(m_highlight);
    m_passage->addChild(tempHlTransNode);

    m_virtualMusicianGeom = OSG::makePlaneGeo(m_diameterSend*4, m_diameterSend*4, 
                                                1, 1);
    m_virtualMusicianGeom->setDlistCache(false);

    m_virtualMusicianGeom->setMaterial(m_otherZoneFboMaterial);

    m_virtualMusicianNode = OSG::Node::create();
    m_virtualMusicianNode->setCore(m_virtualMusicianGeom);
    m_virtualMusicianTrans = OSG::ComponentTransform::create();
    m_virtualMusicianTransNode = OSG::Node::create();
    m_virtualMusicianTransNode->setCore(m_virtualMusicianTrans);
    m_virtualMusicianTransNode->addChild(m_virtualMusicianNode);
    //m_passage->addChild(m_virtualMusicianTransNode);
}

Passage::~Passage(){}

void Passage::makeSendNode() {
    OSG::GeometryRecPtr geo = OSG::makeConicalFrustumGeo(m_heightSend, 
                                        m_diameterSend/2.0, m_diameterSend/2.0, 
                                        10, true, true, true); 
    m_sendMat = OSG::SimpleMaterial::create();
    m_sendMat->setLit(true);
    m_sendMat->setColorMaterial(GL_AMBIENT_AND_DIFFUSE);
    m_sendMat->setDiffuse(OSG::Color3f(0.7,0.7,0.7));
    m_sendMat->setAmbient(OSG::Color3f(0.5,0.5,0.5));
    OSG::PolygonChunkRefPtr matchunk = OSG::PolygonChunk::create();
    matchunk->setCullFace(GL_BACK);
    m_sendMat->addChunk(matchunk);
    geo->setMaterial(m_sendMat);
    OSG::NodeRecPtr tempGeomNode = OSG::Node::create();
    tempGeomNode->setCore(geo);
    OSG::ComponentTransformRecPtr tempTrans 
                = OSG::ComponentTransform::create();
    m_sendNode = OSG::Node::create();
    m_sendNode->setCore(tempTrans);
    m_sendNode->addChild(tempGeomNode);
}

void Passage::makeShareNode(OSG::NodeRecPtr node, 
                    OSG::SimpleMaterialRecPtr mat, 
                    const double& yPos, const double& minDiam, 
                    const double& maxDiam, const double& transparency) {

    OSG::GeometryRecPtr geo = OSG::makeConicalFrustumGeo(m_heightShare, 
                            maxDiam/2.0, minDiam/2.0, 10, true, true, false); 
    mat->setLit(true);
    mat->setColorMaterial(GL_AMBIENT_AND_DIFFUSE);
    mat->setDiffuse(OSG::Color3f(0.7,0.7,0.7));
    mat->setAmbient(OSG::Color3f(0.5,0.5,0.5));
    mat->setTransparency(transparency);
    OSG::PolygonChunkRefPtr matchunk = OSG::PolygonChunk::create();
    matchunk->setCullFace(GL_BACK);
    mat->addChunk(matchunk);
    geo->setMaterial(mat);
    OSG::NodeRecPtr tempGeomNode = OSG::Node::create();
    tempGeomNode->setCore(geo);
    OSG::ComponentTransformRecPtr tempTrans 
                = OSG::ComponentTransform::create();
    tempTrans->setTranslation(OSG::Vec3f(0,yPos,0));
    node->setCore(tempTrans);
    node->addChild(tempGeomNode);
}

void Passage::update() {
    if(m_signalCounter>0) {
        m_passageTrans->editScale()[1]=1.0+0.3*m_signalCounter;
        m_signalCounter-=0.1;
    }
}

void Passage::setPosition(const OSG::Vec3f& pos) {
    m_position=pos;
    m_passageTrans->setTranslation(m_position);
}

void Passage::setActivity(const double& activity) {
    m_passageTrans->editScale()[0]=1+activity*2.0;
    m_passageTrans->editScale()[2]=1+activity*2.0;
}

void Passage::setColor(const double& color) {
    OSG::Color3f &sendColor = m_sendMat->editDiffuse();
    sendColor.setValuesHSV(color*360.0, 0.6, 0.8); 
    OSG::Color3f &shareIndivColor = m_shareIndivMat->editDiffuse();
    shareIndivColor.setValuesHSV(color*360.0, 0.5, 0.9); 
    OSG::Color3f &shareGroupColor = m_shareGroupMat->editDiffuse();
    shareGroupColor.setValuesHSV(color*360.0, 0.3, 1); 
}

void Passage::testCollisionLevel(Ghostable* ghostable, int& level) {

    level=-1;
    OSG::BoxVolume ghostableVolume;
    ghostable->getCollisionNode()->getWorldVolume(ghostableVolume);

    OSG::BoxVolume passageVolume;
    m_passage->getWorldVolume(passageVolume);

    if(passageVolume.intersect(ghostableVolume)) {
        OSG::BoxVolume sendVolume;
        m_sendNode->getWorldVolume(sendVolume);
        if(sendVolume.intersect(ghostableVolume)) { //send level
            level=0;
        }
        else {
            OSG::BoxVolume shareIndivVolume;
            m_shareIndivNode->getWorldVolume(shareIndivVolume);
            if(shareIndivVolume.intersect(ghostableVolume)) {
                level=1;
            }
            else {
                OSG::BoxVolume shareGroupVolume;
                m_shareGroupNode->getWorldVolume(shareGroupVolume);
                if(shareGroupVolume.intersect(ghostableVolume)) {
                    level=2;
                }
                else {
                    OSG::BoxVolume shareAllVolume;
                    m_shareAllNode->getWorldVolume(shareAllVolume);
                    if(shareAllVolume.intersect(ghostableVolume)) {
                        level=3;
                    }
                }
            }
        }
    }
}

void Passage::sendSignal(){
    m_otherPassage->receiveSignal();
}

void Passage::receiveSignal(){
    m_signalCounter=1.0;
}

void Passage::setHighlight(const bool& highlight, const double& hue) {
    if((!highlight && m_nbUsers==0) || (highlight && m_nbUsers>0)) {
        m_highlightMaterial->setTransparency(!highlight);
        OSG::Color3f &color = m_highlightMaterial->editDiffuse();
        color.setValuesHSV(hue*360.0, 0.6, 0.8); 
    }
}


//TUNNELS
void Passage::testCollisionWithTunnel(Tunnel* tunnel) {
    int level=-1;
    testCollisionLevel(tunnel,level);
    switch(level) {
        case 0: { //send level
            if(m_tunnelsMap.find(tunnel->getID())!= m_tunnelsMap.end()) {
                    unshareTunnel(tunnel);
            }
            m_parentCam->getCurrentZone()->removeTunnel(tunnel);
            m_otherPassage->getZone()->addTunnel(tunnel);
            OSG::Vec3f &newPosTunnel = tunnel->getTrans()->editTranslation();
            newPosTunnel[0]=m_otherPassage->getPosition()[0];
            newPosTunnel[1]=m_otherPassage->getPosition()[1]
                            +m_heightSend+3.0*m_heightShare;
            newPosTunnel[2]=m_otherPassage->getPosition()[2];
        }break;
        case 1: { //share indiv level
            if(m_tunnelsMap.find(tunnel->getID())!=
                                    m_tunnelsMap.end()) {
                if(m_tunnelsMap[tunnel->getID()].first!=1) {
                    unshareTunnel(tunnel);
                    shareIndivTunnel(tunnel);
                }
            }
            else {
                shareIndivTunnel(tunnel);
            }
        }break;
        case 2: { //share group level
            if(m_tunnelsMap.find(tunnel->getID())!=
                                    m_tunnelsMap.end()) {
                if(m_tunnelsMap[tunnel->getID()].first!=2) {
                    unshareTunnel(tunnel);
                    shareGroupTunnel(tunnel);
                }
            }
            else {
                shareGroupTunnel(tunnel);
            }
        }break;
        case 3: { //share all level
            if(m_tunnelsMap.find(tunnel->getID())!=
                                    m_tunnelsMap.end()) {
                if(m_tunnelsMap[tunnel->getID()].first!=3) {
                    unshareTunnel(tunnel);
                    shareAllTunnel(tunnel);
                }
            }
            else {
                shareAllTunnel(tunnel);
            }
        }break;
        case -1: { //do not intersect
            unshareTunnel(tunnel);
        }
    }
}

void Passage::addGhostTunnel(Tunnel* ghost) {
    m_ghostTunnels.push_back(ghost);
    m_parentCam->addGhostNode(ghost->getNode());
    updateGhostsPositions();
}

void Passage::removeGhostTunnel(Tunnel* ghost) {
    vector<Tunnel*>::iterator itTun;
    for(itTun=m_ghostTunnels.begin();itTun!=m_ghostTunnels.end();itTun++) {
        if((*itTun)->getID()==ghost->getID()) {
            m_ghostTunnels.erase(itTun);
            break;
        }
    }
    m_parentCam->removeGhostNode(ghost->getNode());
    updateGhostsPositions();
}

void Passage::unshareTunnel(Tunnel* tunnel) {
    if(m_tunnelsMap.find(tunnel->getID())!=
                            m_tunnelsMap.end()) {
        tunnel->setStretch(1.0);
        //remove colliding tunnel
        m_tunnelsMap.erase(tunnel->getID());
        //remove local ghost
        removeGhostTunnel(tunnel);
        //from all
        vector<Passage*>::iterator itAll;
        for(itAll=m_allPassages.begin();itAll!=m_allPassages.end();++itAll) {
            (*itAll)->getOtherPassage()->removeGhostTunnel(tunnel);
        }
        //from group
        vector<Passage*>::iterator itGroup;
        for(itGroup=m_groupPassages.begin();
                itGroup!=m_groupPassages.end();++itGroup) {
            (*itGroup)->getOtherPassage()->removeGhostTunnel(tunnel);
        }
        //remove ghosts 
        tunnel->deleteAllGhosts();
    }
}

void Passage::shareIndivTunnel(Tunnel* tunnel) {
    tunnel->setStretch(m_diameterIndiv);
    m_tunnelsMap[tunnel->getID()]=pair<unsigned int, Tunnel*>(1,tunnel);
    Tunnel* newGhost = static_cast<Tunnel*>(tunnel->createGhost());
    m_otherPassage->addGhostTunnel(newGhost);
}

void Passage::shareGroupTunnel(Tunnel* tunnel) {
    tunnel->setStretch(m_diameterIndiv);
    m_tunnelsMap[tunnel->getID()]=pair<unsigned int, Tunnel*>(2,tunnel);
    Tunnel* localGhost = static_cast<Tunnel*>(tunnel->createGhost());
    addGhostTunnel(localGhost);
    vector<Passage*>::iterator itGroup;
    for(itGroup=m_groupPassages.begin();
            itGroup!=m_groupPassages.end();++itGroup) {
        Tunnel* newGhost = static_cast<Tunnel*>(tunnel->createGhost());
        (*itGroup)->getOtherPassage()->addGhostTunnel(newGhost);
    }
}

void Passage::shareAllTunnel(Tunnel* tunnel) {
    tunnel->setStretch(m_diameterIndiv);
    m_tunnelsMap[tunnel->getID()]=pair<unsigned int, Tunnel*>(3,tunnel);
    Tunnel* localGhost = static_cast<Tunnel*>(tunnel->createGhost());
    addGhostTunnel(localGhost);
    vector<Passage*>::iterator itAll;
    for(itAll=m_allPassages.begin();
            itAll!=m_allPassages.end();++itAll) {
        Tunnel* newGhost = static_cast<Tunnel*>(tunnel->createGhost());
        (*itAll)->getOtherPassage()->addGhostTunnel(newGhost);
    }
}

//WORMS
void Passage::testCollisionWithWorm(Worm* worm) {
    int level=-1;
    testCollisionLevel(worm,level);
    switch(level) {
        case 0: { //send level
            if(m_wormsMap.find(worm->getID())!=m_wormsMap.end()) {
                    unshareWorm(worm);
            }
            m_parentCam->getCurrentZone()->removeWorm(worm);
            m_otherPassage->getZone()->addWorm(worm);
            OSG::Vec3f &newPosWorm = worm->getTrans()->editTranslation();
            newPosWorm[0]=m_otherPassage->getPosition()[0];
            newPosWorm[1]=m_otherPassage->getPosition()[1]
                            +m_heightSend+3.0*m_heightShare;
            newPosWorm[2]=m_otherPassage->getPosition()[2];
        }break;
        case 1: { //share indiv level
            if(m_wormsMap.find(worm->getID())!=
                                    m_wormsMap.end()) {
                if(m_wormsMap[worm->getID()].first!=1) {
                    unshareWorm(worm);
                    shareIndivWorm(worm);
                }
            }
            else {
                shareIndivWorm(worm);
            }
            cout<<"share worm indiv"<<endl;
        }break;
        case 2: { //share group level
            if(m_wormsMap.find(worm->getID())!=
                                    m_wormsMap.end()) {
                if(m_wormsMap[worm->getID()].first!=2) {
                    unshareWorm(worm);
                    shareGroupWorm(worm);
                }
            }
            else {
                shareGroupWorm(worm);
            }
            cout<<"share worm group"<<endl;
        }break;
        case 3: { //share all level
            if(m_wormsMap.find(worm->getID())!=
                                    m_wormsMap.end()) {
                if(m_wormsMap[worm->getID()].first!=3) {
                    unshareWorm(worm);
                    shareAllWorm(worm);
                }
            }
            else {
                shareAllWorm(worm);
            }
            cout<<"share worm all"<<endl;
        }break;
        case -1: { //do not intersect
            unshareWorm(worm);
            cout<<"unshare worm"<<endl;
        }break;
    }
}

void Passage::addGhostWorm(Worm* ghost) {
    m_ghostWorms.push_back(ghost);
    m_parentCam->addGhostNode(ghost->getNode());
    cout<<"added ghost worm"<<endl;
    updateGhostsPositions();
}

void Passage::removeGhostWorm(Worm* ghost) {
    vector<Worm*>::iterator itTun;
    for(itTun=m_ghostWorms.begin();itTun!=m_ghostWorms.end();itTun++) {
        if((*itTun)->getID()==ghost->getID()) {
            cout<<"found worm to remove"<<endl;
            m_ghostWorms.erase(itTun);
            break;
        }
    }
    m_parentCam->removeGhostNode(ghost->getNode());
    updateGhostsPositions();
}

void Passage::unshareWorm(Worm* worm) {
    cout<<"try unshare worm !!"<<endl;
    if(m_wormsMap.find(worm->getID())!=
                            m_wormsMap.end()) {
        cout<<"really unshare worm !!"<<endl;
        //remove colliding worm
        m_wormsMap.erase(worm->getID());
        //remove local ghost
        removeGhostWorm(worm);
        m_otherPassage->removeGhostWorm(worm);
        //from all
        vector<Passage*>::iterator itAll;
        for(itAll=m_allPassages.begin();itAll!=m_allPassages.end();++itAll) {
            (*itAll)->getOtherPassage()->removeGhostWorm(worm);
        }
        //from group
        vector<Passage*>::iterator itGroup;
        for(itGroup=m_groupPassages.begin();
                itGroup!=m_groupPassages.end();++itGroup) {
            (*itGroup)->getOtherPassage()->removeGhostWorm(worm);
        }
        //remove ghosts 
        worm->deleteAllGhosts();
    }
}

void Passage::shareIndivWorm(Worm* worm) {
    m_wormsMap[worm->getID()]=pair<unsigned int, Worm*>(1,worm);
    Worm* newGhost = static_cast<Worm*>(worm->createGhost());
    m_otherPassage->addGhostWorm(newGhost);
}

void Passage::shareGroupWorm(Worm* worm) {
    m_wormsMap[worm->getID()]=pair<unsigned int, Worm*>(2,worm);
    Worm* localGhost = static_cast<Worm*>(worm->createGhost());
    addGhostWorm(localGhost);
    vector<Passage*>::iterator itGroup;
    for(itGroup=m_groupPassages.begin();
            itGroup!=m_groupPassages.end();++itGroup) {
        Worm* newGhost = static_cast<Worm*>(worm->createGhost());
        (*itGroup)->getOtherPassage()->addGhostWorm(newGhost);
    }
}

void Passage::shareAllWorm(Worm* worm) {
    m_wormsMap[worm->getID()]=pair<unsigned int, Worm*>(3,worm);
    Worm* localGhost = static_cast<Worm*>(worm->createGhost());
    addGhostWorm(localGhost);
    vector<Passage*>::iterator itAll;
    for(itAll=m_allPassages.begin();
            itAll!=m_allPassages.end();++itAll) {
        Worm* newGhost = static_cast<Worm*>(worm->createGhost());
        (*itAll)->getOtherPassage()->addGhostWorm(newGhost);
    }
}


//SIGN
void Passage::testCollisionWithSign(Sign* sign) {
    int level=-1;
    testCollisionLevel(sign,level);
    switch(level) {
        case 1: { //share indiv level
            if(m_signsMap.find(sign->getID())!= m_signsMap.end()) {
                if(m_signsMap[sign->getID()].first!=1) {
                    unshareSign(sign);
                    shareIndivSign(sign);
                }
            }
            else {
                shareIndivSign(sign);
            }
        }break;
        case 2: { //share group level
            if(m_signsMap.find(sign->getID())!= m_signsMap.end()) {
                if(m_signsMap[sign->getID()].first!=2) {
                    unshareSign(sign);
                    shareGroupSign(sign);
                }
            }
            else {
                shareGroupSign(sign);
            }
        }break;
        case 3: { //share all level
            if(m_signsMap.find(sign->getID())!= m_signsMap.end()) {
                if(m_signsMap[sign->getID()].first!=3) {
                    unshareSign(sign);
                    shareAllSign(sign);
                }
            }
            else {
                shareAllSign(sign);
            }
        }break;
        case -1: { //do not intersect
            unshareSign(sign);
        }break;
        default:break;
    }
}

void Passage::addGhostSign(Sign* ghost) {
    m_ghostSigns.push_back(ghost);
    m_parentCam->addGhostNode(ghost->getNode());
    updateGhostsPositions();
}

void Passage::removeGhostSign(Sign* ghost) {
    vector<Sign*>::iterator itTun;
    for(itTun=m_ghostSigns.begin();itTun!=m_ghostSigns.end();itTun++) {
        if((*itTun)->getID()==ghost->getID()) {
            m_ghostSigns.erase(itTun);
            break;
        }
    }
    m_parentCam->removeGhostNode(ghost->getNode());
    updateGhostsPositions();
}

void Passage::unshareSign(Sign* sign) {
    if(m_signsMap.find(sign->getID())!=
                            m_signsMap.end()) {
        //remove colliding sign
        m_signsMap.erase(sign->getID());
        //remove local ghost
        removeGhostSign(sign);
        //from all
        vector<Passage*>::iterator itAll;
        for(itAll=m_allPassages.begin();itAll!=m_allPassages.end();++itAll) {
            (*itAll)->getOtherPassage()->removeGhostSign(sign);
        }
        //from group
        vector<Passage*>::iterator itGroup;
        for(itGroup=m_groupPassages.begin();
                itGroup!=m_groupPassages.end();++itGroup) {
            (*itGroup)->getOtherPassage()->removeGhostSign(sign);
        }
        //remove ghosts 
        sign->deleteAllGhosts();
    }
}

void Passage::shareIndivSign(Sign* sign) {
    m_signsMap[sign->getID()]=pair<unsigned int, Sign*>(1,sign);
    Sign* newGhost = static_cast<Sign*>(sign->createGhost());
    m_otherPassage->addGhostSign(newGhost);
}

void Passage::shareGroupSign(Sign* sign) {
    m_signsMap[sign->getID()]=pair<unsigned int, Sign*>(2,sign);
    Sign* localGhost = static_cast<Sign*>(sign->createGhost());
    addGhostSign(localGhost);
    vector<Passage*>::iterator itGroup;
    for(itGroup=m_groupPassages.begin();
            itGroup!=m_groupPassages.end();++itGroup) {
        Sign* newGhost = static_cast<Sign*>(sign->createGhost());
        (*itGroup)->getOtherPassage()->addGhostSign(newGhost);
    }
}

void Passage::shareAllSign(Sign* sign) {
    m_signsMap[sign->getID()]=pair<unsigned int, Sign*>(3,sign);
    Sign* localGhost = static_cast<Sign*>(sign->createGhost());
    addGhostSign(localGhost);
    vector<Passage*>::iterator itAll;
    for(itAll=m_allPassages.begin();
            itAll!=m_allPassages.end();++itAll) {
        Sign* newGhost = static_cast<Sign*>(sign->createGhost());
        (*itAll)->getOtherPassage()->addGhostSign(newGhost);
    }
}

void Passage::setOtherZoneFboMaterial(OSG::SimpleMaterialRecPtr material) {
    m_otherZoneFboMaterial=material;
    m_virtualMusicianGeom->setMaterial(m_otherZoneFboMaterial);
}

void Passage::updateGhostsPositions() {
    unsigned int nbGhosts = 
        m_ghostTunnels.size()+m_ghostWorms.size()+m_ghostSigns.size();
    if(nbGhosts>0) {
        //update positions, worms first then tunnels, then signs
        float step = m_diameterIndiv / float(nbGhosts);
        float counter = -m_diameterIndiv / 2.0;
    
        vector<Worm*>::iterator itWorm;
        for(itWorm=m_ghostWorms.begin();itWorm!=m_ghostWorms.end();itWorm++) {
            (*itWorm)->getTrans()->setTranslation(m_position+
                    OSG::Vec3f(counter,m_heightSend,0));
            counter+=step;
        }
        vector<Tunnel*>::iterator itTunnel;
        for(itTunnel=m_ghostTunnels.begin();
                itTunnel!=m_ghostTunnels.end();itTunnel++) {
            (*itTunnel)->getTrans()->setTranslation(m_position+
                    OSG::Vec3f(counter,m_heightSend,0));
            counter+=step;
        }
        vector<Sign*>::iterator itSign;
        for(itSign=m_ghostSigns.begin();
                itSign!=m_ghostSigns.end();itSign++) {
            (*itSign)->getTrans()->setTranslation(m_position+
                OSG::Vec3f(counter,m_heightSend,0));
            counter+=step;
        }
    }
}

