/***************************************************************************
 *  Passage.hpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _PASSAGE_H
#define _PASSAGE_H

#include <OpenSG/OSGSimpleGeometry.h>
#include <OpenSG/OSGComponentTransform.h>
#include <OpenSG/OSGMaterialGroup.h>
#include <OpenSG/OSGSimpleMaterial.h>
#include <OpenSG/OSGNameAttachment.h>
#include <OpenSG/OSGPolygonChunk.h>


#include "Ghostable.hpp"

class Worm;
class Zone;
class Tunnel;
class Sign;
class Musician;

class Passage {
    public:
        Passage();
        ~Passage();

        void update();

        inline void setOtherPassage(Passage* passage){m_otherPassage=passage;}
        inline Passage* getOtherPassage(){return m_otherPassage;}

        void setOtherZoneFboMaterial(OSG::SimpleMaterialRecPtr material);
        
        inline const OSG::Vec3f& getPosition(){return m_position;}
        void setPosition(const OSG::Vec3f&);
        inline OSG::NodeRecPtr getNode(){return m_passage;}

        void setColor(const double&);

        void testCollisionWithTunnel(Tunnel*);
        void testCollisionWithWorm(Worm*);
        void testCollisionWithSign(Sign*);

        inline void addPassageToAll(Passage* pass){
                                        m_allPassages.push_back(pass);}
        inline void clearAllPassages(){m_allPassages.clear();}

        inline void addPassageToGroup(Passage* pass){
                                        m_groupPassages.push_back(pass);}
        inline void clearGroupPassages(){m_groupPassages.clear();}

        void sendSignal();
        void receiveSignal();

        void setActivity(const double&);

        void setHighlight(const bool&, const double& hue=0);
        inline void addUser(){m_nbUsers++;}
        inline void removeUser(){m_nbUsers--;if(m_nbUsers<0){m_nbUsers=0;}}

        inline const double& getMinDistanceBetweenPassages(){
                                                return m_diameterIndiv;}
        inline const double& getMaxDistanceBetweenPassages(){
                                                return m_diameterGroup;}

        inline const std::vector<Tunnel*>& getGhostTunnels(){
                                                return m_ghostTunnels;}

        inline const std::vector<Worm*>& getGhostWorms(){return m_ghostWorms;}


        inline void setZone(Zone* zone){m_zone=zone;}
        inline Zone* getZone(){return m_zone;}


        inline void setParentMusician(Musician* cam){m_parentCam=cam;}
        inline Musician* getParentMusician(){return m_parentCam;}

    private:
        std::string m_name;
        int m_nbUsers;

        OSG::NodeRecPtr m_passage;
        OSG::ComponentTransformRecPtr m_passageTrans;

        Zone* m_zone;
        Musician* m_parentCam;

        void makeSendNode();
        void makeShareNode(OSG::NodeRecPtr node, OSG::SimpleMaterialRecPtr mat, 
                            const double& yPos, const double& minDiam, 
                            const double& maxDiam, const double& transparency);

        OSG::NodeRecPtr m_sendNode;
        OSG::SimpleMaterialRecPtr m_sendMat;
        OSG::NodeRecPtr m_shareIndivNode;
        OSG::SimpleMaterialRecPtr m_shareIndivMat;
        OSG::NodeRecPtr m_shareGroupNode;
        OSG::SimpleMaterialRecPtr m_shareGroupMat;
        OSG::NodeRecPtr m_shareAllNode;
        OSG::SimpleMaterialRecPtr m_shareAllMat;


        OSG::NodeRefPtr m_highlight;
        OSG::SimpleMaterialRefPtr m_highlightMaterial;

        OSG::GeometryRecPtr m_virtualMusicianGeom;
        OSG::NodeRecPtr m_virtualMusicianNode;
        OSG::ComponentTransformRecPtr m_virtualMusicianTrans;
        OSG::NodeRecPtr m_virtualMusicianTransNode;
        OSG::SimpleMaterialRecPtr m_otherZoneFboMaterial;

        OSG::Vec3f m_position;

        Passage* m_otherPassage;

        double m_diameterSend;
        double m_diameterIndiv;
        double m_diameterGroup;
        double m_diameterAll;

        double m_heightSend;
        double m_heightShare;

        double m_signalCounter;

        std::vector<Passage*> m_allPassages;
        std::vector<Passage*> m_groupPassages;

        std::vector<Tunnel*> m_ghostTunnels;
        std::map<int, std::pair<unsigned int,Tunnel*> > m_tunnelsMap;
        std::vector<Worm*> m_ghostWorms;
        std::map<int, std::pair<unsigned int,Worm*> > m_wormsMap;
        std::vector<Sign*> m_ghostSigns;
        std::map<int, std::pair<unsigned int,Sign*> > m_signsMap;

        void testCollisionLevel(Ghostable*, int&);

        void addGhostTunnel(Tunnel*);
        void removeGhostTunnel(Tunnel*);
        void unshareTunnel(Tunnel*);
        void shareIndivTunnel(Tunnel*);
        void shareGroupTunnel(Tunnel*);
        void shareAllTunnel(Tunnel*);

        void addGhostWorm(Worm*);
        void removeGhostWorm(Worm*);
        void unshareWorm(Worm*);
        void shareIndivWorm(Worm*);
        void shareGroupWorm(Worm*);
        void shareAllWorm(Worm*);

        void addGhostSign(Sign*);
        void removeGhostSign(Sign*);
        void unshareSign(Sign*);
        void shareIndivSign(Sign*);
        void shareGroupSign(Sign*);
        void shareAllSign(Sign*);

        void updateGhostsPositions();
};



#endif
