/***************************************************************************
 *  Piivert.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Piivert.hpp"
#include "Worm.hpp"
#include "Tunnel.hpp"
#include "Room.hpp"
#include "Zone.hpp"
#include "Sign.hpp"
#include "Musician.hpp"
#include "Passage.hpp"

using namespace std;

Piivert::Piivert(const int& id):m_id(id), m_color(0.1), 
                                m_piivertElementsNb(20), m_piivertLength(10), 
                                m_piivertWidth(0.001), 
                                m_nbFingers(3),
                                m_grabThreshold(0.8), 
                                m_initialGrabDirection(false),
                                m_doneDuplicating(false){

    gettimeofday(&m_previousTime, NULL);

    for(int i=0; i<2; ++i) {

        m_grabbingSpeedFilter[i][0]=new LowPassFilter(0.1, 0);
        m_grabbingSpeedFilter[i][1]=new LowPassFilter(0.1, 0);
        m_grabbingSpeedFilter[i][2]=new LowPassFilter(0.1, 0);
        
        m_positionFilter[i][0]=new OneEuroFilter();
        m_positionFilter[i][1]=new OneEuroFilter();
        m_positionFilter[i][2]=new OneEuroFilter();

        m_vibrationCounter[i]=0;
        m_vibrationAmplitude[i]=0;
        m_vibrationStep[i]=0.5;

        m_recording[i]=false;
        m_recordingRotation[i]=0;

        m_grabState[i]=GRAB_NONE;
        m_grabToggle[i]=false;

        m_selectedManipulable[i]=NULL;
        m_grabbedManipulable[i]=NULL;
        m_multiSelectedManipulable[i].resize(m_nbFingers-1, NULL);
        m_multiGrabbedManipulable[i].resize(m_nbFingers-1, NULL);

    }

    {

        //main node
        m_node = OSG::makeCoredNode<OSG::Group>();

        //global material
        m_material = OSG::SimpleMaterial::create();
        m_material->setLit(false);
        m_material->setColorMaterial(GL_NONE);
        OSG::PolygonChunkRefPtr matchunk = OSG::PolygonChunk::create();
        matchunk->setCullFace(GL_BACK);
        matchunk->setFrontMode(GL_LINE);
        matchunk->setBackMode(GL_LINE);
        m_material->addChunk(matchunk);

        double elementsLength = m_piivertLength/double(m_piivertElementsNb);
        for(unsigned int h=0; h<2; ++h) {
            m_piivert[h] = OSG::Node::create();
            m_piivertTrans[h] = OSG::ComponentTransform::create();
            m_piivert[h]->setCore(m_piivertTrans[h]);
            m_node->addChild(m_piivert[h]);
            //create the elements of the ray
            for(unsigned int i=0; i<m_piivertElementsNb; ++i) {
                OSG::GeometryRefPtr geo = 
                    OSG::makeBoxGeo(m_piivertWidth,m_piivertWidth*10.0, 
                                1, true, true, true);
                geo->setMaterial(m_material);

                OSG::NodeRefPtr element = OSG::Node::create();
                element->setCore(geo);

                OSG::NodeRecPtr transNode = OSG::Node::create();
                OSG::ComponentTransformRecPtr trans = 
                                    OSG::ComponentTransform::create();
                trans->setTranslation(
                        OSG::Vec3f(0, 0, double(i)*(-elementsLength)
                                            -elementsLength/2.0));
                trans->setScale(OSG::Vec3f(1,1,elementsLength));
                transNode->addChild(element);
                transNode->setCore(trans);
                m_piivert[h]->addChild(transNode);

                m_elements[h].push_back(element);
                m_elementsTrans[h].push_back(trans);
            }

            //create the multiselection boxes
            OSG::NodeRecPtr mSelCenterTransNode = OSG::Node::create();
            m_multiSelecCenterTrans[h] = OSG::ComponentTransform::create();
            mSelCenterTransNode->setCore(m_multiSelecCenterTrans[h]);
            m_multiSelecCenterMat[h] = OSG::SimpleMaterial::create();
            m_multiSelecCenterMat[h]->setLit(false);
            m_multiSelecCenterMat[h]->setTransparency(1);
            m_multiSelecCenterMat[h]->setColorMaterial(GL_NONE);
            OSG::PolygonChunkRefPtr multiMatchunk = OSG::PolygonChunk::create();
            multiMatchunk->setCullFace(GL_BACK);
            multiMatchunk->setFrontMode(GL_LINE);
            multiMatchunk->setBackMode(GL_LINE);
            m_multiSelecCenterMat[h]->addChunk(multiMatchunk);
            for(int f=0;f<2;++f) {
                OSG::GeometryRefPtr mSelGeo = 
                    OSG::makeBoxGeo(m_piivertWidth*10.0, m_piivertWidth, 
                                    1, true, true, true);
                mSelGeo->setMaterial(m_multiSelecCenterMat[h]);
                OSG::NodeRefPtr mSelNode = OSG::Node::create();
                mSelNode->setCore(mSelGeo);
                OSG::NodeRecPtr mSelTransNode = OSG::Node::create();
                OSG::ComponentTransformRecPtr mSelTrans = 
                                    OSG::ComponentTransform::create();
                mSelTransNode->setCore(mSelTrans);
                mSelTransNode->addChild(mSelNode);
                m_multiSelecTrans[h][f]=mSelTrans;
                m_multiSelecNodes[h][f]=mSelTransNode;
                mSelCenterTransNode->addChild(mSelTransNode);
            }   
            m_node->addChild(mSelCenterTransNode);
        }

        setColor(m_color);

        OSG::commitChanges();
    }
}

Piivert::~Piivert() {}

void Piivert::parse(xmlNodePtr node) {
    char* value=NULL;
    value = (char*)xmlGetProp(node,(xmlChar*)"name");
    if(value!=NULL) {
        m_name=value;
    }
    value=NULL;
    value = (char*)xmlGetProp(node,(xmlChar*)"color");
    if(value!=NULL) {
        setColor(atof(value));
    }
}

void Piivert::setColor(const float& color) {
    m_color=color;
    OSG::Color3f &matCol = m_material->editDiffuse();
    matCol.setValuesHSV(m_color*360.0, 0.6, 0.8); 
    OSG::Color3f &multiSelMatCol1 = m_multiSelecCenterMat[0]->editDiffuse();
    multiSelMatCol1.setValuesHSV(m_color*360.0, 0.6, 0.8); 
    OSG::Color3f &multiSelMatCol2 = m_multiSelecCenterMat[1]->editDiffuse();
    multiSelMatCol2.setValuesHSV(m_color*360.0, 0.6, 0.8); 
}

void Piivert::setParameter(std::string param, std::string value) {

}

void Piivert::setParameter(const int& hand, const PIIVERT_PARAMETER& param, 
                            double* values, unsigned int nbValues, 
                            vector<Event>& outputEvents) {
    switch(param) {
        case PIIVERT_POSQUAT: {
            OSG::Vec3f& pos = m_piivertTrans[hand]->editTranslation();
            OSG::Quaternion& quat = m_piivertTrans[hand]->editRotation();

            //set filtered position
            //pos[0]=m_positionFilter[hand][0]->filter(values[0]);
            //pos[1]=m_positionFilter[hand][1]->filter(values[1]);
            //pos[2]=m_positionFilter[hand][2]->filter(values[2]);

            //keep hand orientation
            m_handRotation[hand][0]=values[3];
            m_handRotation[hand][1]=values[4];
            m_handRotation[hand][2]=values[5];
            m_handRotation[hand][3]=values[6];


/*
            //set position
            pos[0]=values[0];
            pos[1]=values[1];
            pos[2]=values[2];
*/

            //combine position with offset to remove rotation effect
            OSG::Vec3f posOffset((hand*2.0-1.0)*0.02, 0.03, -0.05); 
            m_handRotation[hand].multVec(posOffset, posOffset);
            pos[0]=values[0]+posOffset[0];
            pos[1]=values[1]+posOffset[1];
            pos[2]=values[2]+posOffset[2];


            //set orientation based on line with origin (below the head)
            OSG::Vec3f origin = 
                m_parentMusician->getHeadTrans()->getTranslation();
            origin[0]+=float(hand*2-1)*0.2;
            origin[1]-=0.5;
            origin[2]+=1;
            
            m_piivertDir[hand] = pos - origin;
            
            quat.setValue(OSG::Vec3f(0,0,-1), m_piivertDir[hand]);

    
        }break;
        case PIIVERT_STORE : {


        }break;
        case PIIVERT_RECORD_ONE_HAND : {
            m_recording[hand]=(values[0]==0)?false:true;
            for(unsigned int i=0; i<m_piivertElementsNb; i++) {
                m_elementsTrans[hand][i]->editRotation().setValue(0,0,0);
            }
        }break;
        case PIIVERT_RECORD_TWO_HANDS : {
            m_recording[0]=(values[0]==0)?false:true;
            m_recording[1]=(values[0]==0)?false:true;
            for(unsigned int i=0; i<m_piivertElementsNb; i++) {
                m_elementsTrans[0][i]->editRotation().setValue(0,0,0);
                m_elementsTrans[1][i]->editRotation().setValue(0,0,0);
            }
        }break;
        case PIIVERT_GRAB : {
            if(values[0]==0) {
                release(hand, outputEvents);
            }
            else {
                switch(m_grabState[hand]) {
                    case GRAB_NONE: {
                        m_grabState[hand]=GRAB_ASKED;
                    }break;
                    default:break;
                }
            }
        }break;
        case PIIVERT_HIT : {
            if(values[0]>0) {
                if(m_selectedManipulable[hand]) {
                    m_selectedManipulable[hand]->hit(outputEvents);
                }
                if(m_vibrationStep[hand]>0) {
                    //send vibration to ray
                    m_vibrationCounter[hand]=1;
                    m_vibrationAmplitude[hand]=values[0]*6.0;
                    m_vibrationStep[hand]=0.5;
                }
            }
        }break;
        case PIIVERT_PRESS : {
            if(values[0]>0) {
                //send vibration to ray
                m_vibrationCounter[hand]=1;
                m_vibrationAmplitude[hand]=values[0]*6.0;
                m_vibrationStep[hand]=0;
            }
            else {
                m_vibrationStep[hand]=1;
            }
        }break;
        default:break;
    }
}

void Piivert::release(const int& hand, vector<Event>& outputEvents) {
    if(m_grabState[hand]==GRAB_GRABBING) {

        if(m_grabbedManipulable[hand]) {
            m_grabbedManipulable[hand]->release(this, hand, outputEvents);
            m_grabbedManipulable[hand]=NULL;
        }
    }    
    //set the state
    m_grabState[hand]=GRAB_NONE;    
}

void Piivert::storeTunnel(Tunnel* packedTunnel, int hand) {
    //add to the packed tunnels
    m_packedTunnels[hand].push_back(packedTunnel);
    m_piivert[hand]->addChild(packedTunnel->getNode());

    //set position and pack
    packedTunnel->pack(m_packedTunnels[hand].size());
}

void Piivert::attachGrabbedManipulable(const int& hand) {
    if(m_grabbedManipulable[hand]) {
        m_grabbedManipulable[hand]->removeFromZone(
                                        m_currentRoom->getCreateZone(m_zoneID));
        m_grabbedManipulable[hand]->attachToMusician(m_parentMusician);
    }
}

void Piivert::detachGrabbedManipulable(const int& hand) {
    if(m_grabbedManipulable[hand]) {
        m_grabbedManipulable[hand]->detachFromMusician(m_parentMusician);
        m_grabbedManipulable[hand]->moveToZone(
                                        m_currentRoom->getCreateZone(m_zoneID));
    }
}

void Piivert::removeTunnel(Tunnel*,int) {

    //FIXME 

}

void Piivert::setMusician(Musician* cam) {
    m_parentMusician=cam;
    m_zoneID=m_parentMusician->getID();
}

void Piivert::update(const timeval& currentTime, 
                            std::vector<Event>& outputEvents,
                            OSG::IntersectAction* intersectAction, 
                            Room* currentRoom) {

    m_currentRoom=currentRoom;

    //get timediff
    timersub(&currentTime, &m_previousTime, &m_timeDiff);
    m_previousTime=currentTime;
    //compute speed with dist and timediff
    float timeDiffMs = m_timeDiff.tv_sec*1000.0 
                        + float(m_timeDiff.tv_usec)/1000.0;


    for(int hand=0; hand<2; ++hand) {
        OSG::Vec3f& pos = m_piivertTrans[hand]->editTranslation();
 //       OSG::Vec3f posDiff = pos - m_previousTrackedPosition[hand];
        

/*
        //get speed mean
        m_grabbingSpeed[hand][0] = 
                m_grabbingSpeedFilter[hand][0]
                        ->filter(pos.dist2(m_previousTrackedPosition[hand])
                                    /timeDiffMs*10000.0);

        if(hand==0) {
            cout<<m_grabbingSpeed[hand][0]<<endl;
        }
*/
/*
        m_grabbingSpeed[hand][1] = 
            min<float>(5, 
                max<float>(1, m_grabbingSpeedFilter[hand][1]
                        ->filter(fabs(posDiff[1])/timeDiffMs*5000.0)));
*/
        if(m_grabState[hand]==GRAB_GRABBING) {
            if(m_grabbedManipulable[hand]) {
                m_grabbedManipulable[hand]->moveTo(
                                            this,
                                            hand, 
                                            pos[2]- 
                                                m_parentMusician
                                                    ->getHeadTrans()
                                                        ->getTranslation()[2], 
                                            OSG::Line(pos, m_piivertDir[hand]), 
                                            outputEvents);


            }
        }
        else {
            //test collision with worms, tunnels ...
            testRay(hand, intersectAction, currentRoom, outputEvents);

/*
            //if collided with a center worm, compute the multi selection
            if(m_selectedWorm[hand]) {
                m_multiSelecCenterMat[hand]->setTransparency(0.8);
                testMultipleSelection(hand, currentRoom, outputEvents);
*/
/*
                outputEvents.push_back(Event());
                Event& event = outputEvents.back();
                event.type = EVENT_PIIVERT;
                event.piivert = m_id;
                event.parameter = PIIVERT_SELECT_WORM;
                event.hand = hand;
                event.values[0]=m_multiSelectedWorms[hand][0]->getID();
                event.values[1]=m_multiSelectedWorms[hand][1]->getID();
                event.values[2]=m_multiSelectedWorms[hand][2]->getID();
*/
/*
            }
            else {
                m_multiSelecCenterMat[hand]->setTransparency(1);
*/
/*
                outputEvents.push_back(Event());
                Event& event = outputEvents.back();
                event.type = EVENT_PIIVERT;
                event.piivert = m_id;
                event.parameter = PIIVERT_SELECT_WORM;
                event.hand = hand;
                event.values[0]=-1;
                event.values[1]=-1;
                event.values[2]=-1;
*/
  //          }

        }

        //vibration
        if(m_vibrationCounter[hand]>0) {
            m_vibrationCounter[hand]-=m_vibrationStep[hand];
            double scale=
                1+(m_vibrationAmplitude[hand]*m_vibrationCounter[hand]*10.0);
            for(unsigned int i=0; i<m_piivertElementsNb; i+=2) {
                OSG::Vec3f& sca= m_elementsTrans[hand][i]->editScale();
                sca[0]=scale; 
            }
        }

        //recording
        if(m_recording[hand]) {
            m_recordingRotation[hand]=
                fmod(m_recordingRotation[hand]+timeDiffMs/100.0,M_PI*2.0);;
            for(unsigned int i=1; i<m_piivertElementsNb; i++) {
                m_elementsTrans[hand][i]->editRotation().setValue(0,0,
                                                    m_recordingRotation[hand]);            
            }
        }
        //store tracked position
        m_previousTrackedPosition[hand][0]=pos[0];
        m_previousTrackedPosition[hand][1]=pos[1];
        m_previousTrackedPosition[hand][2]=pos[2];
    }
}

void Piivert::testMultipleSelection(const unsigned int& hand, 
                                    Room* currentRoom,
                                    std::vector<Event>& outputEvents) {
/*

    OSG::Quaternion& mSelCentRot= m_multiSelecCenterTrans[hand]->editRotation();
    mSelCentRot.setValue(m_handRotation[hand]);
    OSG::Vec3f& mSelCentPos = m_multiSelecCenterTrans[hand]->editTranslation();
    mSelCentPos=m_selectedWorm[hand]->getTrans()->getTranslation();
    mSelCentPos[2]-=currentRoom->getCreateZone(m_zoneID)->getDepth();

    OSG::Vec3f dir(1,0,0);
    mSelCentRot.multVec(dir,dir);
    float scale=Worm::WORM_SIZE*2.0;
    scale*=1.0-fabs(dir[2]);

    //scale / translate the two multiselec rays
    for(int b=0; b<2; ++b) {
        OSG::Vec3f& mSelBoxPos = m_multiSelecTrans[hand][b]->editTranslation();
        OSG::Vec3f& mSelBoxScale = m_multiSelecTrans[hand][b]->editScale();
        mSelBoxPos[2]=float(((b%2)*2-1))*(scale*0.5+Worm::WORM_SIZE/2.0);
        mSelBoxScale[2]=scale;
    }

    //get room worms
    vector<Worm*> zoneWorms = currentRoom->getCreateZone(m_zoneID)->getWorms();
    //test collision for fingers
    for(int f=0; f<m_nbFingers-1; ++f) {
        OSG::BoxVolume wormVol, boxVol;
        OSG::Pnt3f wormCent, boxCent;
        m_multiSelecNodes[hand][f%2]->getWorldVolume(boxVol);
        vector<Worm*>::iterator itWo=zoneWorms.begin();
        vector<Worm*>::iterator itWoHit;;
        float minDist=5000;
        Worm* hitWorm=NULL;
        float dist=0;
        for(;itWo!=zoneWorms.end();++itWo) {
            if((*itWo)->getID()!=m_selectedWorm[hand]->getID()) {
                (*itWo)->getNode()->getWorldVolume(wormVol);
                if(boxVol.intersect(wormVol)) {
                    wormVol.getCenter(wormCent);
                    boxVol.getCenter(boxCent);
                    dist=wormCent.dist(boxCent);
                    if(dist<minDist) {
                        minDist=dist;
                        hitWorm=(*itWo);
                        itWoHit=itWo;
                    }
                }
            }
        }
        if(hitWorm) {
            hitWorm->setHighlight(true, m_color);
            zoneWorms.erase(itWoHit);
            //cout<<"for finger "<<f<<" got worm "<<hitWorm->getID()<<endl;

            //m_multiSelectedWorms[hand][m_fingersMap[f]]=hitWorm;

        }
    }
*/
}

void Piivert::testRay(const unsigned int& hand, 
                        OSG::IntersectAction* intersectAction, 
                        Room* currentRoom, 
                        std::vector<Event>& outputEvents) {

    OSG::Vec3f vec, scale;
    OSG::Quaternion quat, scaleQuat;
    m_piivert[hand]->getToWorld().getTransform(vec, quat, scale,scaleQuat);
    OSG::Vec3f dir(0, 0, -1);
    quat.multVec(dir, dir);
    
    OSG::Line line(vec, dir);

    //priorities: worms > tunnels > signs > passages
    currentRoom->markWormZoneForSelection(m_zoneID);
    if(!testIntersectManipulable(hand, line, intersectAction,
                                        currentRoom, outputEvents)) {
        outputEvents.push_back(Event());
        Event& event = outputEvents.back();
        event.type = EVENT_PIIVERT;
        event.piivert = m_id;
        event.parameter = PIIVERT_SELECT_WORM;
        event.hand = hand;
        event.values[0]=-1;
        event.values[1]=-1;
        event.values[2]=-1;
        event.values[3]=-1;
        event.values[4]=-1;

        currentRoom->markTunnelZoneForSelection(m_zoneID);
        if(!testIntersectManipulable(hand, line, intersectAction,
                                        currentRoom, outputEvents)) {
/*
            currentRoom->markSignZoneForSelection(m_zoneID);
            if(!testIntersectManipulable(hand, line, intersectAction,
                                        currentRoom, outputEvents)) {
                currentRoom->markPassageZoneForSelection(m_zoneID);
                testIntersectManipulable(hand, line, intersectAction, 
                                        currentRoom, outputEvents);
            }
*/

            //test ropes
            currentRoom->markRopesForSelection();
            testIntersectManipulable(hand, line, intersectAction,
                                        currentRoom, outputEvents);
        }
    }
    else {
        outputEvents.push_back(Event());
        Event& event = outputEvents.back();
        event.type = EVENT_PIIVERT;
        event.piivert = m_id;
        event.parameter = PIIVERT_SELECT_WORM;
        event.hand = hand;
        event.values[0]=m_selectedManipulable[hand]->getID();
        event.values[1]=m_selectedManipulable[hand]->getID();
        event.values[2]=m_selectedManipulable[hand]->getID();
        event.values[3]=m_selectedManipulable[hand]->getID();
        event.values[4]=m_selectedManipulable[hand]->getID();
    }
    currentRoom->markForSelection();
}

bool Piivert::testIntersectManipulable(const int& h, OSG::Line& line, 
                                        OSG::IntersectAction* intersectAction, 
                                        Room* currentRoom, 
                                        std::vector<Event>& outputEvents) {
    intersectAction->setLine(line, OSG::Inf);
    intersectAction->apply(currentRoom->getNode());
    if(intersectAction->didHit()) {
        OSG::Node* hitNode = intersectAction->getHitObject();
        OSG::AttachmentRefPtr attach = 
                hitNode->findAttachment(OSG::VoidPAttachment::getClassType());
        if(attach) {
            OSG::VoidPAttachmentRefPtr ptr=
                    OSG::dynamic_pointer_cast<OSG::VoidPAttachment>(attach);
            Manipulable* hitManipulable = 
                    static_cast<Manipulable*>(ptr->getField().getValue());
            if(m_selectedManipulable[h]) {
                if(m_selectedManipulable[h]->getID()!=hitManipulable->getID()) {
                    m_selectedManipulable[h]->removeUser();
                    m_selectedManipulable[h]->setHighlight(false);
                    m_selectedManipulable[h]=hitManipulable;
                    m_selectedManipulable[h]->addUser();
                    m_selectedManipulable[h]->setHighlight(true, m_color);
                    OSG::Pnt3f hitPoint = intersectAction->getHitPoint();
                    setVisibleLength(h, hitPoint.dist(line.getPosition()));
                }
            }
            else {
                m_selectedManipulable[h]=hitManipulable;
                m_selectedManipulable[h]->addUser();
                m_selectedManipulable[h]->setHighlight(true, m_color);
                OSG::Pnt3f hitPoint = intersectAction->getHitPoint();
                setVisibleLength(h, hitPoint.dist(line.getPosition()));
            }
            //test grabbing
            switch(m_grabState[h]) {
                case GRAB_ASKED: {
                    m_grabbedManipulable[h]=m_selectedManipulable[h];
                    m_doneDuplicating=false;
                    m_initialGrabPosition[h]=m_previousTrackedPosition[h];
                    m_grabbedManipulable[h]->grab(this, h, outputEvents);
                    m_grabState[h]=GRAB_GRABBING;
                }break;
                default:break;
            }
            return true;
        }
    }
    //remove the selected worm if any
    if(m_selectedManipulable[h]) {
        m_selectedManipulable[h]->removeUser();
        m_selectedManipulable[h]->setHighlight(false);
        m_selectedManipulable[h] = NULL;
        setVisibleLength(h, -1);
    }
    return false;
}

void Piivert::setVisibleLength(const unsigned int& hand, double length) {
    if(length==-1) {
        OSG::Vec3f& piivScale = m_piivertTrans[hand]->editScale();
        piivScale[2]=1;
    }
    else {
        OSG::Vec3f& piivScale = m_piivertTrans[hand]->editScale();
        piivScale[2]=fabs(length/m_piivertLength);
    }
}

