/***************************************************************************
 *            Reconstruct.cpp
 *            Part of Drile
 *
 *  2010- Florent Berthaut
 *  http://hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "Reconstruct.hpp"
#include "EnvManager.hpp"

using namespace std;
using namespace cv;



Reconstruct::Reconstruct(int cam):m_width(640), m_height(480), m_cam(cam) {}

Reconstruct::~Reconstruct() {}

void Reconstruct::parse(xmlNodePtr reconstNode) {
    char* value=NULL;
    value = (char*)xmlGetProp(reconstNode,(xmlChar*)"device");
    if(value!=NULL) {
        m_device=atoi(value);
    }

    value=NULL;
    value = (char*)xmlGetProp(reconstNode,(xmlChar*)"position");
    if(value!=NULL) {
        std::string posStr=value;
        size_t pos;
        std::istringstream is,is2,is3;
        pos = posStr.find_first_of(";");
        is.str(posStr.substr(0,pos));
        posStr = posStr.substr(pos+1);
        is>>m_position[0];
        pos = posStr.find_first_of(";");
        is2.str(posStr.substr(0,pos));
        posStr = posStr.substr(pos+1);
        is2>>m_position[1];
        pos = posStr.find_first_of(";");
        is3.str(posStr.substr(0,pos));
        posStr = posStr.substr(pos+1);
        is3>>m_position[2];
    }

    value=NULL;
    value = (char*)xmlGetProp(reconstNode,(xmlChar*)"rotation");
    if(value!=NULL) {
        std::string rotStr=value;
        size_t pos;
        std::istringstream is,is2,is3;
        pos = rotStr.find_first_of(";");
        is.str(rotStr.substr(0,pos));
        rotStr = rotStr.substr(pos+1);
        is>>m_rotation[0];
        pos = rotStr.find_first_of(";");
        is2.str(rotStr.substr(0,pos));
        rotStr = rotStr.substr(pos+1);
        is2>>m_rotation[1];
        pos = rotStr.find_first_of(";");
        is3.str(rotStr.substr(0,pos));
        rotStr = rotStr.substr(pos+1);
        is3>>m_rotation[2];
        for(int i=0;i<3;++i) {
            m_rotation[i]=m_rotation[i]/180.0*M_PI;
        }
    }

    value=NULL;
    value = (char*)xmlGetProp(reconstNode,(xmlChar*)"scale");
    if(value!=NULL) {
        std::string scaleStr=value;
        size_t pos;
        std::istringstream is,is2,is3;
        pos = scaleStr.find_first_of(";");
        is.str(scaleStr.substr(0,pos));
        scaleStr = scaleStr.substr(pos+1);
        is>>m_scale[0];
        pos = scaleStr.find_first_of(";");
        is2.str(scaleStr.substr(0,pos));
        scaleStr = scaleStr.substr(pos+1);
        is2>>m_scale[1];
        pos = scaleStr.find_first_of(";");
        is3.str(scaleStr.substr(0,pos));
        scaleStr = scaleStr.substr(pos+1);
        is3>>m_scale[2];
    }
}

void Reconstruct::load() {

    //open the camera
    m_cap.open(m_cam);
    m_cap.set(CV_CAP_PROP_FRAME_WIDTH,m_width);
    m_cap.set(CV_CAP_PROP_FRAME_HEIGHT,m_height);
    m_cap.set(CV_CAP_PROP_FPS,60);

    //create the image and texture
    m_image = OSG::Image::create();
    m_image->set(OSG::Image::OSG_BGR_PF, m_width, m_height);
    m_texObj = OSG::TextureObjChunk::create();
    m_texObj->setImage(m_image);
    m_texObj->setMinFilter(GL_LINEAR);
    m_texObj->setMagFilter(GL_LINEAR);
    m_texObj->setWrapS(GL_CLAMP_TO_EDGE);
    m_texObj->setWrapT(GL_CLAMP_TO_EDGE);
    m_texObj->setScale(true);
    m_back = OSG::TextureBackground::create();

    m_back->setTexture(m_texObj);

}


void Reconstruct::update() {
    
    Mat frame;
    m_cap >> frame;
    //cvtColor(frame, frame, CV_BayerBG2BGR);

    OSG::UInt8* data = m_image->editData();
    int pos=0;
    //cout<<"got "<<frame.rows<<" rows and "<<frame.cols<<" cols"<<endl;
    for (int i = 0; i < frame.rows; ++i) {
        for (int j = 0; j < frame.cols; ++j) {
            Vec3b& rgb = frame.at<Vec3b>((frame.rows-1-i), (j));
            data[pos]=rgb[0];
            data[pos+1]=rgb[1];
            data[pos+2]=rgb[2];
            pos+=3;
        }
    }
}



