/***************************************************************************
 *  Reconstruct3D.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef WITH_FREENECT

#include "Reconstruct3D.hpp"

using namespace std;


Reconstruct3D::Reconstruct3D():m_deviceID(0), m_width(640), m_height(480), 
                                m_nbVertices(30000), m_minDepthThreshold(500), 
                                m_maxDepthThreshold(2000), 
                                m_foundHead(false) {

    gettimeofday(&m_firstTime,NULL);
}

Reconstruct3D::~Reconstruct3D() {

}

void Reconstruct3D::parse(xmlNodePtr reconstNode) {
    char* value=NULL;
    value = (char*)xmlGetProp(reconstNode,(xmlChar*)"device");
    if(value!=NULL) {
        m_deviceID=atoi(value);
    }

    value=NULL;
    value = (char*)xmlGetProp(reconstNode,(xmlChar*)"position");
    if(value!=NULL) {
        std::string posStr=value;
        size_t pos;
        std::istringstream is,is2,is3;
        pos = posStr.find_first_of("/");
        is.str(posStr.substr(0,pos));
        posStr = posStr.substr(pos+1);
        is>>m_position[0];
        pos = posStr.find_first_of("/");
        is2.str(posStr.substr(0,pos));
        posStr = posStr.substr(pos+1);
        is2>>m_position[1];
        pos = posStr.find_first_of(";");
        is3.str(posStr.substr(0,pos));
        posStr = posStr.substr(pos+1);
        is3>>m_position[2];
    }

    value=NULL;
    value = (char*)xmlGetProp(reconstNode,(xmlChar*)"rotation");
    if(value!=NULL) {
        std::string rotStr=value;
        size_t pos;
        std::istringstream is,is2,is3;
        pos = rotStr.find_first_of(";");
        is.str(rotStr.substr(0,pos));
        rotStr = rotStr.substr(pos+1);
        is>>m_rotation[0];
        pos = rotStr.find_first_of(";");
        is2.str(rotStr.substr(0,pos));
        rotStr = rotStr.substr(pos+1);
        is2>>m_rotation[1];
        pos = rotStr.find_first_of(";");
        is3.str(rotStr.substr(0,pos));
        rotStr = rotStr.substr(pos+1);
        is3>>m_rotation[2];
        for(int i=0;i<3;++i) {
            m_rotation[i]=m_rotation[i]/180.0*M_PI;
        }
    }

    value=NULL;
    value = (char*)xmlGetProp(reconstNode,(xmlChar*)"scale");
    if(value!=NULL) {
        std::string scaleStr=value;
        size_t pos;
        std::istringstream is,is2,is3;
        pos = scaleStr.find_first_of(";");
        is.str(scaleStr.substr(0,pos));
        scaleStr = scaleStr.substr(pos+1);
        is>>m_scale[0];
        pos = scaleStr.find_first_of(";");
        is2.str(scaleStr.substr(0,pos));
        scaleStr = scaleStr.substr(pos+1);
        is2>>m_scale[1];
        pos = scaleStr.find_first_of(";");
        is3.str(scaleStr.substr(0,pos));
        scaleStr = scaleStr.substr(pos+1);
        is3>>m_scale[2];
    }

    value=NULL;
    value = (char*)xmlGetProp(reconstNode,(xmlChar*)"max_depth");
    if(value) {
        m_maxDepthThreshold=atof(value)*1000.0;
    }

    value=NULL;
    value = (char*)xmlGetProp(reconstNode,(xmlChar*)"min_depth");
    if(value) {
        m_minDepthThreshold=atof(value)*1000.0;
    }
}

void depthCallback(freenect_device *dev, void *depth, uint32_t timestamp) {

    Reconstruct3D *rec3D = static_cast<Reconstruct3D*>(freenect_get_user(dev));
    pthread_mutex_lock(&rec3D->m_mutex);
    
    rec3D->m_tmpDepthBuf = rec3D->m_outputDepthBuf;
    rec3D->m_outputDepthBuf = rec3D->m_inputDepthBuf;
    rec3D->m_inputDepthBuf = rec3D->m_tmpDepthBuf;
    
    freenect_set_depth_buffer(rec3D->m_device, rec3D->m_inputDepthBuf);

    rec3D->m_gotDepth++;
    pthread_mutex_unlock(&rec3D->m_mutex);
}

void rgbCallback(freenect_device *dev, void *rgb, uint32_t timestamp) {

    Reconstruct3D *rec3D = static_cast<Reconstruct3D*>(freenect_get_user(dev));
    pthread_mutex_lock(&rec3D->m_mutex);

    rec3D->m_tmpRgbBuf = rec3D->m_outputRgbBuf;
    rec3D->m_outputRgbBuf = rec3D->m_inputRgbBuf;
    rec3D->m_inputRgbBuf = rec3D->m_tmpRgbBuf;
    
    freenect_set_video_buffer(rec3D->m_device, rec3D->m_inputRgbBuf);

    rec3D->m_gotRgb++;
    pthread_mutex_unlock(&rec3D->m_mutex);

}

void *freenectThreadFunc(void *arg){
    Reconstruct3D *rec3D = static_cast<Reconstruct3D*>(arg);

    freenect_set_depth_callback(rec3D->m_device, depthCallback);
    freenect_set_video_callback(rec3D->m_device, rgbCallback);
    freenect_set_video_mode(rec3D->m_device, 
        freenect_find_video_mode(FREENECT_RESOLUTION_MEDIUM, 
                                FREENECT_VIDEO_RGB));
    freenect_set_video_buffer(rec3D->m_device, rec3D->m_inputRgbBuf);
    freenect_set_depth_mode(rec3D->m_device, 
        freenect_find_depth_mode(FREENECT_RESOLUTION_MEDIUM, 
                                FREENECT_DEPTH_REGISTERED));
    freenect_set_depth_buffer(rec3D->m_device, rec3D->m_inputDepthBuf);

    freenect_start_depth(rec3D->m_device);
    freenect_start_video(rec3D->m_device);

    cout<<"Kinect reconstruction started"<<endl;

    while(1) {
        while(freenect_process_events(rec3D->m_context) >= 0) {
            usleep(1000);
        }
    }
    freenect_stop_depth(rec3D->m_device);
    freenect_stop_video(rec3D->m_device);

    freenect_close_device(rec3D->m_device);
    freenect_shutdown(rec3D->m_context);
    return NULL;
}

void Reconstruct3D::load() {
    {
        OSG::GeoUInt8PropertyRecPtr types = OSG::GeoUInt8Property::create();
        types->addValue(GL_POINTS);
        OSG::GeoUInt32PropertyRecPtr lengths = OSG::GeoUInt32Property::create();
        lengths->addValue(m_nbVertices);
        OSG::GeoPnt3fPropertyRecPtr pos = OSG::GeoPnt3fProperty::create();
        OSG::GeoColor3fPropertyRecPtr colors = 
            OSG::GeoColor3fProperty::create();
        OSG::GeoVec3fPropertyRecPtr norms = OSG::GeoVec3fProperty::create();
        OSG::GeoUInt32PropertyRecPtr indices = OSG::GeoUInt32Property::create();

        for (unsigned int x = 0; x < m_nbVertices; x++) {
            pos->addValue(OSG::Pnt3f(0,0,0));
            colors->addValue(OSG::Color3f(1,0,0));
            norms->addValue(OSG::Vec3f(0,0,-1));
            indices->addValue(x);
        }

        m_reconstructGeom = OSG::Geometry::create();

        m_reconstructGeom->setTypes(types);
        m_reconstructGeom->setLengths(lengths);
        m_reconstructGeom->setIndices(indices);
        m_reconstructGeom->setPositions(pos);
        m_reconstructGeom->setNormals(norms);
        m_reconstructGeom->setColors(colors);

        m_reconstruct = OSG::Node::create();
        m_reconstruct->setCore(m_reconstructGeom);
        
        m_reconstructTrans = OSG::ComponentTransform::create();
        m_reconstructTransNode = OSG::Node::create();
        m_reconstructTransNode->setCore(m_reconstructTrans);
        m_reconstructTransNode->addChild(m_reconstruct);

        m_reconstructMaterial = OSG::SimpleMaterial::create();
        m_reconstructMaterial->setLit(false);
        m_reconstructMaterial->setColorMaterial(GL_DIFFUSE);
        OSG::PointChunkRefPtr pointChunk = OSG::PointChunk::create();
        pointChunk->setSize(5);
        m_reconstructMaterial->addChunk(pointChunk);

        m_reconstructGeom->setMaterial(m_reconstructMaterial);
        m_reconstructGeom->setDlistCache(false);

        OSG::commitChanges();

    }

    //image buffers
    m_inputDepthBuf = new uint16_t[m_width*m_height];
    m_outputDepthBuf = new uint16_t[m_width*m_height];
    m_inputRgbBuf = new uint8_t[m_width*m_height*3];
    m_outputRgbBuf = new uint8_t[m_width*m_height*3];
    m_gotRgb=0;
    m_gotDepth=0;

    
    //freenect
    if(freenect_init(&m_context, NULL) >= 0) {
        if(freenect_open_device(m_context, &m_device, m_deviceID) >= 0) {
            freenect_set_user(m_device,this);
            pthread_mutex_init(&m_mutex,NULL);
            pthread_create(&m_freenectThread, NULL, freenectThreadFunc, this);
        }
        else {
            cout<<"Unable to open kinect device "<<m_deviceID<<endl;
        }
    }
    else {
        cout<<"Unable to create the kinect context"<<endl;
    }
}


void Reconstruct3D::update() {

    gettimeofday(&m_currentTime,NULL);
    timersub(&m_currentTime, &m_firstTime, &m_currentTime);
    m_elapsedTime = double(m_currentTime.tv_sec) 
                    + double(m_currentTime.tv_usec)/1000000.0;
    

    //lock mutex 
    pthread_mutex_lock(&m_mutex);

    //if got rgb and depth 
    if(m_gotRgb && m_gotDepth) {

        OSG::GeoPnt3fPropertyRecPtr pos = 
         static_cast<OSG::GeoPnt3fProperty*>(m_reconstructGeom->getPositions());
        OSG::GeoPnt3fProperty::StoredFieldType *posField = 
            pos->editFieldPtr();
        OSG::GeoPnt3fProperty::StoredFieldType::iterator lastVert, itVert;
        itVert = posField->begin();
        lastVert= posField->end();
        OSG::GeoColor3fPropertyRecPtr color = 
         static_cast<OSG::GeoColor3fProperty*>(m_reconstructGeom->getColors());
        OSG::GeoColor3fProperty::StoredFieldType *colorField = 
            color->editFieldPtr();
        OSG::GeoColor3fProperty::StoredFieldType::iterator itColor;
        itColor = colorField->begin();


        m_foundHead=false;
        double posX=0, posY=0;

        //go through the pixels
        for(unsigned int j=0; j<m_height; j++) {
            for(unsigned int i=0; i<m_width; i++) { 

                //if within thresholds
                if(m_outputDepthBuf[m_width*j+i]<m_maxDepthThreshold
                     && m_outputDepthBuf[m_width*j+i]>m_minDepthThreshold) {

                    //and vertices left to set
                    if(itVert!=lastVert) {

                        freenect_camera_to_world(m_device, i, j, 
                                                m_outputDepthBuf[m_width*j+i],
                                                &posX, &posY);

                        //get position
                        (*itVert)[0]=-posX/1000.0;
                        (*itVert)[1]=-posY/1000.0;
                        (*itVert)[2]=float(m_outputDepthBuf[m_width*j+i])
                                    /1000.0;

                        //test head-tracking
                        if(!m_foundHead) {
                            m_headPos[0]=
                                m_headPosFilter[0].filter((*itVert)[0], 
                                                            m_elapsedTime)
                                +m_position[0];
                            m_headPos[1]=
                                m_headPosFilter[1].filter((*itVert)[1], 
                                                            m_elapsedTime)
                                +m_position[1];
                            m_headPos[2]=
                                m_headPosFilter[2].filter((*itVert)[2], 
                                                            m_elapsedTime) 
                                +m_position[2];
                            m_foundHead=true;
                        }
                        (*itVert)[0]+=m_position[0];
                        (*itVert)[1]+=m_position[1];
                        (*itVert)[2]+=m_position[2];

                        //get color
                        (*itColor)[0]=
                            float(m_outputRgbBuf[(m_width*j+i)*3])/255.0;
                        (*itColor)[1]=
                            float(m_outputRgbBuf[(m_width*j+i)*3+1])/255.0;
                        (*itColor)[2]=
                            float(m_outputRgbBuf[(m_width*j+i)*3+2])/255.0;


                        itVert++;
                        itColor++;
                    }
                }
            }
        }
        m_gotDepth=0;
        m_gotRgb=0;
    }
    pthread_mutex_unlock(&m_mutex);
}


#endif

