/***************************************************************************
 *            Reconstruct3D.hpp
 *            Part of Drile
 *
 *  2010- Florent Berthaut
 *  http://hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifdef WITH_FREENECT

#ifndef _RECONSTRUCT_3D_H
#define _RECONSTRUCT_3D_H

#include <iostream>
#include <cstring>
#include <cmath>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <vector>
#include <map>
#include <pthread.h>

#include <OpenSG/OSGSimpleGeometry.h>
#include <OpenSG/OSGGeometry.h>
#include <OpenSG/OSGComponentTransform.h>
#include <OpenSG/OSGMaterialGroup.h>
#include <OpenSG/OSGSceneFileHandler.h>
#include <OpenSG/OSGSimpleTexturedMaterial.h>
#include <OpenSG/OSGImage.h>
#include <OpenSG/OSGPointChunk.h>
#include <OpenSG/OSGTypedGeoIntegralProperty.h>
#include <OpenSG/OSGTypedGeoVectorProperty.h>

#include <libxml/parser.h>

#include <libfreenect-registration.h>
#include <libfreenect.h>

#include "../utils/OneEuroFilter.hpp"

class Reconstruct3D {
    public:
        Reconstruct3D();
        ~Reconstruct3D();

        void parse(xmlNodePtr);

        void load();
        void update();

        inline OSG::NodeRefPtr getNode(){return m_reconstructTransNode;}

        friend void depthCallback(freenect_device*, void *, uint32_t );
        friend void videoCallback(freenect_device*, void *, uint32_t );
        friend void* freenectThreadFunc(void*);
        uint16_t *m_inputDepthBuf, *m_tmpDepthBuf, *m_outputDepthBuf;
        uint8_t *m_inputRgbBuf, *m_tmpRgbBuf, *m_outputRgbBuf;
        pthread_mutex_t m_mutex;
        int m_gotDepth, m_gotRgb;
        freenect_context *m_context;
        freenect_device *m_device;

        inline OSG::Vec3f& getHeadPosition(){return m_headPos;}

    private:

        std::string m_name;

        OSG::NodeRefPtr m_reconstruct;
        OSG::NodeRefPtr m_reconstructTransNode;
        OSG::ComponentTransformRecPtr m_reconstructTrans;

        OSG::GeometryRefPtr m_reconstructGeom;
        OSG::SimpleMaterialRefPtr m_reconstructMaterial;
        OSG::ImageRecPtr m_texture;
        OSG::UInt8* m_textureData;
        OSG::Vec3f m_position;
        OSG::Vec3f m_rotation;
        OSG::Vec3f m_scale;
        unsigned int m_deviceID;

        pthread_t m_freenectThread;

        OSG::Vec3f m_headPos;

        unsigned int m_width;
        unsigned int m_height;
        unsigned int m_nbVertices;
        float m_minDepthThreshold, m_maxDepthThreshold;
        bool m_foundHead;


        OneEuroFilter m_headPosFilter[3];
        timeval m_currentTime, m_firstTime, m_timeDiff;
        double m_elapsedTime;

        
/*
        friend void* videoThreadFunc(void*);
        pthread_t m_videoThread;
        short *m_depthImage;
        OSG::UInt8 *m_rgbImage;
        bool m_videoUpdated;
*/
};



#endif

#endif
