/***************************************************************************
 *  Room.cpp
 *  Part of Drile
 *  2010- Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Room.hpp"

#include "EnvManager.hpp"
#include "../Drile.hpp"
#include "Zone.hpp"
#include "Passage.hpp"
#include "Tunnel.hpp"
#include "Worm.hpp"
#include "Sign.hpp"
#include "Grid.hpp"
#include "Rope.hpp"

using namespace std;

Room::Room(const int& id): m_id(id) {
    {
        m_room = OSG::Node::create();
        m_roomTrans = OSG::ComponentTransform::create();
        m_roomTrans->setTranslation(m_position);
        m_room->setCore(m_roomTrans);

        m_roomModelTransNode = OSG::Node::create();
        m_roomModelTrans = OSG::ComponentTransform::create();
        m_roomModelTransNode->setCore(m_roomModelTrans);
        m_room->addChild(m_roomModelTransNode);
        m_scale=OSG::Vec3f(1,1,1);

        m_ropesRoot = OSG::makeCoredNode<OSG::Group>();
        m_room->addChild(m_ropesRoot);

        OSG::commitChanges();
    }
}

Room::~Room() {}

void Room::parse(xmlNodePtr roomNode) {
    char* value=NULL;
    value = (char*)xmlGetProp(roomNode,(xmlChar*)"name");
    if(value!=NULL) {
        m_name=value;
    }

    value=NULL;
    value = (char*)xmlGetProp(roomNode,(xmlChar*)"coords");
    if(value!=NULL) {
        std::string valStr = value;
        parseCoords(valStr, m_position);
        m_roomTrans->setTranslation(m_position);
    }

    value=NULL;
    value = (char*)xmlGetProp(roomNode,(xmlChar*)"scale");
    if(value!=NULL) {
        std::string valStr = value;
        parseCoords(valStr, m_scale);
        m_roomModelTrans->setScale(m_scale);
    }

    m_roomModelStr="";
    value=NULL;
    value = (char*)xmlGetProp(roomNode,(xmlChar*)"model");
    if(value!=NULL) {
        setModel(std::string(value));
    }

    value=NULL;
    value = (char*)xmlGetProp(roomNode,(xmlChar*)"ropes_to");
    if(value!=NULL) {
        std::string ropesStr=value;
        size_t pos = 0;
        while ((pos = ropesStr.find(";")) != string::npos) {
            EnvManager* envMan = EnvManager::getInstance();
            Room* roomTo = envMan->getRoom(ropesStr.substr(0, pos));
            if(roomTo) {
                envMan->addRope(this, roomTo);
            }
            ropesStr = ropesStr.substr(pos+1);
        }
    }
}

void Room::parseCoords(std::string &strCoords, OSG::Vec3f &coords) {
        istringstream is, is2, is3;
        size_t pos = strCoords.find_first_of('/');    
        if(pos!=strCoords.npos) {
            is.str(strCoords.substr(0,pos));
            is>>coords[0];
        }
        else {
            coords[0]=0;
        }
        strCoords = strCoords.substr(pos+1);
        pos = strCoords.find_first_of('/');    
        if(pos!=strCoords.npos) {
            is2.str(strCoords.substr(0,pos));
            is2>>coords[1];
        }
        else {
            coords[1]=0;
        }
        strCoords = strCoords.substr(pos+1);
        pos = strCoords.find_first_of(';');    
        is3.str(strCoords.substr(0,pos));
        is3>>coords[2];

        
}

void Room::setModel(const std::string& model) {
    m_roomModelStr=model;
    if(m_roomModel) {
        m_roomModelTransNode->subChild(m_roomModel);
    } 


    if(m_roomModelStr.compare("")==0) {
        m_roomModel = OSG::makeCoredNode<OSG::Group>();
/*
    //model light
    OSG::PointLightRecPtr pLight = OSG::PointLight::create();
    pLight->setPosition(OSG::Pnt3f(0, 100, -20));
    pLight->setConstantAttenuation(3);
    pLight->setLinearAttenuation(0);
    pLight->setQuadraticAttenuation(0);
    pLight->setDiffuse(OSG::Color4f(0.7,0.7,0.7,1));
    pLight->setAmbient(OSG::Color4f(0.1,0.1,0.1,1));
    m_roomModel->setCore(pLight);
        OSG::SimpleMaterialRecPtr tempFloorMaterial = OSG::SimpleMaterial::create();
        tempFloorMaterial->setLit(true);
        tempFloorMaterial->setColorMaterial(GL_NONE);
        tempFloorMaterial->setDiffuse(OSG::Color3f(0.6, 0.6, 0.5));
        tempFloorMaterial->setAmbient(OSG::Color3f(0, 0, 0));
        tempFloorMaterial->setTransparency(0);
        for(int i=0;i<20;++i) {
            for(int j=0;j<20;++j) {
                //floor
                OSG::GeometryRecPtr tempFloorGeo = 
                    OSG::makeBoxGeo(10, (fabs(cos(double(i)/20*M_PI*2.0)) 
                                    + fabs(sin(double(j)/20*M_PI*2.0)))*3.0 +0.5
                                    , 10, 1, 1, 1);
                tempFloorGeo->setMaterial(tempFloorMaterial);
                OSG::NodeRecPtr tempFloorNode = OSG::Node::create();
                tempFloorNode->setCore(tempFloorGeo);
                OSG::ComponentTransformRefPtr tempFloorTrans = 
                        OSG::ComponentTransform::create();
                tempFloorTrans->setTranslation(
                        OSG::Vec3f(-100+i*10, -5, -100+j*10));
                OSG::NodeRecPtr tempFloorTransNode = OSG::Node::create();
                tempFloorTransNode->setCore(tempFloorTrans);
                tempFloorTransNode->addChild(tempFloorNode);
                m_roomModel->addChild(tempFloorTransNode);
            }
        }
*/
    }
    else {
        m_roomModel=OSG::SceneFileHandler::the()->read(m_roomModelStr.c_str());
    }
    m_roomModelTransNode->addChild(m_roomModel);
}

Zone* Room::addZone() { 
    Zone* newZone = new Zone(m_zones.size());
    m_zones.push_back(newZone);
    m_room->addChild(newZone->getNode());
    return newZone;
}

void Room::removeZone() {

}

Zone* Room::getCreateZone(const unsigned int& id) {
    while(id>=m_zones.size()) {
        addZone();
    }
    return m_zones[id];
}

void Room::setZonesParams(std::vector<OSG::Vec3f>& positions, 
                            std::vector<OSG::Quaternion>& orientations) {
    m_zonesPositions=positions;
    m_zonesOrientations=orientations;
    updateZones();
}

void Room::updateZones() {
    for(unsigned int z=0; z<m_zones.size(); ++z) {
        if(z<m_zonesPositions.size()) {
            m_zones[z]->setTranslation(m_zonesPositions[z]);
            m_zones[z]->setRotation(m_zonesOrientations[z]);
        }
    }
}

void Room::update(OSG::IntersectAction* intersectAction, 
                    const timeval& curTime, std::vector<Event>& events) {
    std::vector<Zone*>::iterator itZone;
    for(itZone=m_zones.begin();itZone!=m_zones.end();++itZone) {
        (*itZone)->update(events);
    }
}

void Room::markWormZoneForSelection(const unsigned int& zone) {
    assert(zone<m_zones.size());
    for(unsigned int z=0; z<m_zones.size();++z) {
        if(z!=zone) {
            m_zones[z]->unmarkForSelection();
        }
        else {
            m_zones[z]->markWormsForSelection();
        }
    }
    m_ropesRoot->setTravMask(OSG::TypeTraits<OSG::UInt32>::getZeroElement());
}

void Room::markTunnelZoneForSelection(const unsigned int& zone) {
    assert(zone<m_zones.size());
    for(unsigned int z=0; z<m_zones.size();++z) {
        if(z!=zone) {
            m_zones[z]->unmarkForSelection();
        }
        else {
            m_zones[z]->markTunnelsForSelection();
        }
    }
    m_ropesRoot->setTravMask(OSG::TypeTraits<OSG::UInt32>::getZeroElement());
}

void Room::markRopesForSelection() {
    for(unsigned int z=0; z<m_zones.size();++z) {
        m_zones[z]->unmarkForSelection();
    }
    m_ropesRoot->setTravMask(OSG::TypeTraits<OSG::UInt32>::getMax());
}

void Room::markForSelection() {
    for(unsigned int z=0; z<m_zones.size(); ++z) {
        m_zones[z]->markForSelection();
    }
    m_ropesRoot->setTravMask(OSG::TypeTraits<OSG::UInt32>::getMax());
}

