/***************************************************************************
 *  Room.hpp
 *  Part of Drile
 *  2010- Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _ROOM_H
#define _ROOM_H

#include <iostream>
#include <cstring>
#include <cmath>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <vector>
#include <map>

#include <libxml/parser.h>
#include <libxml/tree.h>

#include <OpenSG/OSGSimpleGeometry.h>
#include <OpenSG/OSGComponentTransform.h>
#include <OpenSG/OSGMaterialGroup.h>
#include <OpenSG/OSGSceneFileHandler.h>
#include <OpenSG/OSGIntersectAction.h>
#include "../Event.hpp"

class Zone;
class Grid;
class Tunnel;
class Worm;
class Rope;

class Room {
    public:
        Room(const int&);
        ~Room();

        void parse(xmlNodePtr);

        void update(OSG::IntersectAction*, const timeval&, 
                    std::vector<Event>&);

        inline void setID(int& id){m_id=id;}
        inline const int& getID(){return m_id;}
        inline const std::string& getName(){return m_name;}

        inline const OSG::Vec3f& getPosition() { 
            return m_roomTrans->getTranslation();
        }

        inline OSG::NodeRecPtr getNode(){return m_room;}
        inline OSG::NodeRecPtr getLightNode(){return m_light;}

        void setModel(const std::string&);

        inline void addRopeNode(OSG::NodeRecPtr ropeNode) { 
            m_ropesRoot->addChild(ropeNode);
        }

        Zone* addZone();
        void removeZone();
        void setZonesParams(std::vector<OSG::Vec3f>&, 
                            std::vector<OSG::Quaternion>&);
        void updateZones();
        Zone* getCreateZone(const unsigned int& id);
        void moveWorm(Worm*, const unsigned int& fromZone, 
                            const unsigned int& toZone);
        void setParameter(const std::string&, const std::string&);

        void markWormZoneForSelection(const unsigned int& zone);
        void markTunnelZoneForSelection(const unsigned int& zone);
        void markRopesForSelection();
        void markForSelection();

    private:

        int m_id;
        std::string m_name;

        OSG::Vec3f m_position;
        OSG::Vec3f m_scale;

        OSG::NodeRecPtr m_room;
        OSG::ComponentTransformRecPtr m_roomTrans;

        OSG::NodeRecPtr m_roomModelTransNode;
        OSG::ComponentTransformRecPtr m_roomModelTrans;
        OSG::NodeRecPtr m_roomModel;
        std::string m_roomModelStr;

        OSG::NodeRecPtr m_roomFloor;

        OSG::NodeRecPtr m_light;
        OSG::ComponentTransformRecPtr m_lightTrans;

        std::vector<OSG::Vec3f> m_zonesPositions;
        std::vector<OSG::Quaternion> m_zonesOrientations;
        std::vector<Zone*> m_zones;

        OSG::NodeRecPtr m_ropesRoot;
        std::vector<Rope*> m_ropes;

        void parseCoords(std::string &strCoords, OSG::Vec3f& coords);
};



#endif
