/***************************************************************************
 *  Rope.hpp
 *  Part of Drile
 *  2010-2013 Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _Rope_H
#define _Rope_H

#include <OpenSG/OSGSimpleGeometry.h>
#include <OpenSG/OSGComponentTransform.h>
#include <OpenSG/OSGMaterialGroup.h>
#include <OpenSG/OSGSimpleMaterial.h>
#include <OpenSG/OSGSceneFileHandler.h>
#include <OpenSG/OSGPolygonChunk.h>
#include <OpenSG/OSGIntersectAction.h>
#include "../Event.hpp"

#include "Manipulable.hpp"

class Room;

class Rope : public Manipulable {
    public:
        Rope();
        ~Rope();

        void setFirstRoom(Room* room); 
        void setSecondRoom(Room* room);
        void updatePosition();

        Room* getOtherRoom(Room* room);

        void grab(Piivert* tool, 
                    const int& hand, 
                        std::vector<Event>& outputEvents);
        void release(Piivert* tool, 
                        const int& hand, 
                            std::vector<Event>& outputEvents);
        void moveTo(Piivert* tool,
                        const int& hand, 
                            const float& depth, 
                                const OSG::Line& ray, 
                                    std::vector<Event>& outputEvents);
        void hit(std::vector<Event>& outputEvents);
        inline virtual const OSG::Vec3f& getPosition() {
            return m_ropeTrans[0]->getTranslation();
        }
        virtual void setHighlight(const bool&, const double& hue=0, 
                                    const double& scale=1.0);
        inline void removeFromZone(Zone*){}
        inline void moveToZone(Zone*){}
        inline void attachToMusician(Musician* mus){}
        inline void detachFromMusician(Musician* mus){}

    private:
        OSG::NodeRecPtr m_rope[2];
        OSG::ComponentTransformRecPtr m_ropeTrans[2];
        OSG::SimpleMaterialRefPtr m_material;
        OSG::PolygonChunkRefPtr m_matChunk;
        float m_thickness;
        Room* m_rooms[2];
        OSG::Vec3f m_initGrabPos;
        OSG::Vec3f m_initCamPos;
        OSG::Vec3f m_ropeDir;
        Room* m_roomFrom;
        Room* m_roomTo;
};

#endif
