/***************************************************************************
 *  Sign.cpp
 *  Part of Drile
 *  2010- Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Sign.hpp"

using namespace std;

Sign::Sign(const int& id, const std::string& text):Ghostable(),
                                                    m_id(id),
                                                    m_text(text), 
                                                    m_nbUsers(0), 
                                                    m_diameter(0.05),     
                                                    m_width(0.01),
                                                    m_grabbingToolID(-1) {


    OSG::VoidPAttachmentRefPtr ptr = OSG::VoidPAttachment::create();
    ptr->editFieldPtr()->setValue(this);

    m_passageSign = OSG::Node::create();
    m_passageSign->addAttachment(ptr);
    m_passageSignTrans = OSG::ComponentTransform::create();
    m_passageSign->setCore(m_passageSignTrans);

/*
    OSG::NodeRecPtr sign = 
        OSG::makeBox(m_diameter, m_diameter, m_width, 1,1,1);
*/

    std::string family = "SANS";
    OSG::TextFace::Style style  = OSG::TextFace::STYLE_PLAIN;
    OSG::TextVectorFaceRefPtr face = 
        OSG::TextVectorFace::create(family, style);
    if (face == 0) {
        throw logic_error("ERROR: Cannot create face object!");
    }

    // Lay out one single line of text
    OSG::TextLayoutParam layoutParam;
    layoutParam.horizontal = true;
    layoutParam.leftToRight = true;
    layoutParam.topToBottom = true;
    layoutParam.majorAlignment = OSG::TextLayoutParam::ALIGN_MIDDLE;
    layoutParam.minorAlignment = OSG::TextLayoutParam::ALIGN_MIDDLE;
    layoutParam.spacing = 1.f;
    layoutParam.length.push_back(0.f);
    layoutParam.maxExtend = 0.f;
    OSG::TextLayoutResult layoutResult;
    face->layout(text, layoutParam, layoutResult);

    // Create the text geometry
    OSG::Real32 scale = 0.03f;
    OSG::Real32 depth = 0.01f;
    OSG::NodeRecPtr sign = face->makeNode(layoutResult, scale, depth);
    sign->addAttachment(ptr);

    m_passageSign->addChild(sign);

    
    OSG::GeometryRefPtr geo2 = OSG::makeBoxGeo(m_diameter*2.0, m_diameter*2.0, 
                                                m_width*2.0, 1, 1, 1);
    m_highlightMaterial = OSG::SimpleMaterial::create();
    m_highlightMaterial->setLit(false);
    m_highlightMaterial->setColorMaterial(GL_NONE);
    m_highlightMaterial->setDiffuse(OSG::Color3f(0.5, 0.5, 0.5));
    m_highlightMaterial->setTransparency(1);
    OSG::PolygonChunkRefPtr chunk = OSG::PolygonChunk::create();
    chunk->setFrontMode(GL_LINE);
    chunk->setBackMode(GL_LINE);
    m_highlightMaterial->addChunk(chunk);
    geo2->setMaterial(m_highlightMaterial);
    m_highlight = OSG::Node::create();
    m_highlight->setCore(geo2);
    m_highlight->addAttachment(ptr);
    m_highlight->addAttachment(ptr);
    m_passageSign->addChild(m_highlight);

}

Sign::~Sign() {
    if(m_passageSign->getParent()) {
        m_passageSign->getParent()->subChild(m_passageSign);
    }
}

void Sign::setHighlight(const bool& highlight, const double& hue)
{
    if((!highlight && m_nbUsers==0) || (highlight && m_nbUsers>0)) {
        {
            m_highlightMaterial->setTransparency(!highlight);
            OSG::Color3f &color = m_highlightMaterial->editDiffuse();
            color.setValuesHSV(hue*360.0, 0.6, 0.8); 
        }
    }
}

bool Sign::addGrabbingTool(const int& toolID)
{
    bool added=false;
    //if nothing grabbing
    if(m_grabbingToolID==-1) {
        m_grabbingToolID=toolID;
        added=true;
    }
    else if(m_grabbingToolID==toolID) {
        added=true;
    }

    return added;
}

void Sign::removeGrabbingTool(const int& toolID)
{
    if(m_grabbingToolID==toolID) {
        m_grabbingToolID=-1;
    }
}

Ghostable* Sign::createGhost() {
    Ghostable* newGhost = new Sign(m_id,m_text);
    m_ghosts.push_back(newGhost);
    return newGhost;
}


