/***************************************************************************
 *  Sign.hpp
 *  Part of Drile
 *  2010- Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _SIGN_H
#define _SIGN_H

#include <iostream>
#include <cstring>
#include <cmath>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <vector>
#include <map>
#include <OpenSG/OSGSimpleGeometry.h>
#include <OpenSG/OSGComponentTransform.h>
#include <OpenSG/OSGMaterialGroup.h>
#include <OpenSG/OSGSimpleMaterial.h>
#include <OpenSG/OSGSimpleTexturedMaterial.h>
#include <OpenSG/OSGIntersectAction.h>
#include <OpenSG/OSGBoxVolume.h>
#include <OSGVoidPAttachment.h>
#include <OpenSG/OSGPolygonChunk.h>
#include <OpenSG/OSGTextVectorFace.h>
#include <OpenSG/OSGTextLayoutParam.h>
#include <OpenSG/OSGTextLayoutResult.h>

#include "Ghostable.hpp"

class Sign: public Ghostable {
    public:
        Sign(const int& id, const std::string& text);
        ~Sign();

        inline const int& getID(){return m_id;}

        inline OSG::NodeRecPtr getNode(){return m_passageSign;}
        inline OSG::NodeRecPtr getCollisionNode(){return m_passageSign;}
        inline void setPosition(const OSG::Vec3f& pos) { 
            m_passageSignTrans->setTranslation(pos); }
        inline OSG::ComponentTransformRecPtr getTrans(){
            return m_passageSignTrans;}

        void setHighlight(const bool& highlight, const double& hue=0);
        inline void addUser(){m_nbUsers++;}
        inline void removeUser(){m_nbUsers--;if(m_nbUsers<0){m_nbUsers=0;}}

        bool addGrabbingTool(const int& toolID);
        void removeGrabbingTool(const int& toolID);

        Ghostable* createGhost();

    protected:
        int m_id;
        std::string m_text;
        int m_nbUsers;

        double m_diameter;
        double m_width;

        OSG::NodeRecPtr m_passageSign;
        OSG::ComponentTransformRecPtr m_passageSignTrans;

        int m_grabbingToolID;
        OSG::NodeRefPtr m_highlight;
        OSG::SimpleMaterialRefPtr m_highlightMaterial;
};



#endif
