/***************************************************************************
 *  SyncWorm.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "SyncWorm.hpp"
#include "../Drile.hpp"

using namespace std;

SyncWorm::SyncWorm(const int& id):Worm(id) {
    std::ostringstream oss;
    oss<<m_id;

    {
        m_nbElements=10;
        m_elementHeight = WORM_SIZE/ m_nbElements;

        for(float i=0; i<m_nbElements-1; ++i) {
            OSG::GeometryRefPtr geo = OSG::makeCylinderGeo(m_elementHeight, 
                                                            WORM_SIZE/4.0, 
                                                            3, true, true, 
                                                            true);
            geo->setMaterial(m_wormMaterial);

            OSG::NodeRefPtr node = OSG::Node::create();
            OSG::setName(node, oss.str());
            node->setCore(geo);

            OSG::NodeRefPtr tempTransNode = OSG::Node::create();
            OSG::ComponentTransformRecPtr tempTrans = 
                        OSG::ComponentTransform::create();
            OSG::Quaternion quat2;
            tempTrans->setRotation(quat2);
            float scale=sin(i/m_nbElements*M_PI)+0.01;
            tempTrans->setScale(OSG::Vec3f(scale,1,scale));
            tempTransNode->addChild(node);
            tempTransNode->setCore(tempTrans);
            OSG::setName(tempTransNode, oss.str());

            OSG::NodeRecPtr transNode = OSG::Node::create();
            OSG::setName(transNode, oss.str());
            OSG::ComponentTransformRecPtr transCore 
                            = OSG::ComponentTransform::create();
            transCore->setTranslation(OSG::Vec3f(0,    
                                        i*m_elementHeight - 
                                            (m_elementHeight*m_nbElements)/2.0,
                                        0));
            OSG::Quaternion quat;
            quat.setIdentity();
            transCore->setRotation(quat);
            transNode->addChild(tempTransNode);
            transNode->setCore(transCore);
            m_elementsGroupTransNode->addChild(transNode);
            m_elementsTrans.push_back(transCore);
            m_elementsRotation.push_back(OSG::Vec3f(0, 0, 0));
            m_elementsWidth.push_back(scale);
        }


        m_worm->updateVolume();

        OSG::commitChanges();
    }

}

SyncWorm::~SyncWorm() {}

void SyncWorm::parse(xmlNodePtr){}

void SyncWorm::addChild(Worm* childWorm) {
    Worm::addChild(childWorm);
}

void SyncWorm::removeChild(Worm* worm) {
    Worm::removeChild(worm);
}

void SyncWorm::setParameter(const WORM_PARAMETER& param, double* values, 
                            unsigned int nbValues, bool valueFromAudio) {
/*
    switch(param) {
        case WORM_SPEED:
        case WORM_ROTATION_OFFSET: {
            if(!valueFromAudio) {
                //modify children 
                vector<Worm*>::iterator itChild;
                for(itChild=m_children.begin(); 
                        itChild!=m_children.end();++itChild) {
                    (*itChild)->setParameter(param,values,
                                            nbValues,valueFromAudio);
                }
            }
        }break;
        default:{
*/
            Worm::setParameter(param, values, nbValues, valueFromAudio);
//        }break;
//    }
}

void SyncWorm::setParameter(const std::string&, const std::string&) {}

Ghostable* SyncWorm::createGhost() {
    SyncWorm* newGhost = new SyncWorm(m_id);
    newGhost->setLiving(this);
    newGhost->setColor(m_parametersValues[WORM_COLOR]);
    newGhost->setHeight(m_parametersValues[WORM_HEIGHT]);
    newGhost->setTransparency(m_parametersValues[WORM_TRANSPARENCY]);
    newGhost->setScattering(m_parametersValues[WORM_SCATTERING]);
    m_ghosts.push_back(newGhost);
    return newGhost;
}

