/***************************************************************************
 *  Worm.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Worm.hpp"
#include "Zone.hpp"
#include "Musician.hpp"

using namespace std;

double Worm::WORM_SIZE=0.2;
double Worm::WORM_DISTANCE=0.2;

//create mappings
map<NODE_PARAMETER,Worm::WORM_PARAMETER> Worm::nodeWormMap = 
                                        Worm::createNodeWormMappings();
map<Worm::WORM_PARAMETER,NODE_PARAMETER> Worm::wormNodeMap = 
                                        Worm::createWormNodeMappings();


Worm::Worm(const int& id):Ghostable(), Manipulable(id), 
                            m_zoneID(0), m_depth(0),
                            m_appearanceChanged(false), 
                            m_staticShapeChanged(false), 
                            m_dynamicShapeChanged(false), 
                            m_looped(false),m_storedCollisionWorm(-1), 
                            m_parent(NULL), m_folded(false), 
                            m_treeRadius(0), 
                            m_curveAngle(M_PI), m_curveCenter(M_PI/2.0), 
                            m_behindTunnelPlane(false),
                            m_highlightScaleStep(0.5), 
                            m_duplicated(false) {

    m_parametersValues[WORM_COLOR]=0.5;
    m_parametersValues[WORM_HEIGHT]=0.5;
    m_parametersValues[WORM_SCATTERING]=0.5;
    m_parametersValues[WORM_TRANSPARENCY]=0.5;
    m_parametersValues[WORM_DISTORTION]=0.5;
    m_parametersValues[WORM_ALIGNEMENT]=0;
    m_parametersValues[WORM_SPEED]=0.5;
    m_parametersStoredValues[WORM_COLOR]=0.5;
    m_parametersStoredValues[WORM_HEIGHT]=0.5;
    m_parametersStoredValues[WORM_SCATTERING]=0.5;
    m_parametersStoredValues[WORM_TRANSPARENCY]=0.5;
    m_parametersStoredValues[WORM_DISTORTION]=0.5;
    m_parametersStoredValues[WORM_ALIGNEMENT]=0;
    m_parametersStoredValues[WORM_SPEED]=0.5;

    m_dynamicShapeValues.resize(1,0);

    gettimeofday(&m_storedCollisionTime, NULL);

    m_position[0]=0;
    m_position[1]=0;
    m_position[2]=0;

    {
        //create main node and material
        OSG::VoidPAttachmentRefPtr ptr = OSG::VoidPAttachment::create();
        ptr->editFieldPtr()->setValue(static_cast<Manipulable*>(this));

        m_worm = OSG::Node::create();
        m_wormTrans = OSG::ComponentTransform::create();
        m_wormTrans->setTranslation(m_position);
        m_worm->setCore(m_wormTrans);
        m_worm->addAttachment(ptr);

        m_elementsGroupTransNode = OSG::Node::create();
        m_elementsGroupTrans = OSG::ComponentTransform::create();
        m_elementsGroupTransNode->setCore(m_elementsGroupTrans);
        OSG::Vec3f& scale =  m_elementsGroupTrans->editScale();
        scale[1]=(m_parametersValues[WORM_HEIGHT]*0.8+0.2);
        scale[0]=0.5;
        scale[2]=0.5;
        m_worm->addChild(m_elementsGroupTransNode);

        m_wormMaterial = OSG::SimpleMaterial::create();
        m_wormMaterial->setLit(true);
        m_wormMaterial->setColorMaterial(GL_DIFFUSE);
        m_wormMaterial->setAmbient(OSG::Color3f(0, 0, 0));
        OSG::PolygonChunkRefPtr matchunk = OSG::PolygonChunk::create();
        matchunk->setCullFace(GL_BACK);
        //matchunk->setFrontMode(GL_LINE);
        matchunk->setSmooth(false);
        m_wormMaterial->addChunk(matchunk);

        //create highlight
        OSG::GeometryRefPtr geo = OSG::makeBoxGeo(WORM_SIZE*0.6, 
                                    WORM_SIZE*1.1, WORM_SIZE*0.6, 1, 1, 1);
        m_highlightMaterial = OSG::SimpleMaterial::create();
        m_highlightMaterial->setLit(false);
        m_highlightMaterial->setColorMaterial(GL_NONE);
        m_highlightMaterial->setDiffuse(OSG::Color3f(0.5, 0.5, 0.5));
        m_highlightMaterial->setTransparency(1.0);
        OSG::PolygonChunkRefPtr chunk = OSG::PolygonChunk::create();
        chunk->setFrontMode(GL_LINE);
        chunk->setBackMode(GL_LINE);
        m_highlightMaterial->addChunk(chunk);
        geo->setMaterial(m_highlightMaterial);

        m_highlight = OSG::Node::create();
        m_highlight->addAttachment(ptr);
        m_highlight->setCore(geo);

        m_highlightTrans = OSG::ComponentTransform::create();
        m_highlightTransNode = OSG::Node::create();
        m_highlightTransNode->setCore(m_highlightTrans);
        m_highlightTransNode->addChild(m_highlight);

        m_worm->addChild(m_highlightTransNode);

        //create record shape
        OSG::GeometryRecPtr recBranch = 
                    OSG::makeCylinderGeo(WORM_SIZE, 
                                        WORM_SIZE/100, 3, true, true, true);
        OSG::GeometryRecPtr recWorm = 
                    OSG::makeCylinderGeo(WORM_SIZE/4, 
                                        WORM_SIZE/4, 3, true, true, true);
        m_recMaterial = OSG::SimpleMaterial::create();
        m_recMaterial->setLit(true);
        m_recMaterial->setColorMaterial(GL_AMBIENT_AND_DIFFUSE);
        m_recMaterial->setAmbient(OSG::Color3f(0.1, 0.1, 0.1));
        OSG::Color3f &recCol = m_recMaterial->editDiffuse();
        recCol.setValuesHSV(180, 0.7, 0.5);
        m_recMaterial->setTransparency(1.0);
        recBranch->setMaterial(m_recMaterial);
        recWorm->setMaterial(m_recMaterial);

        OSG::NodeRecPtr recWormNode = OSG::Node::create();
        OSG::NodeRecPtr recBranchNode = OSG::Node::create();
        recBranchNode->setCore(recBranch);
        recWormNode->setCore(recWorm);
        recBranchNode->addAttachment(ptr);
        recWormNode->addAttachment(ptr);

        OSG::NodeRecPtr recTransNode = OSG::Node::create();
        OSG::NodeRecPtr recWormTransNode = OSG::Node::create();
        recWormTransNode->addChild(recWormNode);
        recTransNode->addChild(recWormTransNode);
        recTransNode->addAttachment(ptr);
        recWormTransNode->addAttachment(ptr);

        m_recTrans = OSG::ComponentTransform::create();
        recWormTransNode->setCore(m_recTrans);

        OSG::ComponentTransformRecPtr recBranchTrans = 
                            OSG::ComponentTransform::create();
        recBranchTrans->setTranslation(OSG::Vec3f(0, 0, WORM_SIZE));
        recBranchTrans->setCenter(OSG::Vec3f(0, 0, -WORM_SIZE/2.0));
        OSG::Quaternion quat;
        quat.setValue(0, -M_PI/3.0, 0);
        recBranchTrans->setRotation(quat);
        recTransNode->setCore(recBranchTrans);
        recTransNode->addChild(recWormTransNode);

        m_worm->addChild(recTransNode);

        //create input / output branches
        m_inputBranchMat = OSG::SimpleMaterial::create();

        //input vert
        OSG::GeometryRecPtr inputVertBranchGeo = 
                OSG::makeBoxGeo(WORM_SIZE/100.0, 
                    WORM_SIZE/2.0, WORM_SIZE/100.0, 1, 1, 1);
        OSG::ComponentTransformRecPtr inputVertBranchTrans = 
                                    OSG::ComponentTransform::create();
        inputVertBranchTrans->setTranslation(OSG::Vec3f(0, WORM_SIZE/4.0, 0));
        OSG::NodeRecPtr inputVertBranchTransNode = OSG::Node::create();
        inputVertBranchGeo->setMaterial(m_inputBranchMat);
        OSG::NodeRecPtr inputVertBranchNode = OSG::Node::create();
        inputVertBranchNode->setCore(inputVertBranchGeo);
        inputVertBranchTransNode->setCore(inputVertBranchTrans);
        inputVertBranchTransNode->addChild(inputVertBranchNode);
        m_worm->addChild(inputVertBranchTransNode);

        //input front
        OSG::GeometryRecPtr inputFrontBranchGeo = 
                OSG::makeBoxGeo(WORM_SIZE/100.0, WORM_SIZE/100.0,
                                WORM_SIZE/2.0, 1, 1, 1);
        OSG::ComponentTransformRecPtr inputFrontBranchTrans = 
                                    OSG::ComponentTransform::create();
        inputFrontBranchTrans->setTranslation(
                                OSG::Vec3f(0, WORM_SIZE/2.0, WORM_SIZE/4.0));
        OSG::NodeRecPtr inputFrontBranchTransNode = OSG::Node::create();
        inputFrontBranchGeo->setMaterial(m_inputBranchMat);
        OSG::NodeRecPtr inputFrontBranchNode = OSG::Node::create();
        inputFrontBranchNode->setCore(inputFrontBranchGeo);
        inputFrontBranchTransNode->setCore(inputFrontBranchTrans);
        inputFrontBranchTransNode->addChild(inputFrontBranchNode);
        m_worm->addChild(inputFrontBranchTransNode);

        m_downOutputBranchMat = OSG::SimpleMaterial::create();
        m_frontOutputBranchMat = OSG::SimpleMaterial::create();
        m_backOutputBranchMat = OSG::SimpleMaterial::create();

        //output vert
        OSG::GeometryRecPtr outputVertBranchGeo = 
                OSG::makeBoxGeo(WORM_SIZE/100.0, 
                    WORM_SIZE/2.0, WORM_SIZE/100.0, 1, 1, 1);
        OSG::ComponentTransformRecPtr outputVertBranchTrans = 
                                    OSG::ComponentTransform::create();
        outputVertBranchTrans->setTranslation(OSG::Vec3f(0,-WORM_SIZE/4.0,0));
        OSG::NodeRecPtr outputVertBranchTransNode = OSG::Node::create();
        outputVertBranchGeo->setMaterial(m_downOutputBranchMat);
        OSG::NodeRecPtr outputVertBranchNode = OSG::Node::create();
        outputVertBranchNode->setCore(outputVertBranchGeo);
        outputVertBranchTransNode->setCore(outputVertBranchTrans);
        outputVertBranchTransNode->addChild(outputVertBranchNode);
        m_worm->addChild(outputVertBranchTransNode);

        //output front
        OSG::GeometryRecPtr outputFrontBranchGeo = 
                OSG::makeBoxGeo(WORM_SIZE/100.0, WORM_SIZE/100.0,
                                WORM_SIZE/2.0, 1, 1, 1);
        OSG::ComponentTransformRecPtr outputFrontBranchTrans = 
                                    OSG::ComponentTransform::create();
        outputFrontBranchTrans->setTranslation(
                                OSG::Vec3f(0,-WORM_SIZE/2.0,WORM_SIZE/4.0));
        OSG::NodeRecPtr outputFrontBranchTransNode = OSG::Node::create();
        outputFrontBranchGeo->setMaterial(m_frontOutputBranchMat);
        OSG::NodeRecPtr outputFrontBranchNode = OSG::Node::create();
        outputFrontBranchNode->setCore(outputFrontBranchGeo);
        outputFrontBranchTransNode->setCore(outputFrontBranchTrans);
        outputFrontBranchTransNode->addChild(outputFrontBranchNode);
        m_worm->addChild(outputFrontBranchTransNode);

        //output back
        OSG::GeometryRecPtr outputBackBranchGeo = 
                OSG::makeBoxGeo(WORM_SIZE/100.0, WORM_SIZE/100.0,
                                WORM_SIZE/2.0, 1, 1, 1);
        OSG::ComponentTransformRecPtr outputBackBranchTrans = 
                                    OSG::ComponentTransform::create();
        outputBackBranchTrans->setTranslation(
                                OSG::Vec3f(0,-WORM_SIZE/2.0,-WORM_SIZE/4.0));
        OSG::NodeRecPtr outputBackBranchTransNode = OSG::Node::create();
        outputBackBranchGeo->setMaterial(m_backOutputBranchMat);
        OSG::NodeRecPtr outputBackBranchNode = OSG::Node::create();
        outputBackBranchNode->setCore(outputBackBranchGeo);
        outputBackBranchTransNode->setCore(outputBackBranchTrans);
        outputBackBranchTransNode->addChild(outputBackBranchNode);
        m_worm->addChild(outputBackBranchTransNode);

        //hide all branches
        m_inputBranchMat->setTransparency(1);
        m_downOutputBranchMat->setTransparency(1);
        m_frontOutputBranchMat->setTransparency(1);
        m_backOutputBranchMat->setTransparency(1);
        
        double val=0.5;
        setParameter(WORM_COLOR, &val, 1);

        OSG::commitChanges();
    }
}

Worm::~Worm() {
    if(m_worm->getParent()) {
        m_worm->getParent()->subChild(m_worm);
    }
}

void Worm::parse(xmlNodePtr node) {
    char* value=NULL;
    value = (char*)xmlGetProp(node,(xmlChar*)"position");
    if(value!=NULL) {
        string valueStr=value;
        istringstream is,is2,is3;
        size_t pos;
        pos=valueStr.find_first_of("/");
        if(pos!=std::string::npos) {
            is.str(valueStr.substr(0,pos));
            is>>m_position[0];
        }
        valueStr=valueStr.substr(pos+1);
        is3.str(valueStr);
        is3>>m_position[1];
    }
    m_wormTrans->setTranslation(m_position);
}

void Worm::setParameter(const std::string& parameter, 
                        const std::string& value) {}

void Worm::update(std::vector<Event>& outputEvents) {
    if(m_appearanceChanged) {
        setColor(m_appearanceValues[WORM_COLOR]);
        setHeight(m_appearanceValues[WORM_HEIGHT]);
        setTransparency(m_appearanceValues[WORM_TRANSPARENCY]);
        setScattering(m_appearanceValues[WORM_SCATTERING]);
        setAlignement(m_appearanceValues[WORM_ALIGNEMENT]);
        setRotationOffset(m_appearanceValues[WORM_ROTATION_OFFSET]);
        m_appearanceChanged=false;
    }

    //add output env events
    outputEvents.insert(outputEvents.end(), m_outputEvents.begin(), 
                            m_outputEvents.end());
    m_outputEvents.clear();
}

void Worm::setColor(const double& value) {
    m_parametersValues[WORM_COLOR]=value;
    OSG::Color3f &color = m_wormMaterial->editDiffuse();
    color.setValuesHSV(fmod(value*5.0,1.0)*360.0,  //hue 
                        0.2*(1.0-abs(value*2.0-1.0))+0.1, //saturation
                        //0.6*(1.0-abs(value*2.0-1.0))+0.1, //saturation
                        value*0.9+0.1); //value
    m_inputBranchMat->setDiffuse(color);
    m_downOutputBranchMat->setDiffuse(color);
    m_frontOutputBranchMat->setDiffuse(color);
    m_backOutputBranchMat->setDiffuse(color);
}

void Worm::setHeight(const double& value) {
    m_parametersValues[WORM_HEIGHT]=value;
    if(m_folded && m_parent) {
        curve();
    }
    else {
        OSG::Vec3f& scale =  m_elementsGroupTrans->editScale();
        scale[1]=(value*0.8+0.2);
    }
}

void Worm::setTransparency(const double& value) {
    m_parametersValues[WORM_TRANSPARENCY]=value;
    m_wormMaterial->setTransparency(value*0.7);
}

void Worm::setScattering(const double& value) {
    m_parametersValues[WORM_SCATTERING]=value;
    vector<OSG::ComponentTransformRecPtr>::iterator itTrans;
    for(itTrans=m_elementsTrans.begin(); 
            itTrans!=m_elementsTrans.end();++itTrans) {
        OSG::Vec3f& scale = (*itTrans)->editScale();
        scale[1] = 1 - (value*0.8);
    }
}

void Worm::setRotationOffset(const double& value) {
    m_parametersValues[WORM_ROTATION_OFFSET]=value;
}

void Worm::setAlignement(const double& value) {
    m_parametersValues[WORM_ALIGNEMENT]=value;
    unsigned int el=0;
    vector<OSG::ComponentTransformRecPtr>::iterator itTrans;
    for(itTrans=m_elementsTrans.begin(); 
            itTrans!=m_elementsTrans.end(); ++itTrans, el++) {
        m_elementsRotation[el][0]=(value*M_PI/8.0)*(float(el%2)*2.0-1.0);
        OSG::Quaternion& rot = (*itTrans)->editRotation();
        rot.setValue(m_elementsRotation[el][0], 
                        m_elementsRotation[el][1], 
                            m_elementsRotation[el][2]);
    }
}

void Worm::setSpeed(const double& value) {
    m_parametersValues[WORM_SPEED]=value;
}

void Worm::sendParameterValue(const WORM_PARAMETER& param) {
    m_outputEvents.push_back(Event());
    Event& event = m_outputEvents.back();
    event.type = EVENT_NODE;
    event.node = m_id;
    event.parameter = Worm::wormNodeMap[param];
    event.values[0]=m_parametersValues[param];
}

void Worm::setParameter(const WORM_PARAMETER& param, double* values, 
                        unsigned int nbValues, bool valueFromAudio) {
    switch(param) {
        case WORM_APPEARANCE: {
            if(!m_appearanceChanged) {
                m_appearanceValues[WORM_HEIGHT]=
                                values[wormNodeMap[WORM_HEIGHT]];
                m_appearanceValues[WORM_COLOR]=
                                values[wormNodeMap[WORM_COLOR]];
                m_appearanceValues[WORM_DISTORTION]=
                                values[wormNodeMap[WORM_DISTORTION]];
                m_appearanceValues[WORM_TRANSPARENCY]=
                                values[wormNodeMap[WORM_TRANSPARENCY]];
                m_appearanceValues[WORM_SCATTERING]=
                                values[wormNodeMap[WORM_SCATTERING]];
                m_appearanceValues[WORM_ALIGNEMENT]=
                                values[wormNodeMap[WORM_ALIGNEMENT]];
                m_appearanceValues[WORM_ROTATION_OFFSET]=
                                values[wormNodeMap[WORM_ROTATION_OFFSET]];
                m_appearanceChanged=true;
            }
        }break;
        case WORM_COLOR: {
            setColor(values[0]);
            if(!valueFromAudio) {
                sendParameterValue(WORM_COLOR);
            }

        }break;
        case WORM_HEIGHT: {
            setHeight(values[0]);
            if(!valueFromAudio) {
                sendParameterValue(WORM_HEIGHT);
            }
        }break;
        case WORM_DISTORTION: {
            //setDistortion(values[0]);
            if(!valueFromAudio) {
                sendParameterValue(WORM_DISTORTION);
            }
        }break;
        case WORM_TRANSPARENCY: {
            setTransparency(values[0]);
            if(!valueFromAudio) {
                sendParameterValue(WORM_TRANSPARENCY);
            }
        }break;
        case WORM_SCATTERING: {
            setScattering(values[0]);
            if(!valueFromAudio) {
                sendParameterValue(WORM_SCATTERING);
            }
        }break;
        case WORM_ALIGNEMENT: {
            setAlignement(values[0]);
            if(!valueFromAudio) {
                sendParameterValue(WORM_ALIGNEMENT);
            }
        }break;
        case WORM_SPEED: {
            setSpeed(values[0]);
            if(!valueFromAudio) {
                sendParameterValue(WORM_SPEED);
            }
        }break;
        case WORM_ROTATION_OFFSET: {
            setRotationOffset(values[0]);                
            if(!valueFromAudio) {
                sendParameterValue(WORM_ROTATION_OFFSET);
            }
        }break;
        case WORM_DYNAMIC_SHAPE: {
            m_dynamicShapeValues.assign(values, values+nbValues);
            if(m_dynamicShapeValues[0]>2.0) {
                m_dynamicShapeValues[0]=2.0;
            }
            if(m_folded && m_parent) {
                OSG::Vec3f &scale = m_elementsGroupTrans->editScale();
                scale[0]=0.5;
                scale[2]=0.5;
                unsigned int el=0;
                vector<OSG::ComponentTransformRecPtr>::iterator itTrans;
                for(itTrans=m_elementsTrans.begin(); 
                        itTrans!=m_elementsTrans.end(); ++itTrans, ++el) {
                    OSG::Vec3f& scale = (*itTrans)->editScale();
                    scale[0]=m_elementsWidth[el]+m_dynamicShapeValues[0];
                    scale[2]=m_elementsWidth[el]+m_dynamicShapeValues[0];
                }
            }
            else {
                OSG::Vec3f &scale = m_elementsGroupTrans->editScale();
                scale[0]=0.5+m_dynamicShapeValues[0];
                scale[2]=0.5+m_dynamicShapeValues[0];
            }

            if(m_recMaterial->getTransparency()<1.0) {
                OSG::Vec3f& recScale = m_recTrans->editScale();
                recScale[0] = 
                recScale[1] = 
                recScale[2] = 
                    min<double>(1,recScale[2]+m_dynamicShapeValues[0]*0.01);
            }
        }break;
        case WORM_STATIC_SHAPE: {
            m_staticShapeValues.assign(values, values+nbValues);
            m_staticShapeChanged=true;
        }break;
        case WORM_ROTATION: {
            unsigned int el=0;
            vector<OSG::ComponentTransformRecPtr>::iterator itTrans;
            for(itTrans=m_elementsTrans.begin(); 
                    itTrans!=m_elementsTrans.end(); ++itTrans, ++el) {
                m_elementsRotation[el][1]=-values[0]*2.0*M_PI;
                OSG::Quaternion& rot = (*itTrans)->editRotation();
                rot.setValue(m_elementsRotation[el][0], 
                                m_elementsRotation[el][1], 
                                    m_elementsRotation[el][2]);
            }
        }break;
        case WORM_LOOP_RECORDING: {
            if(values[0]==1) {
                m_recMaterial->setTransparency(0);
                m_recTrans->setScale(OSG::Vec3f(0.5,0.5,0.5));
            }
            else {
                m_recMaterial->setTransparency(1);
            }
        }break;
        default:break;
    }

    vector<Ghostable*>::iterator itGhost;
    for(itGhost=m_ghosts.begin(); itGhost!=m_ghosts.end();++itGhost) {
        static_cast<Worm*>(*itGhost)->setParameter(param, values, 
                                                    nbValues, true);
    }
}

void Worm::setHighlight(const bool& highlight, const double& hue, 
                        const double& scale) {
    if((!highlight && m_nbUsers==0) || (highlight && m_nbUsers>0)) {
        {
            m_highlightMaterial->setTransparency(!highlight);
            OSG::Color3f &color = m_highlightMaterial->editDiffuse();
            color.setValuesHSV(hue*360.0, 0.6, 0.8); 
        }
    }
}

void Worm::setVisible(bool visible) {
    if(visible) {
        m_wormMaterial->setTransparency(0);
        if(m_parent) {
            m_inputBranchMat->setTransparency(0);
        }
        if(m_children.size()>0) {
            m_downOutputBranchMat->setTransparency(0);
            m_backOutputBranchMat->setTransparency(0);
        }
        m_worm->setTravMask(OSG::TypeTraits<OSG::UInt32>::getMax());
    }
    else {
        m_worm->setTravMask(OSG::TypeTraits<OSG::UInt32>::getZeroElement());
        m_wormMaterial->setTransparency(1);
        m_inputBranchMat->setTransparency(1);
        m_downOutputBranchMat->setTransparency(1);
        m_backOutputBranchMat->setTransparency(1);
    }
}

void Worm::storeParameterValue(const WORM_PARAMETER& param) {
    m_parametersStoredValues[param]=m_parametersValues[param];
}

void Worm::restoreParameterValue(const WORM_PARAMETER& param) {
    double val=m_parametersStoredValues[param];
    setParameter(param, &val, 1);
}

void Worm::setDepth(const int& depth) {
    m_depth=depth;
    if(depth<0) {
        setVisible(false);
    }
    else {
        setVisible(true);
    }
    vector<Worm*>::iterator itWorm=m_children.begin();
    for(;itWorm!=m_children.end(); ++itWorm) {    
        (*itWorm)->setParent(this); 
    }
}

void Worm::setParent(Worm* parent) { 
    m_parent=parent;
    if(parent) {
        //setDepth(m_parent->getDepth()+1);
        m_inputBranchMat->setTransparency(0);
    }
    else {
        //setDepth(0);
        m_inputBranchMat->setTransparency(1);
        m_curveAngle=M_PI;
        m_curveCenter=M_PI/2.0;
        m_treeCenter=OSG::Vec3f(0, 0, 0);
        m_treeRadius=0;
        //uncurve();
    }
}

void Worm::addChild(Worm* childWorm) {
    //add the worm
    m_children.push_back(childWorm);
    childWorm->setParent(this);

    {
        //create a new branch
        OSG::GeometryRecPtr branchGeo = OSG::makeBoxGeo(WORM_SIZE/100.0, 
                    WORM_SIZE/100.0, 1.0, 1, 1, 1);
        branchGeo->setMaterial(m_wormMaterial);
        OSG::NodeRecPtr branchNode = OSG::Node::create();
        branchNode->setCore(branchGeo);
        OSG::ComponentTransformRecPtr branchScale = 
                                    OSG::ComponentTransform::create();
        OSG::NodeRecPtr branchTempTransNode = OSG::Node::create();
        branchTempTransNode->setCore(branchScale);
        branchTempTransNode->addChild(branchNode);
        m_branchesScales.push_back(branchScale);
        OSG::ComponentTransformRecPtr branchTrans = 
                                    OSG::ComponentTransform::create();
        OSG::NodeRecPtr branchTransNode = OSG::Node::create();
        branchTransNode->setCore(branchTrans);
        branchTransNode->addChild(branchTempTransNode);
        m_branches.push_back(branchTransNode);
        m_worm->addChild(branchTransNode);

        //update branches
        updateBranchesPositions();
    }
    updateChildrenPositions();

    if(m_folded) {
        for(unsigned int i=0;i<m_children.size();++i) {
            m_children[i]->curve();
        }
        //hide the output back branch 
        m_downOutputBranchMat->setTransparency(1);
        m_backOutputBranchMat->setTransparency(1);
        m_frontOutputBranchMat->setTransparency(1);
        m_inputBranchMat->setTransparency(1);
    }
    else {
        //display the output back branch 
        m_downOutputBranchMat->setTransparency(0);
        m_backOutputBranchMat->setTransparency(0);
    }
}

void Worm::removeChild(Worm* childWorm) {
    bool erased=false;
    for(vector<Worm*>::iterator itWorm=m_children.begin(); 
                itWorm!=m_children.end() && !erased; ++itWorm) {    
        if((*itWorm)->getID()==childWorm->getID()) {
            erased=true;
            m_children.erase(itWorm);
        }
    }

    if(erased) {
        childWorm->setParent(NULL);
        //remove a branch
        m_worm->subChild(m_branches[0]);
        m_branches.erase(m_branches.begin());
        m_branchesScales.erase(m_branchesScales.begin());

        //update positions
        updateChildrenPositions();
        updateBranchesPositions();
    }
}

void Worm::updateBranchesPositions() {
    if(m_folded) {
        for(unsigned int i=0;i<m_branches.size();++i) {
            OSG::Vec3f &scale = m_branchesScales[i]->editScale();
            scale[2] = 0;
        }
    }
    else {
        //reposition all the branches
        for(unsigned int i=0;i<m_branches.size();++i) {
            OSG::Quaternion& rot = (static_cast<OSG::ComponentTransform*>
                                    (m_branches[i]->getCore()))->editRotation();
            OSG::Vec3f &center = (static_cast<OSG::ComponentTransform*>
                                    (m_branches[i]->getCore()))->editCenter();
            OSG::Vec3f &pos = (static_cast<OSG::ComponentTransform*>
                                (m_branches[i]->getCore()))->editTranslation();
            OSG::Vec3f &scale = m_branchesScales[i]->editScale();
            OSG::Vec3f vectorTo;
            vectorTo[1]= WORM_SIZE*(1.5*double(i) - 
                        int(double(m_branches.size())/2.0) - 0.5) + WORM_SIZE;
            if(m_wormTrans->getTranslation()[2]<-WORM_DISTANCE/2.0) {
                vectorTo[0]=WORM_SIZE/2.0;
                vectorTo[2]=-WORM_SIZE;
            }
            else {
                vectorTo[0]=WORM_DISTANCE/3.0;
                vectorTo[2]=-WORM_DISTANCE+WORM_SIZE;
            }
            rot.setValue(OSG::Vec3f(0,0,-1), vectorTo);
            scale[2] = vectorTo.length();
            center[2] = vectorTo.length()/2.0;
            pos[2]=-vectorTo.length()/2.0 - WORM_SIZE/2.0;
            pos[1]=-WORM_SIZE/2.0;
        }
    }
}

void Worm::updateChildrenPositions() {
    if(m_wormMaterial->getTransparency()==0) {
        const OSG::Vec3f& pos = m_wormTrans->getTranslation();
        if(m_folded) {
            OSG::Quaternion quat;
            OSG::Vec3f initDir(0, m_treeRadius+Worm::WORM_SIZE/2.0, 0);
            OSG::Vec3f dir;
            float angleStep=m_curveAngle / float(m_children.size());
            float angle=m_curveCenter-m_curveAngle/2.0+angleStep/2.0;
            for(unsigned int i=0;i<m_children.size();++i) {
                quat.setValue(0, 0, -angle);
                quat.multVec(initDir, dir);
                m_children[i]->setRadialTree(-dir, initDir[1], 
                                                angle, angleStep);
                dir+=m_wormTrans->getTranslation()+m_treeCenter;
                m_children[i]->getTrans()->setTranslation(dir);
                m_children[i]->updateChildrenPositions();
                angle+=angleStep;
            }
        } else {
            for(unsigned int i=0;i<m_children.size();++i) {
                if(m_children[i]->getZoneID()==m_zoneID) {
                    OSG::Vec3f& childPos = 
                        m_children[i]->getTrans()->editTranslation();
                    if(pos[2]<-WORM_DISTANCE/2.0) {
                        childPos[0]=pos[0] + WORM_SIZE/2.0;
                        childPos[2]=pos[2] - WORM_SIZE;
                    }
                    else {
                        childPos[0]=pos[0] + WORM_DISTANCE/3.0;
                        childPos[2]=pos[2] - WORM_DISTANCE;
                    }
                    childPos[1]=pos[1]+ 
                                WORM_SIZE*(1.5*double(i) 
                                    - int(double(m_children.size())/2.0) - 0.5);
                    m_children[i]->updateChildrenPositions();
                }
            }
        }
        if(pos[2]<-WORM_DISTANCE/2.0 && !m_behindTunnelPlane) {
            m_behindTunnelPlane=true;
            updateBranchesPositions();
        }
        else if(pos[2]>-WORM_DISTANCE/2.0 && m_behindTunnelPlane) {
            m_behindTunnelPlane=false;
            updateBranchesPositions();
        }
    }
}

void Worm::fold(const int& depth) {
    if(depth>0) {
        m_folded=true;
        updateChildrenPositions();
        for(unsigned int i=0;i<m_children.size();++i) {
            m_children[i]->curve();
            m_children[i]->fold(depth-1);
        }
    }
}

void Worm::unfold(const int& depth) {
    if(depth>0) {
        m_folded=false;
        updateChildrenPositions();
        for(unsigned int i=0;i<m_children.size();++i) {
            m_children[i]->uncurve();
            m_children[i]->unfold(depth-1);
        }
    }
}

void Worm::setRadialTree(const OSG::Vec3f& treeCenter, const float& treeRadius, 
                            const float& curveCenter, const float& curveAngle) {
    m_treeRadius=treeRadius;
    m_treeCenter=treeCenter;
    m_curveCenter=curveCenter;
    m_curveAngle=curveAngle;
}

void Worm::curve() {
    m_downOutputBranchMat->setTransparency(1);
    m_backOutputBranchMat->setTransparency(1);
    m_inputBranchMat->setTransparency(1);

    OSG::Vec3f& scale =  m_elementsGroupTrans->editScale();
    scale[1]=0.6;

    OSG::Quaternion quat;
    OSG::Vec3f initDir(0, m_treeRadius, 0);
    OSG::Vec3f dir;
    float angle=m_curveCenter
                + (m_curveAngle/2.0*m_parametersValues[WORM_HEIGHT]);
    float angleStep=(m_curveAngle*m_parametersValues[WORM_HEIGHT])
                        /float(m_elementsTrans.size());
    vector<OSG::ComponentTransformRecPtr>::iterator itTrans;
    unsigned int el=0;
    for(itTrans=m_elementsTrans.begin(); 
            itTrans!=m_elementsTrans.end(); ++itTrans, ++el) {
        quat.setValue(0, 0, -angle);
        quat.multVec(initDir, dir);
        dir+=m_treeCenter;
        (*itTrans)->setTranslation(dir);
        m_elementsRotation[el][2]=-(M_PI/2.0+angle);
        OSG::Quaternion& rot = (*itTrans)->editRotation();
        rot.setValue(m_elementsRotation[el][0], 
                        m_elementsRotation[el][1], 
                            m_elementsRotation[el][2]);
        angle-=angleStep;
    }
    for(unsigned int i=0;i<m_children.size();++i) {
        m_children[i]->curve();
    }
}

void Worm::uncurve() {
    m_downOutputBranchMat->setTransparency(0);
    m_backOutputBranchMat->setTransparency(0);
    if(m_parent) {
        m_inputBranchMat->setTransparency(0);
    }
    vector<OSG::ComponentTransformRecPtr>::iterator itTrans;
    float el=0;
    for(itTrans=m_elementsTrans.begin(); 
            itTrans!=m_elementsTrans.end(); ++itTrans, el++) {
        (*itTrans)->setTranslation(
                        OSG::Vec3f(0, el*m_elementHeight-
                            (m_elementHeight*m_nbElements)/2.0, 0));
        m_elementsRotation[el][2]=0;
        OSG::Quaternion& rot = (*itTrans)->editRotation();
        rot.setValue(m_elementsRotation[el][0], 
                        m_elementsRotation[el][1], 
                            m_elementsRotation[el][2]);
    }
}

bool Worm::testCollisionWithVolume(OSG::BoxVolume& collidingVolume) {
    OSG::BoxVolume wormVolume;
    m_elementsGroupTransNode->getWorldVolume(wormVolume);
    return wormVolume.intersect(collidingVolume);
}

void Worm::testCollisionWithWorm(Worm* collidingWorm, 
                                    std::vector<Event>& events, 
                                            const bool& released) {
    OSG::BoxVolume worm1Volume, worm2BranchVolume, worm2Volume;
    m_elementsGroupTransNode->getWorldVolume(worm1Volume);
    //test collision with main elements    
    collidingWorm->getCollisionNode()->getWorldVolume(worm2Volume);
    if(worm1Volume.intersect(worm2Volume)) {
        if(released) {
            events.push_back(Event());
            Event& event = events.back();
            event.type = EVENT_TREE;
            event.parameter = TREE_COLLIDE;
            event.values[0]=collidingWorm->getID();
            event.values[1]=m_id;
        }
    }
}

void Worm::grab(Piivert* tool, 
                    const int& hand, 
                        std::vector<Event>& outputEvents) {
    Manipulable::grab(tool, hand, outputEvents);
    m_wormTrans->editTranslation()[2]=0;
    updateChildrenPositions();
    updateBranchesPositions();
    m_duplicated=false;
}

void Worm::release(Piivert* tool, 
                        const int& hand, 
                            std::vector<Event>& outputEvents) {
    Manipulable::release(tool, hand, outputEvents);
    m_wormTrans->setScale(OSG::Vec3f(1, 1, 1));

    m_zone->testCollisionsForWorm(this, outputEvents, true);
    m_wormTrans->editTranslation()[2]=0;
    if(m_parent) {
        m_parent->updateChildrenPositions();
    }
    else {
        updateChildrenPositions();
    }

    m_grabInitalised=false;
    updateBranchesPositions();
}

void Worm::moveTo(Piivert* tool,
                        const int& hand, 
                            const float& depth, 
                                const OSG::Line& ray, 
                                    std::vector<Event>& outputEvents) {
    OSG::Vec3f pos=m_wormTrans->editTranslation();
    OSG::Plane plane(OSG::Vec3f(0,0,-1), OSG::Pnt3f(0, 0, depth));
    plane.intersect(ray, pos);
    pos[2]+=m_zone->getDepth();
    if(pos[2]<-WORM_DISTANCE/2.0) {
        pos[2]=-WORM_DISTANCE/2.0;
    }
    else if(pos[2]>0) {
        pos[2]=0;
    }

    if(!m_grabInitalised) {
        m_grabOffset=m_wormTrans->getTranslation()-pos;
        m_storedGrabPos=pos;
        m_grabInitalised=true;
    }

    if(m_isGrabbedWithBothHands && !m_duplicated) {
        if(hand==0) { //first hand, stretch
            float dist = (m_storedGrabPos-pos).length();
            m_wormTrans->setScale(OSG::Vec3f(1.0+dist, 1, 1.0+dist));
            if(dist>1.0) {
                outputEvents.push_back(Event());
                Event& newEvent = outputEvents.back();
                newEvent.type = EVENT_TREE;
                newEvent.values[0] = m_id;
                newEvent.parameter = TREE_DUPLICATE;
                m_duplicated=true;
            }
        } 
        else { //second hand, store
            m_storedGrabPos=pos;
        }
    }
    else {
        m_wormTrans->setTranslation(pos);
        updateChildrenPositions();
        m_zone->testCollisionsForWorm(this, outputEvents, false);
    }
}

void Worm::hit(std::vector<Event>& outputEvents) {

}

void Worm::removeFromZone(Zone* zone) {
    zone->removeWorm(this);
    for(unsigned int i=0;i<m_children.size();++i) {
        m_children[i]->removeFromZone(zone);
    }
}

void Worm::moveToZone(Zone* zone) {
    setZone(zone);
    zone->addWorm(this);
    for(unsigned int i=0;i<m_children.size();++i) {
        m_children[i]->moveToZone(zone);
    }
}

void Worm::attachToMusician(Musician* mus) {
    mus->getInternalLightNode()->addChild(m_worm);
    for(unsigned int i=0;i<m_children.size();++i) {
        m_children[i]->attachToMusician(mus);
    }
}

void Worm::detachFromMusician(Musician* mus) {
    mus->getInternalLightNode()->subChild(m_worm);
    for(unsigned int i=0;i<m_children.size();++i) {
        m_children[i]->detachFromMusician(mus);
    }
}

