/***************************************************************************
 *  Zone.hpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _ZONE_H_
#define _ZONE_H

#include <iostream>
#include <cstring>
#include <cmath>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <vector>
#include <map>

#include <OpenSG/OSGSimpleGeometry.h>
#include <OpenSG/OSGComponentTransform.h>
#include <OpenSG/OSGMaterialGroup.h>
#include <OpenSG/OSGSceneFileHandler.h>
#include <OpenSG/OSGSimpleStage.h>
#include <OpenSG/OSGPointLight.h>
#include <OpenSG/OSGFrameBufferObject.h>
#include <OpenSG/OSGTextureBuffer.h>
#include <OpenSG/OSGRenderBuffer.h>
#include <OpenSG/OSGTextureObjChunk.h>
#include <OpenSG/OSGTextureEnvChunk.h>
#include <OpenSG/OSGTexGenChunk.h>
#include <OpenSG/OSGTwoSidedLightingChunk.h>
#include <OpenSG/OSGSceneFileHandler.h>
#include <OpenSG/OSGImageFunctions.h>
#include <OpenSG/OSGSimpleTexturedMaterial.h>
#include <OpenSG/OSGTextureTransformChunk.h>
#include <OpenSG/OSGGradientBackground.h>
#include <OpenSG/OSGPerspectiveCamera.h>
#include <OpenSG/OSGVisitSubTree.h>

#include <libxml/parser.h>
#include <libxml/tree.h>

#include "../Event.hpp"

class Worm;
class Tunnel;
class Grid;
class Sign;
class Manipulable;

class Zone {
    public:
        Zone(const unsigned int&);
        ~Zone();

        void parse(xmlNodePtr);
    
        inline OSG::NodeRefPtr getNode(){return m_zone;}
        inline void setTranslation(const OSG::Vec3f& pos){
            m_zoneTrans->setTranslation(pos);
        }
        inline const OSG::Vec3f& getTranslation(){
            return m_zoneTrans->getTranslation();
        }
        inline void setRotation(const OSG::Quaternion& rot){
            m_zoneTrans->setRotation(rot);
        }
        inline const OSG::Quaternion& getRotation(){
            return m_zoneTrans->getRotation();
        }

        void update(std::vector<Event>&);
        inline const float& getActivity(){return m_activity;}

        void addWorm(Worm*);
        void removeWorm(Worm*);
        inline unsigned int getNbWorms(){
            return m_worms.size();
        }
        inline OSG::NodeRefPtr getWormsRoot(){
            return m_wormsRoot;
        }
        inline std::vector<Worm*>& getWorms(){return m_worms;}

        void addTunnel(Tunnel*);
        void removeTunnel(Tunnel*);
        inline unsigned int getNbTunnels(){
            return m_tunnels.size();
        }
        inline OSG::NodeRefPtr getTunnelsRoot() {
            return m_toolsRoot;
        }

        void addGrid(Grid*);
        void removeGrid(Grid*);
        inline unsigned int getNbGrids(){
            return m_grids.size();
        }

        void markWormsForSelection();
        void markTunnelsForSelection();
        void unmarkForSelection();
        void markForSelection();

        void testCollisionsForWorm(Worm*, std::vector<Event>&, 
                                    const bool& released=false);
        void testCollisionsForTunnel(Tunnel*, std::vector<Event>& events, 
                                    const bool& released=false);
        void testCollisionsForSign(Sign*, std::vector<Event>& events);

        inline void setDepth(const float& depth) {
            m_depth=depth;
            m_zoneTempTrans->setTranslation(OSG::Vec3f(0,0,-m_depth));
        }

        inline const float& getDepth(){return m_depth;}


    private:
        unsigned int m_id;

        std::vector<Worm*> m_worms;
        std::vector<Tunnel*> m_tunnels;
        std::vector<Grid*> m_grids;

        std::vector<Manipulable*> m_manipulables;

        OSG::NodeRecPtr m_zone;
        OSG::ComponentTransformRecPtr m_zoneTempTrans;
        OSG::ComponentTransformRecPtr m_zoneTrans;
        OSG::NodeRecPtr m_zoneRoot;

        OSG::NodeRecPtr m_wormsRoot;
        OSG::ComponentTransformRecPtr m_wormsRootTrans;
        OSG::NodeRecPtr m_toolsRoot;
        OSG::ComponentTransformRecPtr m_toolsRootTrans;

        //walls
        std::vector<OSG::ComponentTransformRecPtr> m_wallsTrans;

        //render to texture
        OSG::ImageRefPtr             m_fboTexImg;
        OSG::TextureObjChunkRefPtr   m_fboTex;
        OSG::FrameBufferObjectRefPtr m_fbo; 
        OSG::TextureBufferRefPtr     m_texBuf;
        OSG::RenderBufferRefPtr      m_depthBuf;
        OSG::SimpleStageRefPtr m_stage;
        OSG::NodeRefPtr m_stageNode;
        OSG::PerspectiveCameraRecPtr m_stageCam;
        OSG::NodeRecPtr m_stageCamBeaconNode;
        OSG::ComponentTransformRecPtr m_stageCamBeaconNodeTrans;
        OSG::NodeRecPtr m_stageLight;
        OSG::ComponentTransformRecPtr m_stageLightTrans;
        OSG::NodeRecPtr m_stageVisitNode;
        OSG::VisitSubTreeRecPtr m_stageVisitCore;

        float m_depth;

        float m_activity, m_activityCounter;
};

#endif

