/***************************************************************************
 *  InputManager.cpp
 *  Part of Drile
 *  2010  Tardigrade Inc.
 *  http://tardigrade-inc.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "InputManager.hpp"

#include "../Drile.hpp"

using namespace std;

InputManager::InputManager()
{
}

InputManager::~InputManager(){

    m_serialInput.stop();

}

InputManager* InputManager::getInstance()
{
    static InputManager instance;
    return &instance;
}


void InputManager::init() {
    m_midiInput.init();
    m_midiInput.start();
    m_vrpnInput.init();
    m_vrpnInput.start();

    try {
        m_miceInput.init();
        m_miceInput.start();
    } catch(std::exception& e) {
        cout<<"Error in InputManager init: "<<e.what()<<endl;
    }
//    m_oscInput.init();
//    m_oscInput.start();
    m_serialInput.init();
    m_serialInput.start();

}

void InputManager::update(std::vector<InputEvent>& events) {
    //get new events
    events.clear();
    m_midiInput.process(events);
    m_vrpnInput.process(events);
    m_miceInput.process(events);
    m_serialInput.process(events);
}




