/***************************************************************************
 *  InputManager.hpp
 *  Part of Drile
 *  2010  Tardigrade Inc.
 *  http://tardigrade-inc.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef INPUT_MANAGER_h
#define INPUT_MANAGER_h

#include <fstream>
#include <sstream>
#include <iostream>
#include <vector>
#include <map>
#include <pthread.h>
#include <string>
#include <sys/time.h>

#include "GenericInput.hpp"
#include "MidiInput.hpp"
#include "OscInput.hpp"
#include "VrpnInput.hpp"
#include "MiceInput.hpp"

#ifdef WITH_SERIAL
#include "SerialInput.hpp"
#endif

class InputManager
{    
    public:
        static InputManager* getInstance();
        ~InputManager();

        void init();
        //void process(const InputEvent&);
        void update(std::vector<InputEvent>&);

        inline int addVrpnTracker(const std::string& adress){return m_vrpnInput.addTracker(adress);}
        inline int addVrpnButton(const std::string& adress){return m_vrpnInput.addButton(adress);}

        #ifdef WITH_SERIAL
        inline int addSerialDevice(const std::string& device, const int& baud){return m_serialInput.addDevice(device, baud);}
        #endif

    private:
        InputManager();
        InputManager( const InputManager & ); 
        InputManager & operator =( const InputManager & ); 

//        static void* threadFunction(void* );
//        pthread_t m_thread;
    
        std::vector<InputEvent> m_events;

        MidiInput m_midiInput;
        VrpnInput m_vrpnInput;
        OscInput m_oscInput;
        MiceInput m_miceInput;
        #ifdef WITH_SERIAL
        SerialInput m_serialInput;
        #endif

};

#endif

