/***************************************************************************
 *  MiceInput.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "MiceInput.hpp"
#include "../Drile.hpp"


using namespace std;

MiceInput::MiceInput():GenericInput(), m_nbMice(0) {}

MiceInput::~MiceInput(){}

void MiceInput::init(){

    //MICE
    int nbDevices = ManyMouse_Init();
    if(nbDevices<=0) {
        throw std::logic_error("No mice found");
    }

    timeval curTime;
    gettimeofday(&curTime,NULL);
    long longCurTime = (long)(curTime.tv_sec-((int)((float)curTime.tv_sec/1000000.0)*1000000))*10000 + (long)((float)curTime.tv_usec/100.0);
    for(int i=0;i<nbDevices;++i) {
        std::string devName = ManyMouse_DeviceName(i);
        cout<<"Added Mouse Device "<<i<<" : "<<devName<<endl;
        std::vector<long> flamTimes(2,longCurTime);
        m_miceFlamTimes.push_back(flamTimes);
        m_nbMice++;
    }


}

void MiceInput::start(){}

void MiceInput::stop(){}


void MiceInput::process(vector<InputEvent>& events) {
    ManyMouseEvent ev; 
    while(ManyMouse_PollEvent(&ev)) {
        if(!(ev.type==MANYMOUSE_EVENT_BUTTON && ev.value>1)) {
            events.push_back(InputEvent());
            InputEvent &event = events.back();
            event.type=INPUT_MICE;
            event.device=ev.device;
            //event.values.clear();
            if(ev.type==MANYMOUSE_EVENT_BUTTON) {
                event.parameter=ev.item+MOUSE_BUTTONS;
                event.values[0] = ev.value;
            }
            else if(ev.type == MANYMOUSE_EVENT_RELMOTION){
                if(ev.item==0) {
                    event.parameter=MOUSE_X;
                }
                else {
                    event.parameter=MOUSE_Y;
                }
                //event.values.push_back(ev.value);
                event.values[0]=ev.value;
            }
            else {
                event.parameter=MOUSE_SCROLL;
                //event.values.push_back(ev.value);
                event.values[0]=ev.value;
            }
            event.id=event.device*100+event.parameter;
        }
    }
}



