/***************************************************************************
 *  MidiInput.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "MidiInput.hpp"
#include "../Drile.hpp"

using namespace std;

MidiInput::MidiInput():GenericInput() {}

MidiInput::~MidiInput() {}

void MidiInput::init() {
    //Open the ALSA midi sequencer handler
      if (snd_seq_open(&m_seqHandle, "default", SND_SEQ_OPEN_DUPLEX, 0) < 0) {
            throw std::logic_error("Error opening the Alsa midi sequencer handler");
      }
    
      //Client's name
      snd_seq_set_client_name(m_seqHandle, "Drile");
        
    if ((m_inputPort = snd_seq_create_simple_port(m_seqHandle, "Drile MIDI IN", SND_SEQ_PORT_CAP_WRITE|SND_SEQ_PORT_CAP_SUBS_WRITE, SND_SEQ_PORT_TYPE_APPLICATION)) < 0) {
        throw std::logic_error("Error while creating the midi input port");
    }    


}

void MidiInput::process(vector<InputEvent>& events)
{
    m_npfd = snd_seq_poll_descriptors_count(m_seqHandle, POLLIN);
      m_pfd = (struct pollfd *)alloca(m_npfd * sizeof(struct pollfd));
      snd_seq_poll_descriptors(m_seqHandle, m_pfd, m_npfd, POLLIN);
    if(poll(m_pfd, m_npfd, 10) > 0) {        
        snd_seq_event_t *ev;
        do {
            snd_seq_event_input(m_seqHandle, &ev);
            snd_seq_ev_set_subs(ev);  
            snd_seq_ev_set_direct(ev);    
            if(ev->type == SND_SEQ_EVENT_CONTROLLER) {
                events.push_back(InputEvent());
                InputEvent& event = events.back();
                event.type=INPUT_MIDI_CC;
                event.parameter=ev->data.control.param;
                event.id=ev->data.control.param;
                event.values[0]=ev->data.control.value;
            } else if(ev->type == SND_SEQ_EVENT_NOTEON) {
                events.push_back(InputEvent());
                InputEvent& event = events.back();
                event.type=INPUT_MIDI_NOTE;
                event.parameter=ev->data.note.note;
                event.id=ev->data.note.note;
                event.values[0]=ev->data.note.velocity;
            } else if(ev->type == SND_SEQ_EVENT_NOTEOFF) {
                events.push_back(InputEvent());
                InputEvent& event = events.back();
                event.type=INPUT_MIDI_NOTE;
                event.parameter=ev->data.note.note;
                event.id=ev->data.note.note;
                event.values[0]=0;
            }
            
            snd_seq_free_event(ev);
        } while (snd_seq_event_input_pending(m_seqHandle, 0) > 0);
    }
}

void MidiInput::start() {}

void MidiInput::stop(){
    snd_seq_close(m_seqHandle);
}


