/***************************************************************************
 *  SerialInput.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef WITH_SERIAL

#include "SerialInput.hpp"
#include <stdio.h> 
#include <unistd.h> 
#include <fcntl.h> 
#include <errno.h> 
#include <termios.h> 
#include <string.h> 
#include <sys/ioctl.h>


using namespace std;
using namespace LibSerial ;

void SerialInput::process(std::vector<InputEvent>& events) {

    for(int st=0; st<m_nbSerialStreams; ++st) {
        if(m_serialStreams[st].IsOpen()) {
            int sensorVal, sensor, val;
            //for(unsigned int i=0; i<4; ++i) {
            //while(m_serialStreams[st].rdbuf()->in_avail() > 2) {
            while(m_serialStreams[st]>>sensorVal) {
                m_serialStreams[st]>>sensorVal;
                //m_serialStreams[st]>>sensor;
                //m_serialStreams[st] >> val;
                val = sensorVal/10;
                sensor = sensorVal-val*10;
                cout<<sensor<<" "<<val<<endl;
                events.push_back(InputEvent());
                InputEvent &event = events.back();
                event.type=INPUT_SERIAL;
                event.device=st; 
                event.parameter=sensor;
                event.id=event.device*100+event.parameter;
                event.values[0]=val;
            }
        }
    }


/*
    for(unsigned int st=0; st<m_serialFDs.size(); ++st) {
        int sensor, val;
        while(portReadTwoInts(m_serialFDs[st], sensor, val)) {
            cout<<sensor<<" "<<val<<endl;
            events.push_back(InputEvent());
            InputEvent &event = events.back();
            event.type=INPUT_SERIAL;
            event.device=st; 
            event.parameter=sensor;
            event.id=event.device*100+event.parameter;
            event.values[0]=val;
        }
    }
*/

 
}



SerialInput::SerialInput():GenericInput(), m_nbSerialStreams(0){}

SerialInput::~SerialInput(){}

void SerialInput::init(){}

void SerialInput::start(){}

void SerialInput::stop(){
/*
    for(int s=0; s<m_nbSerialStreams; ++s) {
        m_serialStreams[s].Close();
    }
*/
    for(unsigned int s=0; s<m_serialFDs.size(); ++s) {
        close(m_serialFDs[s]);
    }
}

int SerialInput::addDevice(const std::string &deviceName, 
                                    const int& deviceBaud){
    //look if this tracker has already been created
    int id=-1;
    for(unsigned int i=0; i<m_devicesNames.size() && id<0; ++i) {
        if(m_devicesNames[i].compare(deviceName)==0) {
            id=i;
        }
    }

/*
    //otherwise, create it
    if(id<0) {
        int fd = portInit(deviceName, deviceBaud);
        if(fd>=0) {
            m_devicesNames.push_back(deviceName);    
            m_devicesBauds.push_back(deviceBaud);    
            m_serialFDs.push_back(fd);
            id=m_devicesNames.size()-1;        
            cout<<"added stream "<<m_devicesNames[id]<<" "<<m_devicesBauds[id]<<endl;
        }
    }
*/

    //otherwise, create it
    if(id<0) {
        m_devicesNames.push_back(deviceName);    
        m_devicesBauds.push_back(deviceBaud);    
        id=m_nbSerialStreams;
        m_serialStreams[id].Open(m_devicesNames[id].c_str(), 
                                    std::ios_base::in);
        if(m_serialStreams[id].IsOpen()) {
            cout<<"added stream "<<m_devicesNames[id]<<" "<<m_devicesBauds[id]<<endl;
            switch(m_devicesBauds[id]) {
                case 2400: {
                    m_serialStreams[id].SetBaudRate( SerialStreamBuf::BAUD_2400 ) ;
                }break;
                case 4800: {
                    m_serialStreams[id].SetBaudRate( SerialStreamBuf::BAUD_4800 ) ;
                }break;
                case 9600: {
                    m_serialStreams[id].SetBaudRate( SerialStreamBuf::BAUD_9600 ) ;
                }break;
                case 19200: {
                    m_serialStreams[id].SetBaudRate( SerialStreamBuf::BAUD_19200 ) ;
                }break;
                case 38400: {
                    m_serialStreams[id].SetBaudRate( SerialStreamBuf::BAUD_38400 ) ;
                }break;
                case 57600: {
                    m_serialStreams[id].SetBaudRate( SerialStreamBuf::BAUD_57600) ;
                }break;
                case 115200: {
                    m_serialStreams[id].SetBaudRate( SerialStreamBuf::BAUD_115200) ;
                }break;
                default: {
                    cout<<"Baud Rate not supported, using 9600"<<endl;    
                    m_serialStreams[id].SetBaudRate( SerialStreamBuf::BAUD_9600 ) ;
                }break;
            }
            m_serialStreams[id].SetCharSize( SerialStreamBuf::CHAR_SIZE_8) ;
            //m_serialStreams[id].SetParity( SerialStreamBuf::PARITY_EVEN) ;
            m_serialStreams[id].SetFlowControl(
                                    SerialStreamBuf::FLOW_CONTROL_NONE);

            usleep(5000000);
            cout<<"Done adding serial stream"<<endl;
        }
        else {
            cout<<"Problem creating the serial input "<<m_devicesNames[id]<<endl;
        }

        m_nbSerialStreams++;
    }


    return id;
}


int SerialInput::portInit(const std::string& serialPort, const int& baud) {
    struct termios toptions;
    int fd;

    //fd = open(serialport, O_RDWR | O_NOCTTY | O_NDELAY);
    fd = open(serialPort.c_str(), O_RDWR | O_NONBLOCK );

    if(fd == -1) {
        cout<<"serialport_init: Unable to open port "<<endl;;
        return -1;
    }
        
    //int iflags = TIOCM_DTR;
    //ioctl(fd, TIOCMBIS, &iflags); // turn on DTR
    //ioctl(fd, TIOCMBIC, &iflags); // turn off DTR

    if (tcgetattr(fd, &toptions) < 0) {
        cout<<"serialport_init: Couldn't get term attributes"<<endl;
        return -1;
    }
    speed_t brate = baud; // let you override switch below if needed
    switch(baud) {
        case 4800: brate=B4800; break;
        case 9600: brate=B9600; break;
        #ifdef B14400
        case 14400: brate=B14400; break;
        #endif
        case 19200: brate=B19200; break;
        #ifdef B28800
        case 28800: brate=B28800; break;
        #endif
        case 38400: brate=B38400; break;
        case 57600: brate=B57600; break;
        case 115200: brate=B115200; break;
    }
    cfsetispeed(&toptions, brate);
    cfsetospeed(&toptions, brate);

    // 8N1
    toptions.c_cflag &= ~PARENB;
    toptions.c_cflag &= ~CSTOPB;
    toptions.c_cflag &= ~CSIZE;
    toptions.c_cflag |= CS8;
    // no flow control
    toptions.c_cflag &= ~CRTSCTS;

    //toptions.c_cflag &= ~HUPCL; // disable hang-up-on-close to avoid reset

    toptions.c_cflag |= CREAD | CLOCAL; // turn on READ & ignore ctrl lines
    toptions.c_iflag &= ~(IXON | IXOFF | IXANY); // turn off s/w flow ctrl

    toptions.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // make raw
    toptions.c_oflag &= ~OPOST; // make raw

    // see: http://unixwiz.net/techtips/termios-vmin-vtime.html
    toptions.c_cc[VMIN] = 0;
    toptions.c_cc[VTIME] = 0;
    //toptions.c_cc[VTIME] = 20;

    tcsetattr(fd, TCSANOW, &toptions);
    if( tcsetattr(fd, TCSAFLUSH, &toptions) < 0) {
        cout<<"init_serialport: Couldn't set term attributes"<<endl;
        return -1;
    }

    return fd;
}


bool SerialInput::portReadTwoInts(const int& fd, int& int1, int& int2) {

/*
    char b[1]; // read expects an array, so we give it a 1-byte array
    int i=0;
    do {
        int n = read(fd, b, 1); // read a char at a time
        if( n==-1) return -1; // couldn't read
        if( n==0 ) {
            usleep( 1 * 1000 ); // wait 1 msec try again
            timeout--;
            continue;
        }
        buf[i] = b[0];
        i++;
    } while( b[0] != ' && i < buf_max && timeout>0 );

    buf[i] = 0; // null terminate the string
    return 0;
*/

/*
    char buf[6];
    int n = read(fd, buf, 6);
    cout<<"read "<<n<<endl;
    if(n<=0) {
        //cout<<"could not read from serial"<<endl;
        return false;
    }
    else {
        while(
        n
    }
    memcpy(&int1, buf, sizeof(int));
    memcpy(&int2, buf+sizeof(int)+sizeof(char), sizeof(int));
*/  
  return true;
}

#endif
