/***************************************************************************
 *  VrpnInput.cpp
 *  Part of Drile
 *  2010-  Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "VrpnInput.hpp"
#include "../Drile.hpp"

using namespace std;

void VrpnInput::handleTracker(void *userdata, vrpn_TRACKERCB t) {
    int id = ((VrpnDeviceData*)userdata)->id;
    VrpnInput *vrpnInput = ((VrpnDeviceData*)userdata)->vrpnInput;

    vrpnInput->m_vrpnEvents.push_back(InputEvent());
    InputEvent& event = vrpnInput->m_vrpnEvents.back();
    event.id=id*100+t.sensor;
    event.type=INPUT_VRPN_TRACKER;
    event.device=t.sensor;
    event.values[0]=t.pos[0];
    event.values[1]=t.pos[1];
    event.values[2]=t.pos[2];
    event.values[3]=t.quat[0];
    event.values[4]=t.quat[1];
    event.values[5]=t.quat[2];
    event.values[6]=t.quat[3];
}

void VrpnInput::handleButton(void *userdata, vrpn_BUTTONCB b) {
    int id = ((VrpnDeviceData*)userdata)->id;
    VrpnInput *vrpnInput = ((VrpnDeviceData*)userdata)->vrpnInput;

    vrpnInput->m_vrpnEvents.push_back(InputEvent());
    InputEvent& event = vrpnInput->m_vrpnEvents.back();
    event.id=id*100+int(b.button);
    event.type=INPUT_VRPN_BUTTON;
    event.device=b.button;
    event.parameter=b.button;
    event.values[0]=b.state;
}

VrpnInput::VrpnInput():GenericInput() {}

VrpnInput::~VrpnInput() {}

void VrpnInput::init() {
    for(int i=0; i<MAX_VRPN_TRACKERS; ++i) {
        m_trackersID[i]=i;
        m_trackersData[i].id=i;
        m_trackersData[i].vrpnInput=this;
    }
    for(int i=0; i<vrpn_BUTTON_MAX_BUTTONS; ++i) {
        m_buttonsID[i]=i;
        m_buttonsData[i].id=i;
        m_buttonsData[i].vrpnInput=this;
    }
}

void VrpnInput::start() {}

void VrpnInput::stop() {}

int VrpnInput::addTracker(const std::string &trackerName) {
    //look if this tracker has already been created
    int id=-1;
    for(unsigned int i=0; i<m_trackersNames.size() && id<0; ++i) {
        if(m_trackersNames[i].compare(trackerName)==0) {
            id=i;
        }
    }

    //otherwise, create it
    if(id<0) {
        vrpn_Tracker_Remote *newTracker = 
            new vrpn_Tracker_Remote(trackerName.c_str());
        m_trackers.push_back(newTracker);
        m_trackersNames.push_back(trackerName);
        newTracker->register_change_handler(
                (void*)(&m_trackersData[m_trackers.size()-1]), handleTracker);
        id=m_trackersID[m_trackers.size()-1];
    }

    return id;
}

int VrpnInput::addButton(const std::string &buttonName) {
    //look if this button has already been created
    int id=-1;
    for(unsigned int i=0; i<m_buttonsNames.size() && id<0; ++i) {
        if(m_buttonsNames[i].compare(buttonName)==0) {
            id=i;
        }
    }

    //otherwise, create it
    if(id<0) {
        vrpn_Button_Remote *newButton = 
            new vrpn_Button_Remote(buttonName.c_str());
        m_buttons.push_back(newButton);
        m_buttonsNames.push_back(buttonName);
        newButton->register_change_handler(
                (void*)(&m_buttonsData[m_buttons.size()-1]), handleButton);
        id=m_buttonsID[m_buttons.size()-1];
    }

    return id;
}


void VrpnInput::process(vector<InputEvent>& events) {
    m_vrpnEvents.clear();
    for(unsigned int i=0; i<m_trackers.size(); ++i) {
        m_trackers[i]->mainloop();
    }
    for(unsigned int i=0; i<m_buttons.size(); ++i) {
        m_buttons[i]->mainloop();
    }
    events.insert(events.end(), m_vrpnEvents.begin(), m_vrpnEvents.end());
}

