/***************************************************************************
 *  VrpnInput.hpp
 *  Part of Drile
 *  2010- Florent Berthaut
 *  hitmuri.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef VRPN_INPUT_H
#define VRPN_INPUT_H
#include <iostream>
#include <math.h>
#include <vrpn_Tracker.h>
#include <vrpn_Button.h>
#include <vrpn_Analog.h>

#include "GenericInput.hpp"

enum VRPN_DEVICE{VRPN_TRACKER, VRPN_BUTTON, VRPN_ANALOG};

class VrpnInput;

struct VrpnDeviceData {
    VrpnInput* vrpnInput;
    int id;    
};

class VrpnInput: public GenericInput {
    public:
        VrpnInput();
        ~VrpnInput();

        void init();
        void start();
        void stop();
        void process(std::vector<InputEvent>&);

        int addTracker(const std::string&);
        int addButton(const std::string&);

        void receiveMessages();

    private:    

        static const int MAX_VRPN_TRACKERS=32;

        std::vector<vrpn_Tracker_Remote*> m_trackers;
        std::vector<std::string> m_trackersNames;
        int m_trackersID[MAX_VRPN_TRACKERS];
        VrpnDeviceData m_trackersData[MAX_VRPN_TRACKERS];

        std::vector<InputEvent> m_vrpnEvents;

        static void handleTracker(void *userdata, vrpn_TRACKERCB t);

        std::vector<vrpn_Button_Remote*> m_buttons;
        std::vector<std::string> m_buttonsNames;
        int m_buttonsID[vrpn_BUTTON_MAX_BUTTONS];
        VrpnDeviceData m_buttonsData[vrpn_BUTTON_MAX_BUTTONS];

        static void handleButton(void *userdata, vrpn_BUTTONCB b);
};
#endif 

