/***************************************************************************
 *            ElementGroup.cpp
 *
 *  2006  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "ElementGroup.hpp"

#include "../env/EnvManager.hpp"

using namespace std;


ElementGroup::ElementGroup(int x, int y, int w,int h,const char* name):Fl_Group(x,y,w,h,name)
{
	m_name = new Fl_Button(x, y, w-80, h, name);
	m_name->box(FL_FLAT_BOX);
	m_name->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
	m_name->callback(statName,this);
	add(m_name);

	m_add = new Fl_Button(x+w-80, y, 20, h,"+");
	m_add->callback(statAdd,this);
	m_add->box(FL_FLAT_BOX);
	add(m_add);
	m_remove = new Fl_Button(x+w-60, y, 20, h,"-");
	m_remove->callback(statRemove,this);
	m_remove->box(FL_FLAT_BOX);
	add(m_remove);
	m_duplicate = new Fl_Button(x+w-40, y, 20, h, "D");
	m_duplicate->callback(statDuplicate,this);
	m_duplicate->box(FL_FLAT_BOX);
	add(m_duplicate);

	m_nameInput = new Fl_Input(x, y, w-80,h,"");
	m_nameInput->callback(statInput,this);
	m_nameInput->hide();
	add(m_nameInput);

	end();
}

ElementGroup::~ElementGroup()
{
}

void ElementGroup::setType(ELEMENT_TYPE type)
{
	m_type=type;
	switch(m_type) {
		case ELEMENT_INPUTS:{
			m_name->activate();
			m_nameInput->hide();
			m_add->hide();
			m_remove->hide();
			m_duplicate->hide();
		}break;
		case ELEMENT_CAMERAS:
		case ELEMENT_PIIVERTS:
		case ELEMENT_ROOMS:
		case ELEMENT_WORMS:
		case ELEMENT_TUNNELS:{
			m_name->activate();
			m_nameInput->hide();
			m_add->show();
			m_remove->hide();
			m_duplicate->hide();
		}break;
		case ELEMENT_CAMERA:
		case ELEMENT_PIIVERT:
		case ELEMENT_TUNNEL:{
			m_name->activate();
			m_nameInput->hide();
			m_add->hide();
			m_remove->show();
			m_duplicate->show();
		}break;
		case ELEMENT_WORM:{
			m_name->activate();
			m_nameInput->hide();
			m_add->show();
			m_remove->show();
			m_duplicate->show();
		}break;
        default:break;
	}

	this->redraw();
}

void ElementGroup::cbAdd(Fl_Widget*)
{
	m_action=ELEMENT_ADD;
	do_callback();
}

void ElementGroup::cbRemove(Fl_Widget*)
{
	m_action=ELEMENT_REMOVE;
	do_callback();
}

void ElementGroup::cbDuplicate(Fl_Widget*)
{
	m_action=ELEMENT_DUPLICATE;
	do_callback();
}

void ElementGroup::cbName(Fl_Widget*)
{
	m_action=ELEMENT_NAME;
	do_callback();
}

void ElementGroup::cbInput(Fl_Widget*)
{

}

