/***************************************************************************
 *  ElementGroup.h
 *  Part of Drile
 *  2006  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 
#ifndef _ELEMENT_GROUP_H
#define _ELEMENT_GROUP_H

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Menu_Item.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Text_Display.H>
#include <FL/Fl_Text_Buffer.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Output.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Toggle_Button.H>
#include <FL/Fl_Slider.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Counter.H>
#include <FL/x.H>
#include <math.h>
#include <iostream>
#include <string>
#include <vector>
#include <map>

#include <cmath>

enum ELEMENT_ACTION{ELEMENT_NAME, ELEMENT_ADD, ELEMENT_DUPLICATE, ELEMENT_REMOVE};
enum ELEMENT_TYPE {ELEMENT_CAMERAS, ELEMENT_CAMERA, ELEMENT_INPUTS, ELEMENT_PIIVERTS, ELEMENT_PIIVERT, ELEMENT_ROOMS, ELEMENT_ROOM, ELEMENT_WORMS, ELEMENT_WORM, ELEMENT_TUNNELS, ELEMENT_TUNNEL};

class ElementGroup : public Fl_Group
{
  public:
	ElementGroup(int , int , int, int, const char*);
	~ElementGroup();

	
	static void statAdd(Fl_Widget* w,void* f){ElementGroup *tmpf = static_cast<ElementGroup *>(f);tmpf->cbAdd(w);}	
	void cbAdd(Fl_Widget*);
	static void statRemove(Fl_Widget* w,void* f){ElementGroup *tmpf = static_cast<ElementGroup *>(f);tmpf->cbRemove(w);}	
	void cbRemove(Fl_Widget*);
	static void statDuplicate(Fl_Widget* w,void* f){ElementGroup *tmpf = static_cast<ElementGroup *>(f);tmpf->cbDuplicate(w);}	
	void cbDuplicate(Fl_Widget*);
	static void statName(Fl_Widget* w,void* f){ElementGroup *tmpf = static_cast<ElementGroup *>(f);tmpf->cbName(w);}	
	void cbName(Fl_Widget*);
	static void statInput(Fl_Widget* w,void* f){ElementGroup *tmpf = static_cast<ElementGroup *>(f);tmpf->cbInput(w);}	
	void cbInput(Fl_Widget*);

	inline void setName(std::string name){m_name->copy_label(name.c_str());}
	inline void setDecay(int decay){m_name->position(x()+decay, m_name->y()); m_nameInput->position(x()+decay, m_name->y());}
	void setType(ELEMENT_TYPE type);
	inline void setID(int id){m_id=id;}
	inline ELEMENT_TYPE getType(){return m_type;}
	inline int getID(){return m_id;}

	inline ELEMENT_ACTION getAction(){return m_action;}
  private:

	Fl_Button* m_add;
	Fl_Button* m_remove;
	Fl_Button* m_duplicate;
	Fl_Button* m_name;
	Fl_Input* m_nameInput;

	ELEMENT_TYPE m_type;
	int m_id;
	ELEMENT_ACTION m_action;
	
};

#endif 
