/***************************************************************************
 *            MainWindow.cpp
 *
 *  part of Drile
 *  2010  Tardigrade inc.
 *  http://tardigrade-inc.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <iostream>
#include <math.h>
#include <FL/x.H>

#include "MainWindow.hpp"
#include "../env/EnvManager.hpp"
#include "../Drile.hpp"


using namespace std;


MainWindow::MainWindow(int w,int h,const char* titre): Fl_Double_Window(w,h,titre), m_fileSaved(true)
{
	//default theme
	Fl::set_color(FL_BACKGROUND_COLOR,0,0,0);
	Fl::set_color(FL_BACKGROUND2_COLOR,80,80,80);
	Fl::set_color(FL_FOREGROUND_COLOR,255,255,255);
	Fl::set_color(FL_INACTIVE_COLOR,40,40,40);
	Fl::set_color(FL_SELECTION_COLOR,0,0,255);

	//resizable
	//resizable(this);
	
	//borders off
	//border(false);
	
	//set window icon
	/*
	fl_open_display();
	Pixmap p, mask;
	XpmCreatePixmapFromData(fl_display, DefaultRootWindow(fl_display), vjpirate_xpm, &p, &mask, NULL);
	icon((char *)p);
	*/

	//callback
	this->callback(statWindow,this);

	m_windowBar = new Fl_Group(0,0,w,MENU_HEIGHT,"");
	m_windowBar->color(FL_INACTIVE_COLOR);
	m_windowBar->box(FL_FLAT_BOX);
	m_windowBar->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
	m_windowBar->labelcolor(FL_BACKGROUND2_COLOR);
	m_windowBar->labelsize(20);
	this->add(*m_windowBar);
	m_windowBar->begin();

	//menu
	Fl_Menu_Item menuitems[] = 
	{
		{ "DRILE",0, 0, 0, FL_SUBMENU },
			{ "&Open File",FL_CTRL + 'o',(Fl_Callback *)statOpen,this },
			{ "&Save File",FL_CTRL + 's',(Fl_Callback *)statSave,this},
			{ "S&ave File as",FL_CTRL + FL_SHIFT + 's',(Fl_Callback *)statSaveAs,this},
			{ "&Quit",FL_CTRL + 'q', (Fl_Callback *)statQuit,this},
			{0},
		{0}
	};
	m_menu = new Fl_Menu_Bar(0, 0, 50, MENU_HEIGHT);
	m_menu->copy(menuitems);
	m_menu->box(FL_NO_BOX);
	m_menu->textcolor(FL_BACKGROUND2_COLOR);
	m_menu->selection_color(FL_SELECTION_COLOR);
	m_menu->color(FL_INACTIVE_COLOR);
	m_menu->clear_visible_focus();
	
	m_windowBar->end();
	
	/*
	m_applicationsGroup = new Fl_Group(WIDGETS_SEP, MENU_HEIGHT, w, APPLICATIONS_HEIGHT, "");
	m_applicationsGroup->box(FL_BORDER_BOX);
	m_applicationsGroup->color(FL_BACKGROUND2_COLOR);
	m_applicationsGroup->end();

	end();


	m_applicationsGroup->begin();
		m_audioApplicationGroup = new Fl_Group(0, m_applicationsGroup->y(), w/2, APPLICATIONS_HEIGHT/2, "AUDIO");
		m_audioApplicationGroup->box(FL_BORDER_BOX);
		m_audioApplicationGroup->color(FL_BACKGROUND2_COLOR);
		m_audioApplicationGroup->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT|FL_ALIGN_TOP);
		m_audioApplicationGroup->end();
		m_envApplicationGroup = new Fl_Group(0, m_audioApplicationGroup->y()+m_audioApplicationGroup->h(), w/2, APPLICATIONS_HEIGHT/2, "ENVIRONMENT");
		m_envApplicationGroup->box(FL_BORDER_BOX);
		m_envApplicationGroup->color(FL_BACKGROUND2_COLOR);
		m_envApplicationGroup->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT|FL_ALIGN_TOP);
		m_envApplicationGroup->end();
		m_displayApplicationsGroup = new Fl_Group(w/2, m_applicationsGroup->y(), w/2, APPLICATIONS_HEIGHT, "DISPLAYS");
		m_displayApplicationsGroup->box(FL_BORDER_BOX);
		m_displayApplicationsGroup->color(FL_BACKGROUND2_COLOR);
		m_displayApplicationsGroup->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT|FL_ALIGN_TOP);
		m_displayApplicationsGroup->end();
	m_applicationsGroup->end();
	

	m_audioApplicationGroup->begin();
		m_audioMachine = new Fl_Choice(m_audioApplicationGroup->x()+WIDGETS_SEP, m_audioApplicationGroup->y()+WIDGETS_SEP*4, 80, MENU_HEIGHT, "");
		m_audioMachine->add("Local");
		m_audioMachine->add("Distant");
		m_audioMachine->value(0);
		m_audioMachine->box(FL_FLAT_BOX);
		m_audioMachine->down_box(FL_FLAT_BOX);
		m_audioAdress =  new Fl_Input(m_audioMachine->x()+80+WIDGETS_SEP, m_audioMachine->y(), 100, MENU_HEIGHT, "Adress");
		m_audioAdress->box(FL_FLAT_BOX);
		m_audioAdress->color(FL_INACTIVE_COLOR);
		m_audioAdress->align(FL_ALIGN_BOTTOM|FL_ALIGN_LEFT);
		m_audioAdress->labelsize(9);
		m_audioAdress->hide();
		m_audioPort =  new Fl_Value_Input(m_audioAdress->x()+m_audioAdress->w()+WIDGETS_SEP, m_audioMachine->y(), 50, MENU_HEIGHT, "Port");
		m_audioPort->box(FL_FLAT_BOX);
		m_audioPort->step(1);
		m_audioPort->bounds(1,60000);
		m_audioPort->color(FL_INACTIVE_COLOR);
		m_audioPort->align(FL_ALIGN_BOTTOM|FL_ALIGN_LEFT);
		m_audioPort->labelsize(9);
		m_audioPort->hide();
		m_audioInit = new Fl_Button(m_audioApplicationGroup->x()+m_audioApplicationGroup->w()/2 - 30, m_audioApplicationGroup->y()+m_audioApplicationGroup->h()-MENU_HEIGHT-WIDGETS_SEP, 60, MENU_HEIGHT, "START");
		m_audioInit->box(FL_FLAT_BOX);
		m_audioInit->type(FL_TOGGLE_BUTTON);
		m_audioInit->selection_color(FL_SELECTION_COLOR);
		m_audioInit->clear_visible_focus();
	m_audioApplicationGroup->end();

	m_envApplicationGroup->begin();
		m_envMachine = new Fl_Choice(m_envApplicationGroup->x()+WIDGETS_SEP, m_envApplicationGroup->y()+WIDGETS_SEP*4, 80, MENU_HEIGHT, "");
		m_envMachine->add("Local");
		m_envMachine->add("Distant");
		m_envMachine->value(0);
		m_envMachine->box(FL_FLAT_BOX);
		m_envMachine->down_box(FL_FLAT_BOX);
		m_envAdress =  new Fl_Input(m_envMachine->x()+80+WIDGETS_SEP, m_envMachine->y(), 100, MENU_HEIGHT, "Adress");
		m_envAdress->box(FL_FLAT_BOX);
		m_envAdress->color(FL_INACTIVE_COLOR);
		m_envAdress->align(FL_ALIGN_BOTTOM|FL_ALIGN_LEFT);
		m_envAdress->labelsize(9);
		m_envAdress->hide();
		m_envPort =  new Fl_Value_Input(m_envAdress->x()+m_envAdress->w()+WIDGETS_SEP, m_envMachine->y(), 50, MENU_HEIGHT, "Port");
		m_envPort->box(FL_FLAT_BOX);
		m_envPort->step(1);
		m_envPort->bounds(1,60000);
		m_envPort->color(FL_INACTIVE_COLOR);
		m_envPort->align(FL_ALIGN_BOTTOM|FL_ALIGN_LEFT);
		m_envPort->labelsize(9);
		m_envPort->hide();
		m_envInit = new Fl_Button(m_envApplicationGroup->x()+m_envApplicationGroup->w()/2 - 30, m_envApplicationGroup->y()+m_envApplicationGroup->h()-MENU_HEIGHT-WIDGETS_SEP, 60, MENU_HEIGHT, "START");
		m_envInit->box(FL_FLAT_BOX);
		m_envInit->type(FL_TOGGLE_BUTTON);
		m_envInit->selection_color(FL_SELECTION_COLOR);
		m_envInit->clear_visible_focus();
	m_envApplicationGroup->end();


	m_displayApplicationsGroup->begin();
		m_displayApplicationsList = new Fl_Scroll(m_displayApplicationsGroup->x(), m_displayApplicationsGroup->y()+MENU_HEIGHT, m_displayApplicationsGroup->w(), m_displayApplicationsGroup->h()-MENU_HEIGHT,"");
		m_displayApplicationsList->type(Fl_Scroll::VERTICAL_ALWAYS);
		m_displayApplicationsList->box(FL_BORDER_BOX);
		m_displayApplicationsList->color(FL_BACKGROUND2_COLOR);
	m_displayApplicationsGroup->end();

	//ELEMENTS
	m_elementsList = new Fl_Scroll(WIDGETS_SEP, ELEMENTS_Y, ELEMENTS_W, ELEMENTS_H, "");
	m_elementsList->type(Fl_Scroll::VERTICAL_ALWAYS);
	add(m_elementsList);

	//ELEMENT PARAMETERS
	m_parametersGroup = new Fl_Group(2*WIDGETS_SEP+ELEMENTS_W, ELEMENTS_Y, ELEMENTS_H, w-ELEMENTS_W, "Cameras Group");
	m_parametersGroup->end();
	add(m_parametersGroup);

	*/

	//Render Widget
	m_renderWidget = new RenderWidget(0,0,1,1,"");

	//set the idle function
	Fl::add_idle(idle, this);
}

void MainWindow::idle(void* pnt)
{
	Drile* drile = Drile::getInstance();
	drile->update();
	MainWindow *win = (MainWindow*)pnt;
	win->m_renderWidget->redraw();
	win->redraw();

}

MainWindow::~MainWindow()
{
}

void MainWindow::cbWindow(Fl_Widget*)
{
	if (Fl::event()==FL_SHORTCUT) {
		 if(Fl::event_key()==FL_Escape) {
			return; // ignore Escape
		}
	}
	
	cbQuit(this);
}

void MainWindow::cbOpen(Fl_Widget* w)
{
/*
	TardiFileBrowser *tFB = new TardiFileBrowser();
	std::string currentFileStr="";
	if(m_vj->getFileName().compare("")!=0) {
		char* currentFile= new char[FL_PATH_MAX];
		fl_filename_absolute(currentFile,m_vj->getFileName().c_str());
		currentFileStr = currentFile;
		delete [] currentFile;
	}
	tFB->open("Choose a .vjp file","{*.vjp}",currentFileStr);
	while(tFB->shown())
	{
		Fl::wait();
	}
	char* newFile=tFB->getResult();
	if(newFile)
	{
		m_vj->openFile(newFile);
		delete [] newFile;
	}
	delete tFB;
	resetSettingsTab();
*/
}

void MainWindow::cbSave(Fl_Widget* w)
{
/*
	//if no previous file
	if(m_vj->getFileName().compare("")==0) {
		cbSaveAs(this);
	}
	else {
		char* buffer = new char[m_vj->getFileName().size()+1];
		strcpy(buffer,m_vj->getFileName().c_str());
		m_vj->saveToFile(buffer);
		delete [] buffer;
		m_fileSaved=true;
	}
*/	
}

void MainWindow::cbSaveAs(Fl_Widget* w)
{
/*
	//open a dialog
	TardiFileBrowser *tFB = new TardiFileBrowser();
	tFB->open("Save to a .vjp file","{*.vjp}","");
	while(tFB->shown())
	{
		Fl::wait();
	}
	char* newFile=tFB->getResult();
	if(newFile) {
		std::string strNewFile=newFile;
		if(!fl_filename_match(strNewFile.c_str(),"*.vjp")) {
			strNewFile+=".vjp";
			newFile=strdup(strNewFile.c_str());
		}
		m_vj->saveToFile(newFile);
		std::string title = "VjPirate: ";
		title+=m_vj->getShortFileName();
		label(title.c_str());
		delete [] newFile;
		m_fileSaved=true;
	}
	delete tFB;
*/
}

void MainWindow::cbQuit(Fl_Widget* w)
{
	if(!m_fileSaved) {
		switch(fl_choice("File not saved ! Quit without saving ?", "Quit without saving","Save then Quit" ,"Don't quit")) {
			case 0:
			break;
			case 1: {
				cbSave(this);
				if(!m_fileSaved)
					return;
			}
			break;
			default: return;
				
		}
	}	

	exit(0);
}

void MainWindow::setTitle(std::string t)
{
	label(t.c_str());	
}

void MainWindow::resetElementsList()
{
/*
	EnvManager *env = EnvManager::getInstance();


	//get nbelements 
	int nbElements=0;

	nbElements+=1;//Cameras
	nbElements+=env->getNbCameras();

	nbElements+=2;//Inputs+Piiverts
	nbElements+=env->getNbPiiverts();

	nbElements+=1;//Rooms
	nbElements+=env->getNbRooms();
	for(unsigned int i=0;i<env->getNbRooms();++i) {
		Room* room = env->getRoom(i);
		nbElements++;
		nbElements+=room->getNbWorms();
		nbElements++;
		nbElements+=room->getNbTunnels();
	}


	//adapt number of ElementGroups to element number
	if(nbElements>m_elements.size()) {
		int nbElementsToAdd = nbElements-m_elements.size();
		for(unsigned int i=0;i<nbElementsToAdd;++i) {
			ElementGroup* newEl = new ElementGroup(m_elementsList->x(), m_elementsList->y()+m_elements.size()*20, m_elementsList->w(), 20, "");
			newEl->callback(statElements,this);
			m_elements.push_back(newEl);
			m_elementsList->add(newEl);
		}
	}

	//set elementgroups
	int elementCounter=0;
	//Cameras
	m_elements[elementCounter]->setName("Cameras/Inputs");
	m_elements[elementCounter]->setType(ELEMENT_CAMERAS);
	elementCounter++;
	for(unsigned int i=0; i<env->getNbCameras(); ++i && ++elementCounter) {
		m_elements[elementCounter]->setName(env->getCamera(i)->getName());
		m_elements[elementCounter]->setType(ELEMENT_CAMERA);
		m_elements[elementCounter]->setID(i);
		m_elements[elementCounter]->setDecay(5);
		++elementCounter;
		//Inputs
		m_elements[elementCounter]->setName("Piiverts");
		m_elements[elementCounter]->setType(ELEMENT_PIIVERTS);
		m_elements[elementCounter]->setDecay(10);
		elementCounter++;
		for(unsigned int j=0; j<env->getNbPiiverts(); ++j && ++elementCounter) {
			m_elements[elementCounter]->setName(env->getPiivert(i)->getName());
			m_elements[elementCounter]->setType(ELEMENT_PIIVERT);
			m_elements[elementCounter]->setID(i);
			m_elements[elementCounter]->setDecay(15);
		}
	}

	//Rooms
	m_elements[elementCounter]->setName("Rooms");
	m_elements[elementCounter]->setType(ELEMENT_ROOMS);
	elementCounter++;
	for(unsigned int r=0; r<env->getNbRooms(); ++r) {
		Room* room = env->getRoom(r);

		m_elements[elementCounter]->setName(room->getName());
		m_elements[elementCounter]->setType(ELEMENT_ROOM);
		m_elements[elementCounter]->setID(r);
		m_elements[elementCounter]->setDecay(5);
		elementCounter++;
		m_elements[elementCounter]->setName("Worms");
		m_elements[elementCounter]->setType(ELEMENT_WORMS);
		m_elements[elementCounter]->setID(r);
		m_elements[elementCounter]->setDecay(10);
		elementCounter++;

		for(unsigned int w=0;w<room->getNbWorms();++w, elementCounter++) {
			Worm* worm = env->getWorm(room->getWormID(w));
			m_elements[elementCounter]->setName(worm->getName());
			m_elements[elementCounter]->setType(ELEMENT_WORM);
			m_elements[elementCounter]->setID(worm->getID());
			m_elements[elementCounter]->setDecay(15);
		}

		m_elements[elementCounter]->setName("Tunnels");
		m_elements[elementCounter]->setType(ELEMENT_TUNNELS);
		m_elements[elementCounter]->setID(r);
		m_elements[elementCounter]->setDecay(10);
		elementCounter++;

		for(unsigned int t=0;t<room->getNbTunnels();++t, ++elementCounter) {
			Tunnel* tunnel = env->getTunnel(room->getTunnelID(t));
			m_elements[elementCounter]->setName("Tunnel");
			m_elements[elementCounter]->setType(ELEMENT_TUNNEL);
			m_elements[elementCounter]->setID(tunnel->getID());
			m_elements[elementCounter]->setDecay(15);
		}
	}

	m_elementsList->redraw();
	this->redraw();
	*/
}

void MainWindow::cbElements(Fl_Widget* w)
{

	ElementGroup *element = dynamic_cast<ElementGroup*>(w);
	cout<<"element type "<<element->getType()<<" ID "<<element->getID()<<" action "<<element->getAction()<<endl;
	switch(element->getType()) {
		case ELEMENT_CAMERAS:{
			switch(element->getAction()) {
				case ELEMENT_NAME: {
					//display global camera position ...
				}break;
				case ELEMENT_ADD: {
					//drile->addCamera();
				}break;
                default:break;
			}
		}break;
		case ELEMENT_CAMERA:{
			switch(element->getAction()) {
				case ELEMENT_NAME: {
					//display camera parameters
				}break;
				case ELEMENT_DUPLICATE: {
					//drile->duplicateCamera(element->getID());
				}break;
				case ELEMENT_REMOVE: {
					//drile->removeCamera(element->getID());
				}break;
                default:break;
			}
		}break;
		case ELEMENT_PIIVERTS:{
			switch(element->getAction()) {
				case ELEMENT_ADD: {
					//drile->addPiivert();
				}break;
                default:break;
			}
		}break;
		case ELEMENT_PIIVERT:{
			switch(element->getAction()) {
				case ELEMENT_NAME: {
					//display piivert
				}break;
				case ELEMENT_DUPLICATE: {
					//drile->duplicatePiivert(element->getID());
				}break;
				case ELEMENT_REMOVE: {
					//drile->removePiivert(element->getID());
				}break;
                default:break;
			}
		}break;
		case ELEMENT_ROOMS:{
			switch(element->getAction()) {
				case ELEMENT_ADD: {
					//drile->addRoom();
				}break;
                default:break;
			}
		}break;
		case ELEMENT_ROOM:{
			switch(element->getAction()) {
				case ELEMENT_NAME: {
				}break;
				case ELEMENT_DUPLICATE: {
					//drile->duplicateRoom(element->getID());
				}break;
				case ELEMENT_REMOVE: {
					//drile->removeRoom(element->getID());
				}break;
                default:break;
			}
		}break;
		case ELEMENT_WORMS:{
			switch(element->getAction()) {
				case ELEMENT_ADD: {
					//drile->addWormToRoom(element->getID());
				}break;
                default:break;
			}
		}break;
		case ELEMENT_TUNNELS:{
			switch(element->getAction()) {
				case ELEMENT_ADD: {
					//drile->addTunnelToRoom(element->getID());
				}break;
                default:break;
			}
		}break;
		case ELEMENT_WORM:{
			switch(element->getAction()) {
				case ELEMENT_NAME: {
				}break;
				case ELEMENT_ADD: {
					//drile->addWormToWorm(element->getID());
				}break;
				case ELEMENT_DUPLICATE: {
					//drile->duplicateWorm(element->getID());
				}break;
				case ELEMENT_REMOVE: {
					//drile->removeWorm(element->getID());
				}break;
                default:break;
			}
		}break;
		case ELEMENT_TUNNEL:{
			switch(element->getAction()) {
				case ELEMENT_NAME: {
				}break;
				case ELEMENT_DUPLICATE: {
					//drile->duplicateTunnel(element->getID());
				}break;
				case ELEMENT_REMOVE: {
					//drile->removeTunnel(element->getID());
				}break;
                default:break;
			}
		}break;
        default:break;
	}

}


