/***************************************************************************
 *            MainWindow.h
 *
 *  2006  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 
#ifndef _MAINWINDOW_H
#define _MAINWINDOW_H

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Gl_Window.H>
#include <FL/Fl_Tabs.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Menu_Item.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Text_Display.H>
#include <FL/Fl_Text_Buffer.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Output.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Menu_Button.H>
#include <FL/Fl_Toggle_Button.H>
#include <FL/Fl_Value_Slider.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Value_Input.H>
#include <FL/Fl_Float_Input.H>
#include <FL/Fl_Counter.H>
#include <FL/Fl_Dial.H>
#include <FL/Fl_Scroll.H>
#include <FL/Fl_Shared_Image.H>
#include <FL/Fl_Hold_Browser.H>
#include <iostream>
#include <string>
#include <vector>
#include <algorithm>
#include <locale>
#include <ctype.h>

#include "RenderWidget.hpp"
#include "ElementGroup.hpp"
#include "TardiFileBrowser.hpp"


enum TABS{PLAY_TAB, SETTINGS_TAB,NB_TABS};

class MainWindow : public Fl_Double_Window 
{
  public:
		MainWindow(int ,int ,const char*);
		~MainWindow();
		void setTitle(std::string);
		void reset();

		static void idle(void* pnt);
		
		//window callback
		static void statWindow(Fl_Widget* w,void* f){MainWindow *tmpf = static_cast<MainWindow *>(f);tmpf->cbWindow(w);}	
		void cbWindow(Fl_Widget*);

		//menu callback
		static void statOpen(Fl_Widget* w,void* f){MainWindow *tmpf = static_cast<MainWindow *>(f);tmpf->cbOpen(w);}	
		void cbOpen(Fl_Widget*);
		static void statSave(Fl_Widget* w,void* f){MainWindow *tmpf = static_cast<MainWindow *>(f);tmpf->cbSave(w);}	
		void cbSave(Fl_Widget*);
		static void statSaveAs(Fl_Widget* w,void* f){MainWindow *tmpf = static_cast<MainWindow *>(f);tmpf->cbSaveAs(w);}	
		void cbSaveAs(Fl_Widget*);
		static void statQuit(Fl_Widget* w,void* f){MainWindow *tmpf = static_cast<MainWindow *>(f);tmpf->cbQuit(w);}	
		void cbQuit(Fl_Widget*);

		static void statElements(Fl_Widget* w,void* f){MainWindow *tmpf = static_cast<MainWindow *>(f);tmpf->cbElements(w);}	
		void cbElements(Fl_Widget*);

		void resetElementsList();
		
		void updateClient();
		void updateServers();

  private:
		bool m_fileSaved;

		//Menu
		Fl_Group* m_windowBar;
		Fl_Menu_Bar* m_menu;

		//Applications
		Fl_Group* m_applicationsGroup;
			//Audio: Local/Distant, INIT
			Fl_Group* m_audioApplicationGroup;
				Fl_Choice* m_audioMachine;
				Fl_Input* m_audioAdress;
				Fl_Value_Input* m_audioPort;
				Fl_Button* m_audioInit;
			//Environnement: Local/Distant, INIT
			Fl_Group* m_envApplicationGroup;
				Fl_Choice* m_envMachine;
				Fl_Input* m_envAdress;
				Fl_Value_Input* m_envPort;
				Fl_Button* m_envInit;
			//Display (list , one for each camera) with Local/Distant, INIT + mode (streamsock,multicast)
			Fl_Group *m_displayApplicationsGroup;
				Fl_Scroll *m_displayApplicationsList;
					std::vector<Fl_Choice*> m_displayMachines;
					std::vector<Fl_Input*> m_displayAdresses;
					std::vector<Fl_Value_Input*> m_displayPorts;
					std::vector<Fl_Button*> m_displayInits;

		//Elements
		Fl_Scroll* m_elementsList;
		std::vector<ElementGroup*> m_elements;

		Fl_Group* m_parametersGroup;

		Fl_Input* m_elementName;
		//Fl_Group* 
		

		//Render Widget
		RenderWidget* m_renderWidget;


		static const int MENU_HEIGHT=20;
		static const int WIDGETS_SEP=5;
		static const int APPLICATIONS_HEIGHT=160;

		static const int ELEMENTS_Y=180;
		static const int ELEMENTS_W=200;
		static const int ELEMENTS_H=400;
};

#endif /* _MAINWINDOW_H */
