/***************************************************************************
 *            RingBuffer.cpp
 *  	      Part of Drile
 *
 *  2010  Tardigrade Inc.
 *  http://tardigrade-inc.com 
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "RingBuffer.hpp"

using namespace std;

RingBuffer::RingBuffer(size_t eventSize): m_eventSize(eventSize)
{
	m_ringBuffer = jack_ringbuffer_create(10000*m_eventSize);
	jack_ringbuffer_mlock(m_ringBuffer);
}

RingBuffer::~RingBuffer()
{

}

void RingBuffer::addEvent(const Event& event)
{
	if(jack_ringbuffer_write_space(m_ringBuffer)>m_eventSize) {
		jack_ringbuffer_write(m_ringBuffer,(char* )&event, m_eventSize);
	}
}


bool RingBuffer::eventsAvailable()
{
	return (jack_ringbuffer_read_space(m_ringBuffer)>=m_eventSize);
}

void RingBuffer::getEvent(Event& event)
{
	jack_ringbuffer_read(m_ringBuffer, (char*)&event, m_eventSize);
}



